/*****************************************************************************
*  Function Name : CalcIpCheckSum                                            *
*  Organisation  : FutureSoftware Limited                                    *
*  Description   : CalcIpCheckSum calculates the checksum of temp file       *
*****************************************************************************/

/* Mathewk [26/08/05] Starts
 *
 * After migrating, the 'temp' file contains the migrated configuration
 * with the last line as CKSUM: (WITHOUT checksum value after it).
 * 
 * The full contents of this file is read to a buffer. 
 * 
 * If the size of the file is an ODD number, append a NULL character 
 * and call UtlCalcIpCheckSum () to calculate its checksum.
 * 
 * A NULL characted is first appended to 'temp' file, if its size was ODD.
 * The returned 16bit checksum is appended to the file as character.      
 * 
 * Mathewk [26/08/05] Ends                                                */


#include <stdio.h>
#include <string.h>

unsigned short int UtlCalcIpCheckSum (char *, unsigned int);

int main (int argc, char *argv[])
{
	FILE *fp;
	unsigned int 		u4Len = 0, u4TmpSize=0, u4Size = 0;
	unsigned short int 	u2Ret=0, u2NullChar = 0, msb, lsb, u2flag;
	unsigned char 		*pu1BuffCurrent, *pu1BuffStart, u1Tmp = 0;
	
        u2flag = atoi (argv[2]);

    /* Open the file in binary mode */
	fp = fopen (argv[1], "rb");
	if (fp == NULL)
	{
			printf("\n Error: Could not open temp file for reading \n");
			exit (1);
	}
	fseek (fp, 0L, SEEK_END);
	u4Size = ftell (fp);
	rewind (fp);
	
	pu1BuffStart = (unsigned char *)malloc (u4Size + sizeof (unsigned char));
	if (pu1BuffStart == NULL)
	{
			printf ("\n Error: Could not allocate memory \n");
			exit (1);
	}

	memset (pu1BuffStart, 0, u4Size);
	pu1BuffCurrent = pu1BuffStart;		

	/* Copy contents of 'temp' file to a buffer */
	while (u4Len < u4Size)
	{
			fread (pu1BuffCurrent, sizeof (unsigned char), 1, fp);
			
			pu1BuffCurrent += sizeof (unsigned char);
			u4Len          += sizeof (unsigned char);
	}
	fclose (fp);

	u4TmpSize = u4Size;
    
    /* Add a NULL character to buff if the u4Size is an ODD number */	
    if ((u4Size % 2) == 1)
    {
        strncpy (pu1BuffCurrent, &u1Tmp, sizeof (unsigned char));
        u4Size++;
        pu1BuffCurrent += sizeof (unsigned char);
    }

	u2Ret = UtlCalcIpCheckSum (pu1BuffStart, u4Size);
	
        if (u2flag == 1)
        {

            if (u2Ret == 0)
            {

                return 0;
            }
            else
            {

                return 1;
            }
        }
	
	/* The checck sum value returned by UtlCalcIpCheckSum is a 
	 * 16bit number. Separate out msb and lsb of this number
	 * and append them to 'temp' file as 'char'.             */
	
	msb = (u2Ret & 0xff00) >> 8;
	lsb = u2Ret & 0xff;

	fp = fopen (argv[1], "ab");

    	/* ALU02179950 Start */
        if(fp == NULL)
    	{
       	    printf("\r\nFile open error during cksum calc\r\n");
            exit(1);
    	}
        /* ALU02179950 End */ 

	/* Append a NULL character to 'temp' file if its
	 * size was an ODD numeber                      */
	/* if(u2Ret != 0) */ /* ALU02179950 */
	/* { */ /* ALU02179950 */
	if ((u4TmpSize % 2) != 0)
	{
			fputc (u1Tmp, fp);
	}
	fputc (msb, fp);
	fputc (lsb, fp);
	/* } */ /* ALU02179950 */
	fclose (fp);
}

		
unsigned short int
UtlCalcIpCheckSum (char *pBuf, unsigned int u4Size)
{
		
	unsigned int 		u4Sum = 0 ;
    unsigned short int	u2Tmp = 0;

    short int u2Tmp1;
    unsigned short int u2Flag = 0;
    char u1Endian[2];

    u2Tmp1 = 0xffee;
    
    u1Endian[0] = *(char *)&u2Tmp1;
    u1Endian[1] = *((char *)&u2Tmp1 + 1);
    
    if((((int)0xff & (int)u1Endian[0]) == (int)0xff) && (((int)0xff &(int)u1Endian[1]) == (int)0xee))
    {
	    printf("\nSystem is BigEndian.\n");
	    u2Flag = 1;
    }

    if((((int)0xff & (int)u1Endian[0]) == (int)0xee) && (((int)0xff &(int)u1Endian[1]) == (int)0xff))
    {
	    printf("\nSystem is LittleEndian.\n");
	    u2Flag = 2;
    }

    if (pBuf == NULL)
    {
	       printf ("\n Error: Unable to calculate checksum \n");
		   exit (1);
    }
	
    while (u4Size > 1)
    {
			
            /* Note
			 * 
			 * Assume that value in pBuf = "ab". The above typecast will
			 * return a value "ab", which is stored in u2Tmp.
			 * But in LANX, the same line returns value "ba". So to match
			 * the checksum calculated by this function with LANX checksum,
			 * interchange msb and lsb, store it in u2Tmp. So new 
			 * u2Tmp = "ba". Add this to u4Sum to calculate checksum.     */
    if(u2Flag == 2)  /*check added for checksum calc of migrated lanx db in little endian platform */
    {
            u2Tmp = *(unsigned char*)(pBuf);
            u2Tmp = u2Tmp << 8;
            u2Tmp = u2Tmp + *(unsigned char*)(pBuf+1);
	        
    }

   if(u2Flag == 1)
   {
	    u2Tmp = *(unsigned short int*)pBuf;

   }
	        
			u4Sum  	+= u2Tmp;
	        pBuf   	+= sizeof (unsigned short int);
	        u4Size 	-= sizeof (unsigned short int);
    }

    if(u4Size == 1)
    {
	u2Tmp = 0;
	*((unsigned char *) &u2Tmp) = *pBuf;
	u4Sum +=u2Tmp;


     }

    u4Sum = (u4Sum >> 16) + (u4Sum & 0xffff);
    u4Sum += (u4Sum >> 16);

	u2Tmp = ~((unsigned short int) u4Sum);

	printf("\nThe checksum is %d.\n", u2Tmp);
	return (u2Tmp);
}
