
#****************************************************************************
# Routine      : Migrationtool
# Organisation : FutureSoftware Limited,
# Description  : Migrationtool performs migration of configuration files
#                of one release to another release.
#                In this process, .conf file has to pass through intermediate
#                migrations, finally reaching the target version. Items in
#                .conf file to be changed are given in a .dat file.
#*****************************************************************************

#  Mathewk [26/08/05] Starts
#
#  A single .dat file contains migrations for all releases.
#
#  The configuration file (.conf) contains information in the form of OIDs.
#  While performing any operations on the OIDs using perl regular expressions,
#  make sure that the action affects only those lines in .conf file for which
#  an exact match is found.
#
#  For eg:
#        <RENAMED>
#              1.3.6.1.4.1.2076.90.4.1.2         1.3.6.1.4.1.2076.90.5.1.2
#        </RENAMED>
#    
#        This should not affect the lines in the following format
#
#            X.1.3.6.1.4.1.2076.90.4.1.2 
#            1.3.6.1.4.1.2076.90.4.1.2X   ( There IS NO '.' between 2 and X )
#
#        Instead, it should rename lines in the following format
#
#            1.3.6.1.4.1.2076.90.4.1.2.X   ( There IS '.' between 2 and X )
#            1.3.6.1.4.1.2076.90.4.1.2     ( This is a complete OID )
#            
#  This can be done using the following Metacharacters in regular expression.
#
#       ^  -  begining of string
#       \b -  "word" boundary
#
#
#  CLI format is as follows

#  Jayalk [08-Dec-2005]: Changed for FR-BDFh69158 (Change of CLI for Mtool).
#
#      To print Migrationtool version number:
#
#         perl Migrationtool.pl -ver
#
#
#      To migrate:
#
#         perl Migrationtool.pl -in <inputconfigfile> -out <newconfigfile>
#         -from <version-from> -to <version-to> -cfg <config-file>
#         -trc -log <logfile>
#
#  Here -trc and -ver are optional simple cmd-line arguments.
#  If mentioned in the cmd line ,correponding options can be enabled.
#
#  Mathewk [26/08/05] Ends


#! /usr/bin/perl -w

use strict;
use IO::File;
use File::Find;
use POSIX;
use Cwd;
use File::Basename;
use Getopt::Long;
#use HTTP::Date;

#Jayalk :"use warnigs" option is not supported by Solaris .
#use warnings;


my $version       = 0;
my $trace         = 0;
my $traceBW       = 0;
my $bandwidth     = "";
my $destfile      = "";  
my $inputfile     = ""; 
my $newconfigfile = ""; 
my $miglogfile    = "";  
my $cliversionfrom= "";  
my $HairpinStatus= "";  
my $cliversionto  = ""; 
#indu - added for cli command change [07/11/06][START]
my $linesininput    = 11;
my $climacmode      = 0;
my $macmode         = "";
my $interfaceBase   = "";
my $interactivemode     = "";
my $filemode            = "";
my $inputFileName       = "";
my $modeoption          = "";
my $dispmsg     = "";
my @ARGV                = "";
my @linesinput          = "";
my @resultarray         = "";
my $inputFileHandle     = '';
my @temparray           = "";
my @dispresult      = "";
my $current     = "";
my $day         = "";
my $sec         = 0;
my $min         = 0;
my $hour        = 0;
my $mday        = 0;
my $mon         = 0;
my $year        = 0;    
my $wday        = 0;
my $yday        = 0;
my $isdst       =0; 
#indu - added for cli command change [07/11/06][END]
 
#narasimrg for ECNTA/ECNTC to ECNTC migration
my $climodefrom   = "";
my $climodeto     = "";
#narasimrg
my $path          = "";
my $cmd           = "";
my $dir           = "";
my $script        = "";
my $CurrentDirFlag= 0; # This flag is set to true if the 

#R3.6 Optimised iss.conf support
my $constMaxIPIf = 256;
my $constMaxPort  = 24; 
my $confFileMode  = "CONF_FILE_OPT";

# 4.0 No Dual IP Feature
my $DualIpSet = 0;

# ALU01613898
my $cliversiontoOrig  = ""; 
# ALU01660534
my $cliversionfromOrig  = ""; 
# Mig tool is being run from the Installation Directory.


#Jayalk [08-Dec-2005] : Following BEGIN block is added to determine the
#                       startup directory of the Mtool
#                       so that the Migtool can also be executed out of 
#                       the Installation Directory .

BEGIN
{
    $dir = dirname($0); # $0- holds the invocation path name of the
    # program .   

    $script = basename($0); 


    if($dir =~ /^\//)
    {
        $path = $dir ;
    }
    elsif($dir eq "." )
    {
        $path = getcwd() ;
    }
    else
    {
        $dir =~ s/^\.\///;
        $path = getcwd() . "/$dir";
    }

    unshift(@INC,$path); # adding the Mig tool path to the INC . 

}


#Jayalk :  because prev block is not retaining the
#          values of the global variables $path ,$dir the same code  
#          was repeated here.  


{
    $dir = dirname($0); 

    if($dir =~ /^\//)
    {
        $path = $dir ;
    }
    elsif($dir eq "." )
    {
        $path = getcwd() ;
        $CurrentDirFlag = 1;
    }
    else
    {
        $dir =~ s/^\.\///;
        $path = getcwd() . "/$dir";
    }

}


# As INC contains the Installation  path for Mig tool.
# the mandatory files for Migration  (like DataChangePackage.pm...) can be 
# found, even if we run the Migtool  out of the Installation directory .


use DataChangePackage;

#  Check the syntax of the CLI
#indu - added for cli command change[07/11/06][START]
if (argcheck() != 0)
{
        print "\n\t Enter correct inputs!!!\n";
        print "\n\n Note: Migration can be done ECNTA --> ECNTA, ECNTC --> ECNTC.";
        print "\n       Migration cannot be done ECNTC --> ECNTA\n\n";
        exit 1;
}
#indu - added for cli command change[07/11/06][END]

# Create an array to store log file contents

my @logfile = ""; 
my $msg     = "";

chomp ($cliversionfrom);
chomp ($cliversionto);
chomp ($climodefrom);
chomp ($climodeto);
$msg = "\n ==========================";
print $msg;
push (@logfile, $msg);
$msg = "\n Migrationtool version 1.84\n"; # BUG 14244 #Bug 14380 #bug14439 #bug14620 #bug14763 #bug14875 #bug14927
print $msg;
push (@logfile, $msg);
$msg = " ========================== \n";
print $msg;
push (@logfile, $msg);

$msg = "\n VERSION FROM $cliversionfromOrig $climodefrom \n";

#print $msg;
push (@logfile, $msg);
$msg = "\n VERSION TO $cliversiontoOrig $climodeto\n";
#print $msg;
push (@logfile, $msg);

#  Print the status of 'trace'
$msg = "\n Trace OFF \n";

if ($trace == 1)
{
    $msg = "\n Trace ON \n";
    push(@logfile,$msg);
}

#print $msg;
push (@logfile, $msg);

# Lines between tags in the migration.dat file is stored in @list

my @list      = "";
#ALU02078598 added REPLACE_VALUE tag
my @tag       = ("RENAMED", "OBSOLETE", "DATATYPE_CHANGE",
    "ENUM_OLD", "ACCESS_CHANGE", "TABLE_INDEX_CHANGE",
    "SCALAR_TABLE_CHANGE", "OBJECT_DEPENDENCIES",
    "MULTICAST_TABLE_INDEX_CHANGE", "INSERT",
    "OBSOLETE_WITH_INDEX","INSERT_PER_VLAN", "PORT_CHANGE","OBSOLETE_WITH_VALUE","RENAME_WITH_INDEX",
    "RENAME_SCALAR","RENAME_TABLE",
    "OBSOLETE_SCALAR","OBSOLETE_TABLE",
    "ENUM_CHANGE", "REPLACE_TABLE_INDEX", "OBSOLETE_OPTIM", "REPLACE_VALUE"); 

#The following lines have been commented after adding the "INSERT_PER_VLAN" tag 
#in the above table.


# Sujithg [16/11/05]  Starts
# Added new tag for migration from 2.4 to 3.0
#push @tag, "INSERT_PER_VLAN";
# Sujithg Ends

my $tagref    = \@tag;

my @subtags   = ("TABLE_OBJECTS", "OLD_INDICES", "NEW_INDICES",
    "OBJECT_NAMES", "INDEX_VALUES");

my $subtagref = \@subtags;

# Jayalk [08-Dec-2005]
# Specifying the default-path(Installation path) for the config file ,
# if absolute path was not given in the CLI.

if ($inputfile !~ m/\//) #warning 
{
    $inputfile = $path . "/" . $inputfile ;
}

#  Open .conf and .dat file for reading

my $filehandle = IO::File -> new ($inputfile, "r") or 
die "Couldn't Open $inputfile for reading";

my $confighandle = IO::File -> new ($destfile, "r") or 
die "Couldn't Open $destfile for reading";

#  Copy the contents of the files to the arrays

my @linesinfile  =  <$filehandle>;
$filehandle   -> close();
my @newconfig    =  <$confighandle>;
$confighandle -> close();

my $i       = -1;
my $tagsize = @tag; 
my @tagsfound = "";

#  From now, these reference are used as arguments to all tag
#  process routines

my $configref           = \@newconfig;
my  $refFullDatLines = \@linesinfile;

# Find out the working directory

my $workingDir = ".";
my $issPath    = $destfile;

if ($issPath =~ m/\//)
{
    my @fields  = split (/\//, $issPath);

    # Last enty in @fields is 'iss.conf'. Remove it,
    # and the remaining entried joined with '/'
    # gives the working directory.
    pop (@fields);
    $workingDir    = join ("/", @fields);
}

my $issnvramFilePath = $workingDir."/"."issnvram.txt";
my $IGMPXmlFilePath  = $workingDir."/"."IGMPXmlfile.txt";

#Check whether .conf file can be migrated to the new version
$msg = "\n Checking whether given file can be migrated to the new version...\n";
print $msg;
push (@logfile, $msg);

# ALU01660534 
if ($cliversionfrom ne $cliversionfromOrig)
{
    #ALU01822907
    $cliversionfrom =~ m/(R|F)(\d+)(\.)(\d+$)/;
    my $streams = $1;
    my $versionsreal = $2;
    my $versionsdec = $4;
    if (($streams eq "R") && ($versionsreal == 4) && ($versionsdec == 3))
    {
       $msg = "INFO: Lanx has a common build for ISR43 and ISR44. Hence migration will take place from R4.3";
       print $msg;
       push (@logfile, $msg);
    }
    else
    { 
       $msg = "\n WARNING !! FROM VERSION greater than $cliversionfrom !!";
       print $msg;
       push (@logfile, $msg);
       $msg = "\n Default value $cliversionfrom chosen\n";
       print $msg;
       push (@logfile, $msg);
    }
}

# ALU01613898
if ($cliversionto ne $cliversiontoOrig)
{
    #ALU01822907 
    $cliversionto =~ m/(R|F)(\d+)(\.)(\d+$)/;
    my $streams = $1;
    my $versionsreal = $2;
    my $versionsdec = $4;
    if (($streams eq "R") && ($versionsreal == 4) && ($versionsdec == 3))
    {
       $msg = "INFO: Lanx has a common build for ISR43 and ISR44. Hence migration will take place to R4.3";
       print $msg;
       push (@logfile, $msg);
    }
    else
    { 
       $msg = "\n WARNING !! TO VERSION greater than $cliversionto !!";
       print $msg;
       push (@logfile, $msg);
       $msg = "\n Default value $cliversionto chosen\n";
       print $msg;
       push (@logfile, $msg);
    }
}

# ALU01909609 - R5.0 Support - Removed the check so that the validation happens for all versions
# ALU01660534 start
 my $validatedResult = validateMigrationHigherRel($cliversionfrom, $cliversionto, $cliversionfromOrig, $cliversiontoOrig);
 
    if ($validatedResult eq 0)
    { 
        my $systemCommand = "cp ".$destfile." ".$newconfigfile;
        system ($systemCommand);
        exitMigTool (0);
    }
    elsif ($validatedResult eq -1)
    {
        exitMigTool (1);
    }
# ALU01660534 end

#<vikash> here we can take care of version 2.4.14 
#the logic is when sombody says 2.4.14 then change that value to 2.4 now its the same stuff......
#</vikash>

my $validatedResult = validateMigration ($refFullDatLines, $cliversionfrom, 
    $cliversionto);
if ($validatedResult eq -1)
{
    exitMigTool (1);
}

# Check whether the input .conf file is corrupted or not.
# Pass the file to CalcIpCheckSum. If the file is 
# corrupted, CalcIpCheckSum returns 0.


# Specifying the absolute path for the file: CalcIpCheckSum.

my $is_big_endian     = 0;
my $is_little_endian  = 0;

$is_big_endian   = unpack("h*", pack("s", 1)) =~ /01/;
$is_little_endian = unpack("h*", pack("s", 1)) =~ /^1/;
my $solaris = "solaris";
 
if($^O eq $solaris)
{
if($is_big_endian eq 1)
{
             print "\n Big Endian =";
             print $is_big_endian;
	     $cmd = $path . "/./CalcIpCheckSum_BE" ;
}
 
if($is_little_endian eq 1)
{
            print "\n Little Endian =";
            print $is_little_endian ;
	    $cmd = $path . "/./CalcIpCheckSum_LE" ;
}
}
else
{
	if($is_big_endian eq 1)
	{
             print "\n Big Endian =";
             print $is_big_endian;
	     $cmd = $path . "/./CalcIpCheckSum_BE_linux" ;
	}
 
	if($is_little_endian eq 1)
	{
            print "\n Little Endian =";
            print $is_little_endian ;
	    $cmd = $path . "/./CalcIpCheckSum_LE_linux" ;
	}
}


#$cmd = $path . "/./CalcIpCheckSum" ;

my $retSystem = system ("$cmd $destfile 1");

#indu - added for CalcIpCheckSum permission check [18/04/07][START]
if($retSystem ne 0)
{
    if ($retSystem eq 1)
    {
        $msg = "\n Error: $destfile is Corrupted \n";
        print $msg;
        push (@logfile, $msg);
        exitMigTool (1);
    }
    else
    {
        $msg = "\n Permission to CalcIpCheckSum file should be executable or conf file corruped\n";
        print $msg;
        push (@logfile, $msg);
        exitMigTool (1);
    } 
}

#indu - added for CalcIpCheckSum permission check [18/04/07][END] 

my $versionto   = -1;
my $versionfrom = $cliversionfrom;
my $FromMode = $climodefrom;
my $ToMode = $climodeto;
$msg = "\n Migrating.... Please wait: \n\n";

print $msg;
push (@logfile, $msg);
my $ANSI =0;

# changes made for multiple stream modification - ANSI ETSI baseline
my $Checkchar = substr($cliversionto ,0,1);
if($Checkchar eq "F")
{
   $ANSI =1;
}


# ANTmt42349 - start
my $relVerfrom = $cliversionfrom;
my $relVerto = $cliversionto;
my $relVerInConf = $newconfig[0];
chomp ($relVerInConf);
if($relVerfrom ge "R4.0")
{
    substr($relVerInConf,0,1) = 'R';
    
        if ( $relVerInConf ne $relVerfrom)
    {
        $msg = "\n Error : input conf file version number and version number provided by user are not matching \n";
        print $msg;
        push(@logfile,$msg);
        exitMigTool(1);
    }
}
else
{
my $check = substr($relVerfrom ,0,1);
   if( $ANSI eq 1 && $check eq "F" )
   {

        substr($relVerInConf,0,1) = 'F';
        if ( $relVerInConf ne $relVerfrom)
    {
        $msg = "\n Error : input conf file version number and version number provided by user are not matching \n";
        print $msg;
        push(@logfile,$msg);
        exitMigTool(1); 
    }
    
    }
    else
    {
        if (($relVerInConf eq "A4.0") || ($relVerInConf eq "A4.1") || ($relVerInConf eq "N4.1"))
        {
            $msg = "\n Error : input conf file version number and version number provided by user are not matching \n";
            print $msg;
            push(@logfile,$msg);
            exitMigTool(1); 
        }
    }
}
#ANTmt42349 - end

# 4.0 No Dual Ip Support

if($DualIpSet == 1)
{
   $configref = FunctionFor4_0_No_Dual_Ip($configref); 
}

# Mathewk added for auto-negotiation Starts

if ($cliversionfrom =~ "R1.0")
{

    my $negOID       = "1.3.6.1.4.1.2076.81.2.2.1.2";
    my $duplexOID    = "1.3.6.1.4.1.2076.81.2.2.1.3"; 
    my @removeOID    = "";
    my $i            = 0;
    my @tempLines    = @{$configref};
    my $refRemoveOID = 0; 

    for (@tempLines)
    {
        my @tempFields = split (/\s+/, $tempLines[$i++]);
        if ($tempFields[0] =~ m/^$negOID\b/)
        {

            if ($tempFields[2] eq 1)
            {

                my @dotFields = split (/$negOID/, $tempFields[0]);
                my $portNo    = $dotFields[1];
                my $delString = $duplexOID.$portNo."\n";

                push (@removeOID, $delString);
            }
        }
    } 

    $refRemoveOID = \@removeOID;
    $configref = obsolete ($refRemoveOID, $configref);
}
#R3.6 Optimised iss.conf support
#if ($cliversionto =~ "R3.6" )
# for using variable $confFileMode in future, keep appending the from_version
# in the elsif case below
{
    if($cliversionfrom =~ "R3.5")
    {
        $confFileMode = "CONF_FILE_OPT";
    }
    else
    {
        $confFileMode = "CONF_FILE_UNOPT";
    }
    if(($interactivemode)||($modeoption eq "i"))
    {   
    
    print "\n\t Enter the Interface Base option\n";
    print "\n\t Enter a value between 256 - 448 (Default = 256)\n";
    $interfaceBase = <STDIN>; 
    chop $interfaceBase;
    if(($interfaceBase < 256) || ($interfaceBase > 448))
    {
        print "\n InterfaceBase should range between 256-448 \n";
        exitMigTool(1); 
    }   
    $dispmsg = "\n\t The InterfaceBase is :$interfaceBase\n";
    push (@dispresult,$dispmsg);
    push (@logfile,$dispmsg);
    }

    if(($filemode) || ($modeoption eq "f"))
    {
    if(($interfaceBase < 256) || ($interfaceBase > 448))
    {
        print "\n InterfaceBase should range between 256-448 \n";
        exitMigTool(1);
    }
    }
    $constMaxIPIf = $interfaceBase;
}
# Mathewk Auto Negotiation Ends
my $ECNTAtoCConverted = 0;

while ($cliversionto !~ m/$versionto/)
{
    my $linesref;
    my $sameVersion = 0;
    if(($cliversionto eq $cliversionfrom) && ($climodefrom ne $climodeto))
    {
        $linesref = getIntermediateDat($refFullDatLines, "ECNTA");
    }
    else
    {

        if(($cliversionto ne $cliversionfrom) && ($climodefrom ne $climodeto)&&($ECNTAtoCConverted eq 1))
        {
            $linesref = getIntermediateDat($refFullDatLines, "ECNTA");
        }
        else
        {
            if($versionfrom eq "R4.0")
            {
                my $Char = substr ($cliversionto, 0,1);
                if($Char eq "F")
                {
                    $versionfrom = "F4.0";
                }
            }
            $linesref = getIntermediateDat ($refFullDatLines, $versionfrom);
        }
    }

    if ($linesref eq -1)
    {
        $msg = "\n Error: Could not get migration data \n";
        print $msg;
        push (@logfile, $msg);
        exitMigTool (1);
    }

    #  Copy all tags found in .dat file except subtags
    #  in to 'tagsfound' array
    my @tagsfound    = gettags ($linesref,$subtagref);
    my $tagreference = \@tagsfound;
    my $numberoftags = @tagsfound;

    my $tagcount = 0;

    #  Add some blank lines at the end of the lines.. 
    #  Just for fixing small problem.
#    for (1 .. 50005)
#    {
#        push (@{$linesref}, "\n");
#    }

    #  Perfrom migrations specific for R2.1
    if ($versionto =~ "R2.1") 
    {
        $configref = FunctionForR2_1 ($configref);
    }
    #  Perfrom migrations specific for R2.2
    if ($versionto =~ "R2.2") 
    {
        $configref = FunctionForR2_2 ($configref);
    }

    for (@{$linesref})
    {
        #  Copy the contents between the tag '$tagsfound[$tagcount]'
        #  in .dat file to array '@list'      

        @list       = parse ($linesref, $tagsfound[$tagcount]);
        my $listref = \@list;

        #  Get the position of this tag in the array '@tag'
        $i = wheretag ($tagref, $tagsfound[$tagcount]);
        $tagcount++;

        #  Now '@list' have the lines between start and end of the 
        #  tag '$tagsfound[$tagcount]'

        if ($i == 0)
        {
            #  RENAMED Tag
            $configref = renamed ($listref, $configref);
        }
        else
        {
            if ($i == 1)
            {
                #  OBSOLETE Tag
                $configref = obsolete ($listref, $configref);
            }
            else
            {
            if ($i == 2)
            {
                #  DATATYPE_CHANGE Tag
                $configref = datatype_change ($listref, $configref);
            }
            else
            {
            if ($i == 3)
            {
                #  ENUM_CHANGE_OLD Tag
                $configref = enum_change ($listref, $configref);
            }
            else
            {
            if ($i == 4)
            {
                #  ACCESS_CHANGE Tag
                $configref = access_change ($listref, $configref);
            }
            else
            {
            if ($i == 5)
            {
                #  TABLE_INDEX_CHANGE Tag
                $configref = table_index_change ($listref,
                $configref, $tagsfound[$tagcount-1]);
            }
            else
            {
            if ($i == 6)
            {
                #  SCALAR_TABLE_CHANGE Tag
                $configref = ScalarToTable ($listref, $configref); 
            }
            else
            {
            if ($i == 7)
            {
                #  OBJECT_DEPENDENCIES Tag
                $configref = object_dependencies ($listref, $configref);
            }
            else
            {
            if ($i == 8)
            {
                #  MULTICAST_TABLE_INDEX_CHANGE tag
                $configref = table_index_change ($listref,
                $configref, $tagsfound[$tagcount-1]);
            }
            else
            {
            if ($i == 9)
            {
                #OBJECT INSERT tag
                $configref = object_insert ($listref, $configref);
            }
            else
            {
            if ($i == 10)
            {
                #OBSOLETE_WITH_INDEX Tag
                $configref = obsolete_with_index ($listref, $configref);
            }
            #Sujithg [16/11/05] Starts
            else
            {
            if ($i == 11)
            {
                #OBSOLETE_PER_VLAN Tag
                $configref = object_insert_per_vlan ($listref, $configref);
            }
            #sujithg ends

            #indu-preethu [29/09/06] start
            else
            {
            if ($i == 12)
            {
            #port_change tag
                $configref = object_port_change ($listref, $configref);
            }
            else
            {
            if ($i == 13)
            {
            #object_value_delete
                $configref = obsolete_with_value ($listref, $configref);    
            
            }
            else
            {
            if ($i == 14)
            {
            #rename_with_index tag
                $configref = rename_with_index ($listref, $configref);  
            }
            else
            {
            if ($i == 15)
            {
            #rename_scalar tag
                $configref = rename_scalar ($listref, $configref);  
            }
            else
            {
            if ($i == 16)
            {
            #rename_table tag
                $configref = rename_table ($listref, $configref);   
            }
            else
            {
            if ($i == 17)
            {
            #obsolete_scalar tag
                $configref = obsolete_scalar ($listref, $configref);    
            }
            else
            {
            if ($i == 18)
            {
            #obsolete_table tag
                $configref = obsolete_table ($listref, $configref); 
            }
            else
            {
            if ($i == 19)
            {
            #enum_change optimised tag
                $configref = enum_change_optim ($listref, $configref);  
            } 
            else
            {
            if ($i == 20)
            {
            #replace_table_index optimised tag
                $configref = replace_table_index ($listref, $configref);    
            } 
            if($i == 21)
            {
                $configref = obsolete_optim($listref, $configref);
            }  
            #ALU02078598 -S
	    if($i == 22)
	    {
	        $configref = replace_value($listref, $configref);
	    }   
             #ALU02078598-E   
            } 
                        }
                        } 
                        } 
                        } 
                        } 
                        } 
                        } 
            }
            }
            }
            }
            }
            }
            }
            }
            }
            }
            } 
            }

    } #  End of the for loop;


    # Perform migration specific for R2.1 
    if ($versionto =~ "R2.1")
    {

        #Jayalk : because of enabling 'strict refs' subroutine calls
        #         by refernce are modiifed .

        my  $PackageName = "DataChangePackage";
        my $str = $PackageName. "::" ."Qosupdates";
        my $funref = \&$str;
        $configref = $funref->($configref); 
    } 
    # Perform migration specific for R2 when migrating from R1.0 

    if (($versionto =~ "R2") and ($versionfrom =~ "R1.0"))
    {
        $configref = vlanmodeaddition ($configref);         
    }

    # Perform migration specific for R2.4

    if ($versionto =~ "R2.4")
    {
        $configref = FunctionForR2_4 ($configref)
    }
    if ($versionto =~ "R2.4.15")
    {
        # do nothing;
    }
    if ($versionto =~ "R2.4.16")
    {
        # do nothing;
    }
        #indu-preethu [05.10.06] [start]
            
#This function is called during the migration of R2.4 ECNT-A/C to R2.5 ECNT-C.
    if((($cliversionfrom eq "R2.0") ||($cliversionfrom eq "R2.1") || 
    ($cliversionfrom eq "R2.2") || ($cliversionfrom eq "R2.3") || 
    ($cliversionfrom eq "R2.4")) && 
    ($ToMode eq "ECNTC") && ($versionto eq "R2.5") && ($FromMode eq "ECNTA"))
    {
              $configref = cpe_vlans_insert ($configref);
    }

    # Perform migration specific for R2.5

    if ($versionto =~ "R2.5")
    {
        #<vikash> here we have to put the condition if versionto is 3.0 or 3.1 , this function should not be called </vikash>


        if (!($cliversionto eq "R3.0" || $cliversionto eq "R3.1"))
        {


            FunctionForR2_5 ($configref)
        }
    }

    #Sujithg [16/11/05] Starts
    # Perform migration specific for R3.0
    if ($versionto =~ "R3.0")
    {
        if(!($cliversionfrom eq "R2.5"))
        {   

            FunctionForR3_0 ($configref);
        }
    }
    #Sujith Ends

    #Sujithg [16/11/05] Starts
    # Perform migration specific for R3.3
    if ($versionto =~ "R3.3")
    {
        $configref = FunctionForR3_3 ($configref);
    }
    #Sujith Ends

    if ($versionto =~ "R3.3")
    {
        $configref = FunctionForR3_3ph2 ($configref);
    #indu - for issSwitchMacMode[22/11/06][START]
        $configref = FunctionForR3_3macmode ($configref);
    #indu - for issSwitchMacMode[22/11/06][END]
    }
    #<vikash>
    if ($versionto =~ "R3.1")
    {

        $configref = FunctionFor3_1($configref);
    }
    #</vikash>


    #<Start:selvam M:20-04-07:changed for arp proxy table>
    if ($versionto =~ "R3.4")
    {

        $configref = FunctionFor3_4($configref);
        $configref = EgressRateChangeforR3_7($configref);# Fix for ANTmt45471
    }
        # ANTmt40020 START
        if(($ToMode eq "ECNTC") && ($versionto =~ "R3.5"))
        {
                 $configref = RipIfAndMD5KeyCheck($configref);
        }
        # ANTmt40020 END
    #</end:selvam>
#Karthikeyan.T - for Vlan 4093 Interface ID rename[14/06/07][START]
    if ($versionto =~ "R3.5" and $ToMode eq "ECNTC")
    {
        $configref = FunctionFor3_5($configref);
    }
#Karthikeyan.T - for Vlan 4093 Interface ID rename[14/06/07][END]
        if ($versionto =~ "R3.6")
        {
               #$configref = FunctionFor3_6($configref); 
        }
    if(($cliversionto eq $cliversionfrom) && ($climodefrom ne $climodeto))
    {
        $versionto = -1;
        last;
    }
    else
    { 
        if(($cliversionto ne $cliversionfrom) && ($climodefrom ne $climodeto) && ($ECNTAtoCConverted eq 1))
        {
            $versionto = -1;
            last;
        }
        else
        {
            if(($climodefrom ne $climodeto)&&($cliversionto =~ m/$versionto/))
            {
                $ECNTAtoCConverted = 1;
                $versionto = -1;
            }

            $versionfrom = $versionto;

            if($versionfrom eq "R4.0")
            {
                my $Char = substr ($cliversionto, 0,1);
                if($Char eq "F")
                {
                    $versionfrom = "F4.0";
                }
            }
        }
    }

} # End of the while loop

if ( ($climodefrom eq "ECNTA") && ($climodeto eq "ECNTC") && ( ($cliversionfrom gt "R3.5") || ($ANSI eq 1) ) )
{
        $configref = RipIfAndMD5KeyCheck($configref);
}
#ANTmt42349 - start
if (($cliversionto ge "R4.0") && ($cliversionfrom ne $cliversionto))
{
     #changed for multiple stream baseline
    if($cliversionfrom ge "R4.0") 
    {
        $configref = RemoveVersionAtBeg($configref, $cliversionto);
    }
    $configref = AddVersionAtBeg($configref, $cliversionto);
}
    # changed for the Multiple stream baseline - ANSI and ETSI streams
if( $ANSI eq 1)
{
    if( ($cliversionto ge "F4.1") && ($cliversionfrom ne $cliversionto ))
    {
        if($cliversionfrom ge "F4.1") 
        {
         $configref = RemoveVersionAtBeg($configref, $cliversionto);
        }
          $configref = AddVersionForFGNAtBeg($configref, $cliversionto);
         
    }
}
if ($cliversionto eq "R4.3")
{
        $configref = FunctionForR4_3($configref);
}
#ANTmt42349 - end
#9526 
$configref = FunctionFor_unnumbered_int($configref);

# Jayalk : The following  code is common for all migrations inorder  
# to move the position of IssPortCtrlTable  to just before the 
# ffFastForwardingEnable .  

{
    my $ObjectTobeMoved   = "1.3.6.1.4.1.2076.81.2.2.1."; # IssPortCtrlEntry
    my $InsertBefore      = "1.3.6.1.4.1.2076.27.2.1.0" ; # ffFastForwardingEnable 
    $configref  = Alter_Object_Position($ObjectTobeMoved, $InsertBefore ,$configref);

}
# Ravi16921  changes for Rip Global table and RRD table
#if (($versionto =~ "R3.5")||($versionto=~ "R3.6")||($versionto=~ "R3.7"))
#ANTmt51710

if(($versionto ge "R3.5") || ($ANSI eq 1) )
{
    my $ObjectTobeMoved_1   = "1.3.6.1.4.1.2076.151.1."; # fsMIRipRRDTable
    my $InsertBefore_1      = "1.3.6.1.4.1.2076.151.2" ; # fsMIRip2GeneralTable 
    $configref  = Alter_Object_Position($ObjectTobeMoved_1, $InsertBefore_1 ,$configref);
        my $ObjectTobeMoved_1   = "1.3.6.1.4.1.2076.151.1.1.1."; # fsMIRipRRDTable
        my $InsertBefore_1      = "1.3.6.1.4.1.2076.152" ; # fsMIStdRip2IfConfTable
        $configref  = Alter_Object_Position($ObjectTobeMoved_1, $InsertBefore_1 ,$configref);


}
$msg = "\n Checksum Calculation is under progress.......Please Wait : \n";
print $msg;
push (@logfile, $msg);
#  Remove the previous checksum value 
#  This is required because Checksum is Calculated Excluding 
#  the pevious Checksum. 
for (@{$configref})
{
#   /*[ANTms50738][Start] [While restoring code will skip unwanted Ctrl-M character in
#   the iss.conf file]*/
#   s/^M//;
    s/CKSUM:.*/CKSUM:/g;
} # End of for ,  

#  Store the migrated file Excluding Checksum 
#  in a file named 'temp'.
my $fileHandletemp = IO::File -> new ("temp", "w") or 
die " Could not create 'temp' file \n";
print $fileHandletemp @{$configref};

$fileHandletemp -> close();

#  The temp file created now is given as the
#  input to CalcIpCheckSum to calculate checksum.
#  CalcIpCheckSum is an exe file created by
#  compiling CalcIpCheckSum.c file.


my $retsystem = system ("$cmd temp 0");

#if ($retSystem eq 0)
#{
#   $msg = "\n Error: $destfile is Corrupted \n";
#   print $msg;
#   push (@logfile, $msg);
#   exitMigTool (1);
#}

#  After this system call, the migrated file with checksum
#  is in 'temp'. Rename 'temp' to the name given
#  by the user.

my $systemCommand = "mv temp ".$newconfigfile;
system ($systemCommand);

exitMigTool (0);


#*****************************************************************************
# Name :- FunctionForR2_1
# Descrion :- This function is for R2_1 only .
# Input :- iss.conf file. 
# Output :- None. 
# ****************************************************************************
use DataChangePackage;
sub FunctionForR2_1
{
    my ($refConfigFile)=@_;

    #Jayalk : subroutine calls by reference  are modified as
    #         'strict refs' is enabled.

    my $PackageName = "DataChangePackage";
    my $str = $PackageName. "::" ."TableMigration";
    my $funref = \&$str;
    my $returnVal = $funref->($refConfigFile,\$path); 

    $str = $PackageName . "::" . "DhcpRelayTable" ;
    $funref  = \&$str;
    $returnVal = $funref->($returnVal,\$path); 
    $str = $PackageName . "::" . "Qosupdates" ;
    $funref = \&$str;
    my $qosreturn = $funref->($returnVal); 

    #Jayalk [15/3/2006] : The following routine is called to move the
    # position of ifMainAdminStatus to after ifMultiNetipRowStatus

    # Jayalk : 17/08/2006 

    my $ObjectTobeMoved   = "1.3.6.1.4.1.2076.27.1.4.1.4."; # ifAdminStatus
    my $InsertBefore      = "1.3.6.1.4.1.2076.92.1.1.0" ;  # radiusRelaying 

    my $returnRef = Alter_Object_Position($ObjectTobeMoved, $InsertBefore ,$qosreturn);

    #Jayalk [27-Jul-2006]:To Bring the PortCtrlTable Change into  R21 Migration  
    my $ConfigRef  = IssPortCtrlTableForR2_1($returnRef);

    #Jayalk [10Apr2007][ANTms40388]  
    $ConfigRef  = IssEgressRateChangeforR2_1($ConfigRef);

    return $ConfigRef;

}


#*****************************************************************************
#  Name     :- FunctionForR2_2
#  Descrion :- This function is for R2_2_* only .
#  Input    :- iss.conf file Array. 
#  Output   :- None. 
#*****************************************************************************
sub FunctionForR2_2
{

    my ($refConfigFile) = @_;
    my @issnvram;
    my $configref = $refConfigFile;

# BEGIN1
    # New requirement for ipifTable
    my $obsoleteObjName = "1.3.6.1.4.1.2076.81.1.4";
    my $newObjName      = "1.3.6.1.4.1.2076.27.1.5.1.1";

    $configref = ipifTableHandling ($obsoleteObjName, $newObjName,
        $configref);
#END1

# BEGIN2
    # New requirement
    # Remove the entries dot1qVlanStaticRowStatus which has the indexes 
    # internal Vlan and external Vlan. Use 'issnvram.txt' to get them. 

    my $nvramRef         = nvramContent ();
    @issnvram            = @{$nvramRef};
    my $intVlanName      = "INTERNAL_VLANID";
    my $extVlanName      = "EXTERNAL_VLANID";
    my $extendedVlanName = "EXTENDED_VLANID";
    my $intVlanId        = getval ($nvramRef, $intVlanName, 0);
    chomp ($intVlanId);
    my $extVlanId        = getval ($nvramRef, $extVlanName, 0);
    chomp ($extVlanId);
    my $extendedVlanId   = getval ($nvramRef, $extendedVlanName, 0);
    chomp ($extendedVlanId);

    if ($trace == 1)
    {
        $msg = "\n INTERNAL_VLANID = $intVlanId \n";
        push (@logfile, $msg);
        push (@dispresult,$msg);
        $msg = "\n EXTERNAL_VLANID = $extVlanId \n";
        push (@logfile, $msg);
        push (@dispresult,$msg);
    }

    # Now obsolete dot1qVlanStaticRowStatus which having 
    # indexes $intVlanId and $extVlanId 

    my @objectNameList="";
    $objectNameList[0] = "1.3.6.1.2.1.17.7.1.4.3.1.5.".$intVlanId;
    $objectNameList[1] = "1.3.6.1.2.1.17.7.1.4.3.1.5.".$extVlanId;

    $configref = obsolete_with_index (\@objectNameList, $configref); 

# END2

#BEGIN3
    # This is for the new requirement "search for dot1qPvid and add newindex"
    # (for the index search for EXTERNAL_VLAN PORTS PVID in issnvram.txt)
    # with datatype 66 value = (search for EXTERNAL_VLANID in issnvram.txt)

    my $extVlanPPvid    = "EXTERNAL_VLAN UNTAGGED PORTS";
    my $extVlanPPvidVal = getval ($nvramRef, $extVlanPPvid, 0);
    chomp($extVlanPPvidVal);

    # This portPvid's may be more than one seperated by ','.
    # So split by Comma.

    my @portPvidList = split(/,/,$extVlanPPvidVal);
    if ($trace == 1)
    {
        $msg = "\n EXTERNAL_VLAN PORTS PVID \n";
        push (@logfile, $msg);
        push (@logfile, @portPvidList);
    }
    my $size         = @portPvidList;
    my $insertBefore = "1.3.6.1.2.1.17.7.1.4.5.1.1";
    my @lineToInsert="";
    for (my $count =0; $count <$size; $count++)
    {
        $lineToInsert[$count] = "$insertBefore.".$portPvidList[$count].
        "\t"."66"."\t".$extVlanId."\n";
    }

    $configref = InsertLinesRoutine ($insertBefore, \@lineToInsert,
        $configref,$size);

#END3

#BEGIN4
    # New Requirement
    # For this dot1qFutureVlanConfStatus having internal Vlan as index
    # (search for INTERNAL_VLANID in issnvram.txt) Obsolete this Row   

    $objectNameList[0] = "1.3.6.1.4.1.2076.65.1.11.1.2.".$intVlanId;

    $configref = obsolete_with_index (\@objectNameList, $configref); 

#END4 

#BEGIN5
    my @objectNames="";
    $objectNames[0] = "1.3.6.1.4.1.2076.74.1.4";
    my $objectVal   = getval ($configref, $objectNames[0], 1);
    chomp ($objectVal);
    if ($objectVal == 0)
    {
        $configref = obsolete (\@objectNameList, $configref);
    }   
#END5

#BEGIN6
    # issDefaultGateway to be added with type  64 and value 
    # (search for DEFAUT_GATEWAY in issnvram.txt) and add this new object 
    # Before issPortCtrlMode  

    my $defGateWay = "DEFAUT_GATEWAY";
    my @defGateWayVal="";

    $defGateWayVal[0] = getval ($nvramRef, $defGateWay, 0);
    chomp ($defGateWayVal[0]);
    my @addGateWay="";
    $addGateWay[0] = "1.3.6.1.4.1.2076.81.1.39.0"."\t"."64"."\t".
    $defGateWayVal[0]."\n";
    $insertBefore   = "1.3.6.1.4.1.2076.81.2.2.1.2";
    $size =1;

    $configref = InsertLinesRoutine ($insertBefore, \@addGateWay,
        $configref, $size);
#END6

#BEGIN7
    # issExternalVlanId has to be added at the begining with 
    # datatype 66 & value taken from issnvram.txt  "EXTERNAL_VLANID"

    my @addExternalVlanId="";
    $size =1;
    $addExternalVlanId[1] = "1.3.6.1.4.1.2076.81.1.34.0"."\t".
    "66"."\t".$extVlanId."\n";

    if ($trace == 1)
    {
        $msg = "\n ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n extvlanId =
        $extVlanId \n ";
        push (@logfile, $msg);
        $msg = $addExternalVlanId[0];
        push (@logfile, $msg);
    }
    $configref = InsertLinesRoutine ("TOP", \@addExternalVlanId,
        $configref, $size);
#END7

#BEGIN8
    # Add fsSnmpCommunityTableAccess.0  with data Type 64 and value = 
    # (Search for COMMUNITY_TABLE_PERM in issnvram.txt);and add
    # at the END of the config file before checksum,.,

    my $communityTablePerm = "COMMUNITY_TABLE_PERM";
    $communityTablePerm    = getval ($nvramRef, $communityTablePerm, 0);
    chomp ($communityTablePerm);

    my @snmpCommTableAcess="";
    $snmpCommTableAcess[0] = "1.3.6.1.4.1.2076.50.5.0"."\t"."64"."\t".
    $communityTablePerm."\n";
    $size =1;
    $configref = InsertLinesRoutine ("END", \@snmpCommTableAcess,
        $configref, $size);
#END8

#BEGIN9
    # Add dot1qVlanStaticName.externalvlanid with datatype 4 value 0  
    # Add it just above  dot1qVlanStaticName
    my @addVlanStaticName="";
    $addVlanStaticName[0] = "1.3.6.1.2.1.17.7.1.4.3.1.1.".$extVlanId.
    "\t"." 4"."\t"."0"."\n";
    $size =1;
    $configref = InsertLinesRoutine ("1.3.6.1.2.1.17.7.1.4.3.1.1",
        \@addVlanStaticName, $configref,$size);
#END9

#BEGIN10
    # Add dot1qVlanStaticEgressPorts.externalvlanid with datatype 4 
    # value 3 with list from nvram 
    # EXTERNAL_VLAN EGRESS PORTS (these will be separated by Commaa ",", 
    # we have to split.
    # EXTERNAL_VLAN PORTS PVID Calculate its Equivalent
    # 3 Chars BY CONVERTING THE Decimal Values to the CHAR 
    # add it just above  dot1qVlanStaticEgressPorts
    my $egressPorts = getval ($nvramRef, "EXTERNAL_VLAN EGRESS PORTS", 0);
    chomp ($egressPorts);

    # @portEgressPorts contains the EXTERNAL_VLAN EGRESS PORTS list.
    my @portEgressPorts = split (/,/, $egressPorts);
    $size            = @portEgressPorts;  # Maximum size is 24 (ports)

    if ($trace == 1)
    {
        $msg = "\n At BEGIN10 \n ";
        push (@logfile, $msg);
        push (@logfile, @portEgressPorts);
    }

    my $char0 = 0;
    my $char1 = 0;
    my $char2 = 0;
    my $newVal='';

    for (my $i=0; $i< $size; $i++)
    {
        if ($trace == 1)
        {
            $msg = " Before If condition statement";
            push (@logfile, $msg);
        }

        if (($portEgressPorts[$i] > 0) and ($portEgressPorts[$i] < 9))
        {
            if ($trace == 1 )
            {  
                push (@logfile, "\n CASE 1 .. 8 \n ");
            }
            $newVal = 8 - $portEgressPorts[$i];
            $char2  = $char2 + 2 ** $newVal;
        }
        else
        {
            if (($portEgressPorts[$i] > 8) and ($portEgressPorts[$i] < 17))
            {
                if ($trace == 1)
                {
                    push (@logfile, "\n CASE 9 .. 16 \n ");
                } 
                $newVal = 16 - $portEgressPorts[$i];
                $char1  = $char1 + 2 ** $newVal;
            } # End of if (($portEgressPorts[$i] > 8) 
            else
            {
                if (($portEgressPorts[$i] > 15) and ($portEgressPorts[$i] < 25))
                {
                    if ($trace == 1) 
                    {
                        push (@logfile, "\n CASE 17 .. 24 \n ");
                    }
                    $newVal = 24 - $portEgressPorts[$i];
                    $char0  = $char0 + 2 ** $newVal;
                } # End of if (($portEgressPorts[$i] > 15)
            } # End of else
        } # End of else of if (($portEgressPorts[$i] > 0) 
    } # End of for

    # Get the char value of the specified decimal value.
    if ($trace == 1)
    {
        $msg = $char2."\t".$char1."\t".$char0."\n";
        push (@logfile, $msg);
    }
    $char0 = chr ($char0);
    $char1 = chr ($char1);
    $char2 = chr ($char2);

    my @addLine="";
    $addLine[0] = "1.3.6.1.2.1.17.7.1.4.3.1.2.".$extVlanId."\t "."4"."\t      "."3"."\t".$char2.$char1.$char0."\n";
    $size =1;
    $configref = InsertLinesRoutine("1.3.6.1.2.1.17.7.1.4.3.1.2",
        \@addLine, $configref, $size);
#END10

#BEGIN11
    # Add dot1qVlanStaticEgressPorts.externalvlanid with datatype 4 
    # value 3 with list from nvram 
    # EXTERNAL_VLAN UNTAGGED PORTS (these will be separated by Commaa ",", 
    # we have to split.
    # EXTERNAL_VLAN PORTS PVID Calculate its Equivalent 
    # 3 Chars BY CONVERTING THE Decimal values to the CHAR 
    # Add it just above  dot1qVlanStaticEgressPorts

    my $egressPorts1 = getval ($nvramRef, "EXTERNAL_VLAN UNTAGGED PORTS", 0);
    chomp($egressPorts1);
    # @portEgressPorts contains the EXTERNAL_VLAN EGRESS PORTS list.
    my @portEgressPorts1 = split (/,/, $egressPorts1);
    $size             = @portEgressPorts1;  # Maximum size is 24 (ports)

    if ($trace == 1)
    {
        $msg = "\n At BEGIN11 \n ";
        push (@logfile, $msg);
        push (@logfile, @portEgressPorts1);
    }
    my $char3 = 0;
    my $char4 = 0;
    my $char5 = 0;

    for (my $i=0; $i< $size; $i++)
    {

        if ($trace == 1)
        {
            $msg = "\n Before If condition statement \n";
            push (@logfile, $msg);
        } 

        if( ($portEgressPorts1[$i] > 0) and ($portEgressPorts1[$i] < 9))
        {
            if ($trace == 1)
            {
                push (@logfile, "\n CASE 1 .. 8 \n ");
            }
            $newVal = 8 - $portEgressPorts1[$i];
            $char5  = $char5 + 2 ** $newVal;
        }
        else
        {
            if (($portEgressPorts1[$i] > 8) and ($portEgressPorts1[$i] < 17))
            {
                if ($trace == 1)
                {
                    push (@logfile, "\n CASE 9 .. 16 \n ");
                }
                $newVal = 16 - $portEgressPorts1[$i];
                $char4  = $char4 + 2 ** $newVal;
            }
            else
            {
                if (($portEgressPorts1[$i] > 15) and ($portEgressPorts1[$i] < 25))
                {
                    if ($trace == 1)
                    {
                        push (@logfile, "\n CASE 17 .. 24 \n ");
                    }
                    $newVal = 24 - $portEgressPorts1[$i];
                    $char3  = $char3 + 2 ** $newVal;
                } # end of if(($portEgressPorts1[$i] > 15))
            }
        }
    } # end of for i

    # Get the char value of the specified decimal value.
    if ($trace == 1)
    {
        $msg = $char5."\t".$char4."\t".$char3."\n";
        push (@logfile, $msg);
    }
    $char3 = chr ($char3);
    $char4 = chr ($char4);
    $char5 = chr ($char5);

    $addLine[0] = "1.3.6.1.2.1.17.7.1.4.3.1.4.".$extVlanId .
    "\t "."4"."\t   "."3"."\t".$char5.$char4.$char3."\n";
    $size =1;
    $configref = InsertLinesRoutine ("1.3.6.1.2.1.17.7.1.4.3.1.4",
        \@addLine, $configref, $size);
#END11

#BEGIN12
    $objectNameList[0] = "1.3.6.1.2.1.17.7.1.4.3.1.5.".$extVlanId;
    $objectNameList[1] = "1.3.6.1.2.1.17.7.1.4.3.1.5.".$extendedVlanId;
    $configref         = obsolete_with_index (\@objectNameList, $configref);
#END12

    return ($configref);
}


#******************************************************************************
# Name vlanmodeaddition 
# Description :- This function is used to add default vlan mode for VLAN.
# Input :- iss.conf file
# OutPut :- None
#******************************************************************************
sub vlanmodeaddition 
{
    my ($refconfigfile) = @_;
    my @configfile      = @{$refconfigfile};

    my $objectToInsert = "1.3.6.1.4.1.2076.65.1.18.1.1";
    my $afterObject    = "1.3.6.1.2.1.17.7.1.4.3.1.5";    

    my $count      = 0;
    my $tempcount  = 0;
    my @tempconfig = "";

    for (@configfile)
    {
        my @fieldEntry = split(/\s+/, $configfile[$count]);
        if ($fieldEntry[0] =~ /\w/)
        { 
            $tempconfig[$tempcount] = $configfile[$count];
            $count++;
            $tempcount++;

            if (($fieldEntry[0] =~ $afterObject) and ($fieldEntry[2] !~ "1"))
            {
                my @TempFieldEntry = split (/^$afterObject\b/,
                    $fieldEntry[0]);
                my @objectIndex    = split (/\./, $TempFieldEntry[1]);
                $tempconfig[$tempcount] = $objectToInsert."." .
                $objectIndex[1]."\t"." 2"."\t"."1"."\n";       
                $tempcount++;
            }
        }
    }

    return \@tempconfig; 
}


#******************************************************************************
#the parse Routine starts here.
#******************************************************************************
#AUTHOR:L.Venkateswara Rao.
#       FutureSoftware Limited
#Description:
#       This routine is used to search the given input TAG in the given file
#   and returns the entries  except the coments in between start and end 
#   of TAG.
#******************************************************************************
sub parse
{
    my ($reference,$searchfor) = @_;
    my $searchfor1 = "<$searchfor>";
    my $search_end = "/$searchfor";
    my $flag       = 0;
    my (@lines)                = "";
    my $count                  = 0;
    my $loopcount;

    my $referencecount = @{$reference};
    for ($loopcount = 0; $loopcount <= $referencecount; $loopcount++)
    {
	my $line = shift(@{$reference});

	# Skip the blank lines
	if ($line =~ /^\s*\n/g)
	{
	    next;
	}
	# Return at end of tag
	if ($line =~ /$search_end/)
	{
	    return @lines;
	}
	elsif ($flag == 1)
	{
	    my @chars = split (/[a-z,A-Z,0-9,\n,\t, ]/, $line);
            # if the line is not a comment then write in array.
            if ($chars[0] ne "#")
            {
                $lines[$count] = $line;
                $count++;
            }
	}
	elsif ($line =~ /$searchfor1/)
	{
	    $flag = 1;
	}	
   }
}#parse routine ends here.

#Renamed 
#******************************************************************************
#Routine:renamed
#       FutureSoftware Limited
#Description:
#       This routine is used to rename the old object Id with the new one
#Input: reference of the lines between the <RENAMED> tag and reference 
#   of config list.
#******************************************************************************
sub renamed
{
    my ($reflist,$configref) = @_;
    my @renlines             = "";
    my $oldobject            = "";
    my $newobject            = "";

    # Source file is migration.dat file,
    # Destination File is iss.config,

    # Get the rename list in an local array.
    @renlines = @{$reflist};
    # Get the objects to be renamed line by line
    for (@renlines)
    {
        my $line = $_;
        # Split the line in to the Oldobject and NewObject
#       next if $line =~ /^\s*$/ ; #warning 
        my @words  = split (/\s+/, $line);
        $oldobject = $words[0];
        $newobject = $words[1];
        # Send the Oldobject and newObject's to the rename routine
        # this will do the actual replacement of the old object with new one
        if( ($oldobject) && ($newobject) ) #warning 
        {
            $configref = renameobj ($oldobject, $newobject, $configref);
        }
    }

    return $configref;
}#end of renamed subroutine.


# renameobj use to rename the Oldobj name with new Objectname in config file
sub renameobj
{
    my ($olddata,$newdata,$configref) = @_;

    my @lines = @{$configref};

    # Replace all olddata  with newdata;
    for(@lines)
    {
        s/$olddata\b/$newdata/;
    }

    return (\@lines);
}#end of the subroutine renameobj

#R36_OPTIMISATION
#******************************************************************************
#Routine:renamed_scalar
#       FutureSoftware Limited
#Description:
#       This routine is used to rename the old scalar object Id with the new one
#Input: reference of the lines between the <RENAMED> tag and reference 
#   of config list.
#******************************************************************************
sub rename_scalar
{
    my ($reflist,$configref) = @_;
    my @renlines             = "";
    my $oldobject            = "";
    my $newobject            = "";
    # Get the rename list in an local array.
    @renlines = @{$reflist};
    my @lines = @{$configref};
    
    my $length = @lines;
    my $start_next = 0;
    my $count = 0;
    # Get the objects to be renamed line by line
    for (@renlines)
    {
        my $line = $_;
        # Split the line in to the Oldobject and NewObject
        my @words  = split (/\s+/, $line);
        $oldobject = $words[0];
        $newobject = $words[1];
        if( ($oldobject) && ($newobject) ) #warning 
        {
            # Traverse through the conf file array
            for ($count = $start_next; $count < $length; $count++)
            {
                my @fields = split (/\s+/, $lines[$count]);
                if ($fields[0] =~ m/^$oldobject\b/) 
                {
                    # Found scalar match, rename and exit
                    $lines[$count] =~ s/$oldobject\b/$newobject/;
                    $start_next = $count+1;
                    last;
                }
                elsif ($count eq ($length - 1))
                {
                    # Array exhausted without any match, exit loop
                    #$start_next = 0; #Start next search from start
                    last;
                }
            }
        }
    }
    $configref = \@lines;
    return $configref;
}#end of renamed subroutine.


# ALU02078598 -S
#******************************************************************************
#Name :- 


#Description :- This function used to chenge enum value of an Object.
#Input :- objectList, Lines
#OutPut :- None
#organisation :- Future Software.
#******************************************************************************
sub replace_value
{

    my($refobjectList,$reflines)=@_;
    my @objectList = @{$refobjectList};
    my @lines = @{$reflines};
    my $count = 0;
    my $flag =0;
    my $temp=0;

    $msg = "\n Replace_Value Starts .........  \n";
    push (@logfile, $msg);
    for(@objectList)
    {
        my @fieldArray = split (/\s+/, $objectList[$temp++]);
        my $objectName = $fieldArray[0];
        my $objectValue = $fieldArray[1];
        $count = 0; 
        if( ($objectName ) && ( $objectName =~ /\w/) ) #warning 
        {
            for(@lines)
            {
 #                next if $lines[$count] =~ /^\s*$/ ; #warning 
                my @fields = split (/\s+/, $lines[$count]);
                my $confObjId= substr($fields[0],0,28);
                if($confObjId =~ m/^$objectName\b/)
                {
                     if($fields[2] > $objectValue )
			{  
		      	    $fields[2]=$objectValue;
			    my $Newline = $fields[0]."\t ".$fields[1]."\t".$fields[2]."\n";
			    $lines[$count] = $Newline ;
                         $msg = "\n New Oid vlaue: $lines[$count]  \n";
                         push (@logfile, $msg);
                         # to change all the other index values
                         $count++;
	                  for(@lines)
                         {
                                #next if $lines[$count] =~ /^\s*$/ ; #warning 
                                my @index_fields = split (/\s+/, $lines[$count]);
				    if('B' eq substr($index_fields[0],0,1))
				    {
				       if($index_fields[2] > $objectValue)
				       {
				          $index_fields[2]=$objectValue;
				          my $New_index_line = $index_fields[0]."\t ".$index_fields[1]."\t".$index_fields[2]."\n";
			                 $lines[$count] = $New_index_line ;
                                      $msg = "\n New Oid vlaue: $lines[$count]  \n";
                                      push (@logfile, $msg);
                                      $count++;
				        }
				     }
				     else
				     {
				         last;
				     }
			      }
            	             $flag=1; 
                      }

                   }  
                 if($flag==1)
                {
                     last;
                } 
                $count++;
            }
        }
    }
    my $refReturn = \@lines;
    return $refReturn;
}
#ALU02078598-E




#******************************************************************************
#Routine:renamed_table
#       FutureSoftware Limited
#Description:
#       This routine is used to rename the old table object Id with the new one
#Input: reference of the lines between the <RENAMED> tag and reference 
#   of config list.
#******************************************************************************

sub rename_table
{
    my ($reflist,$configref) = @_;
    my @renlines             = "";
    my $oldobject            = "";
    my $newobject            = "";

    @renlines = @{$reflist};
    my @lines = @{$configref};
    
    my $length = @lines;
    my $start_next = 0;
    my $count = 0;
    
    # Get the objects to be renamed line by line
    for (@renlines)
    {
        my $line = $_;
        # Split the line in to the Oldobject and NewObject
        my @words  = split (/\s+/, $line);
        $oldobject = $words[0];
        $newobject = $words[1];
        if( ($oldobject) && ($newobject) ) 
        {
            my $flag        = 0;
            #Traverse through the cinfig file array
            for ($count = $start_next; $count < $length; $count++)
            {
                my @fields = split (/\s+/, $lines[$count]);
                if ($fields[0] =~ m/^$oldobject\b/) 
                {
                    $flag = 1; # Match found , Rename
                    $lines[$count] =~ s/$oldobject\b/$newobject/;
                }
                elsif ($flag == 1)
                {
                    # First mismatch ie finished renaming table, exit loop
                    $start_next = $count; #Start next obj search from here
                    last;
                }
                elsif ($count eq ($length - 1))
                {
                    # Object not found, exit loop
                    #$start_next = 0; # Start next obj search from beginning
                    last;
                }
            }
        }
    }
    $configref = \@lines;
    return $configref;
}#end of renamed subroutine.

#******************************************************************************
#Routine:replace_table_index
#       FutureSoftware Limited
#Description:
#       This routine is used to rename the old table object index with the new 
#       index(specified in R3.6 Common Build requirement). 
#Input: reference of the lines between the <RENAMED> tag and reference 
#   of config list.
#******************************************************************************

sub replace_table_index
{
    my ($reflist,$configref) = @_;
    if ($confFileMode eq "CONF_FILE_OPT") 
    {
        return (replace_table_index_optim($reflist,$configref));
    }
    elsif ($confFileMode eq "CONF_FILE_UNOPT") 
    {
        return (replace_table_index_unoptim($reflist,$configref));
    }
    else
    {
        print "ERROR:Invalid conf file mode $confFileMode\n";
        return $configref;
    }
}

sub replace_table_index_optim
{
    my ($reflist,$configref) = @_;
    my @renlines             = "";
    my $oldobject            = "";
    my $tempobject           = "";
    my $newobject            = "";
    my $newindex             = 0;

    @renlines = @{$reflist};
    my @lines = @{$configref};

    my $length = @lines;
    my $start_next = 0;
    my $count = 0;
    my $currobj = "NOT_DEFINED";
    # Get the objects to be renamed line by line
    for (@renlines)
    {
        my $line = $_;
        # Split the line in to the Oldobject 
        my @words  = split (/\s+/, $line);
        $oldobject = $words[0];
        $currobj = $oldobject;
        my $option = $words[1];
        my $objFound = "OBJ_NOT_FOUND";
        
        if( $oldobject)  
        {
            my $flag        = 0;
            $oldobject = "A".$oldobject;
            #Traverse through the config file array
            if($option eq "REPEAT")
            {
                $start_next = 0; #Rpeat option required full search
                #print "Option:$option\n";
            }
            for ($count = $start_next; $count < $length; $count++)
            {
                my @fields = split (/\s+/, $lines[$count]);
                my $firstchar = substr ($fields[0], 0, 1) ;

                if (($firstchar eq "A") and ($fields[0] =~ m/^$oldobject\b/) )
                {
                    $objFound = "OBJ_FOUND";
                    $flag = 1; # Match found , Rename
                    my @arr = split (/$oldobject/ , $fields[0]); 
                    my @oldindex = split (/\./ , $arr[1]);

                    if( validateIndex($oldindex[1],$currobj) eq "INVALID")
                    {
                        next;
                    }
                    else
                    {
                        $newindex = ifIndexMap($oldindex[1]);
                    }
                    $newobject  =  $oldobject.".".$newindex;
                    $tempobject =  $oldobject.".".$oldindex[1];
                    $lines[$count] =~ s/$tempobject\b/$newobject/;
                }
                elsif(($firstchar eq "B") and ($flag == 1))
                {
                    #Next line with matching OID (starts with "B")
                    #Do the same change
                    #In case the format is B<Index>.<Parameter> <> <>
                    my @index  = split (/\./ , $fields[0] );
                    $index[0] = substr($index[0],1);  

                    my $oldindex = $index[0]; 

                    if(validateIndex($oldindex,$currobj) eq "INVALID")
                    {
                        next;
                    }
                    else
                    {
                        $newindex = ifIndexMap($oldindex);
                    }
                    $tempobject = "B".$oldindex;
                    $newobject = "B".$newindex;
                    $lines[$count] =~ s/$tempobject\b/$newobject/;
                }
                elsif (($flag == 1)  and ($option ne "REPEAT"))  #quit loop only if REPEAT option is not given
                {
                    # First mismatch ie finished renaming table, exit loop
                    $start_next = $count; #Start next obj search from here
                    $flag = 0;
                    #print "Quitting at $count for $currobj\n";
                    last;
                }
                elsif (($flag == 1)  and ($option eq "REPEAT"))  #quit loop only of REPEAT option is not given
                {
                    #print "Not quitting at $count for $currobj\n";
                    $flag = 0;
                }
                elsif ($count eq ($length - 1))
                {
                    # Object not found, exit loop
                    #$start_next = 0; # Start next obj search from from where we stopped last
                    last;
                }
            }#End of conf file for loop
        }
    }#End of tag file for loop
    $configref = \@lines;
    return $configref;
}#end of renamed subroutine.


sub replace_table_index_unoptim
{
    my ($reflist,$configref) = @_;
    my @renlines             = "";
    my $oldobject            = "";
    my $tempobject           = "";
    my $newobject            = "";
    my $newindex             = 0;
    my $currobj = "NOT_SPECIFIED";

    @renlines = @{$reflist};
    my @lines = @{$configref};

    my $length = @lines;
    my $start_next = 0;
    my $count = 0;
    # Get the objects to be renamed line by line
    for (@renlines)
    {
        my $line = $_;
        # Split the line in to the Oldobject 
        my @words  = split (/\s+/, $line);
        $oldobject = $words[0];
        $currobj = $oldobject;
        my $option = $words[1];
        my $objFound = "OBJ_NOT_FOUND";
        if( $oldobject)  
        {
            my $flag        = 0;
            if($option eq "REPEAT")
            {
                $start_next = 0; #Rpeat option required full search
                #print "Option:$option\n";
            }
            #Traverse through the config file array
            for ($count = $start_next; $count < $length; $count++)
            {
                my @fields = split (/\s+/, $lines[$count]);
                if ($fields[0] =~ m/^$oldobject\b/ )
                {
                    $objFound = "OBJ_FOUND";
                    $flag = 1; # Match found , Rename
                    my @arr = split (/$oldobject/ , $fields[0]); 
                    my @oldindex = split (/\./ , $arr[1]);
                               
                    if( validateIndex($oldindex[1],$currobj) eq "INVALID")
                    {
                        next;
                    }
                    else
                    {
                        $newindex = ifIndexMap($oldindex[1]);
                    }
                    #print "oldindex:$oldindex[1] newindex:$newindex\n";
                    
                    $newobject  =  $oldobject.".".$newindex;
                    $tempobject =  $oldobject.".".$oldindex[1];
                    $lines[$count] =~ s/$tempobject\b/$newobject/;
                }
                elsif (($flag == 1)  and ($option ne "REPEAT"))  #quit loop only if REPEAT option is not given)
                {
                    # First mismatch ie finished renaming table, exit loop
                    #print "fisrt mismatch at $count\n";
                    $start_next = $count; #Start next obj search from here
                    $flag = 0;
                    #print "Quitting at $count for $currobj\n";
                    last;
                }
                elsif (($flag == 1)  and ($option eq "REPEAT"))  #quit loop only of REPEAT option is not given
                {
                    #print "Not quitting at $count for $currobj\n";
                    $flag = 0;
                }
                elsif ($count eq ($length - 1))
                {
                    # Object not found, exit loop
                    #$start_next = 0; # Start next obj search from beginning where we stopped last
                    last;
                }
            }#End of conf file for loop
        }
    }#End of tag file for loop
    $configref = \@lines;
    return $configref;
}#end of renamed subroutine.

sub validateIndex
{
    my ($oldindex,$currobj) = @_;
    my $retVal = "VALID";
    if ($oldindex < 0)
    {
        print "Error:Invalid index:$oldindex received for $currobj from iss.conf file\n";
        $retVal =  "INVALID";
    }
    elsif ($oldindex <= $constMaxPort)  
    {
        #print "Ignoring index:$oldindex received from iss.conf file\n";
        $retVal =  "INVALID";#Ignore till 24
    }
    elsif($oldindex > ($constMaxIPIf+$constMaxPort))
    {
        #for ALU00977467 
        #print "Ignoring index:$oldindex received for $currobj from iss.conf file\n";
        $retVal =  "INVALID";
    }
    return $retVal; 
}


sub ifIndexMap
{
    my ($oldindex) = @_;
    my $newindex = 0;
    if($oldindex eq ($constMaxIPIf+$constMaxPort-1)) 
    {
        # 279/447 -> 81920 (65536 + 16384) 
        $newindex = 81920; 
    }
    elsif($oldindex eq ($constMaxIPIf+$constMaxPort))
    {
        # 280/448 -> 81921
        $newindex = 81921; 
    }
    elsif(($oldindex > $constMaxPort) and 
    ($oldindex < ($constMaxIPIf+$constMaxPort-1) ))
    {
        # 25->65536 and so on
        $newindex = $oldindex + (65536 - $constMaxPort - 1);
    }
    return $newindex;
}
#******************************************************************************
# Routine:rename_with_index
# Description :- This routine is used to rename a scalar entry with an index   
#                derived from the output of a search function. Thus this  
#                routine is used for scalar to table conversions.
# Input :- Old OID, New OID, Search OID and Search function.
# OutPut:- None
# Orgtanisation :- Aricent
#******************************************************************************
sub rename_with_index
{
    my ($reflist,$configref) = @_;
    my @renlines             = "";
        my $PackageName          = "DataChangePackage";
    
    @renlines = @{$reflist};
    
    for (@renlines)
    {
        my $line = $_;
        my @words  = split (/\s+/, $line);
        
        my $str = $PackageName. "::" .$words[3];
        my $funref = \&$str;
        $configref = $funref->($words[0],$words[1],$words[2],$configref);
    }
    return $configref;
}


#*****************************************************************************
# Name :- Obsolete 
# Description :- This routine is used to delete a entry from configuration file 
# Input :- objectNameList, Lines
# OutPut:- None
#orgtanisation :- Future Soft
#*****************************************************************************
sub obsolete 
{
    my ($RefobjectNameList, $Reflines) = @_;
    my @objectNameList = @{$RefobjectNameList};
    my @lines          = @{$Reflines};
    my $countArray     = 0;

    if ($trace == 1)
    {

        $msg = "\n OBSOLETED \n";
        push (@logfile, $msg);
    }

    for (@objectNameList)
    {
        my @fieldArray = split (/\s+/, $objectNameList[$countArray++]);
        my $objectName = $fieldArray[0];
        my @tempArray = " ";
        my $count       = 0;
        my $temp        = 0;
        if ( ($objectName) && ($objectName =~ /\w/) ) #warning 
        {
            for (@lines) 
            {
#                next if $lines[$count] =~ /^\s*$/ ; #warning 
                my @fields = split (/\s+/, $lines[$count]);
                if ($fields[0] !~ m/^$objectName\b/) 
                {
                    $tempArray[$temp++] = $lines[$count];
                }
                else
                {
                    if ($trace == 1)
                    {
                        $msg = " $lines[$count]";
                        push (@logfile, $msg);
                    }
                }
                $count++;
            }
            @lines = @tempArray; 
        }
    }
    my $refReturn = \@lines;
    return $refReturn;
}

#*****************************************************************************
# Name :- Obsolete_optim 
# Description :- This routine is used to delete a entry from configuration file
#                This routine is applicable to the Conf file from R3.6 onwards 
#                since the conf file is made optimised from R3.6 onwards.
# Input :- objectNameList, Lines
# OutPut:- None
#orgtanisation :- Future Soft
#*****************************************************************************
sub obsolete_optim
{
    my ($RefobjectNameList, $Reflines) = @_;
    my @objectNameList = @{$RefobjectNameList};
    my @lines          = @{$Reflines};
    my $countArray     = 0;



    my $length = @lines;
    my $start_next = 0;
    my $count = 0;

    if ($trace == 1)
    {

        $msg = "\n OBSOLETED \n";
        push (@logfile, $msg);
    }
    for (@objectNameList)
    {
        my @fieldArray = split (/\s+/, $objectNameList[$countArray++]);
        my $objectName = $fieldArray[0];
        my @tempArray = " ";
        my $count       = 0;
        my $temp        = 0;
        $objectName = "A".$objectName;
        my $objFound = "OBJ_NOT_FOUND";
        my $tempcount   = 0;

        if ( ($objectName) && ($objectName =~ /\w/) ) #warning 
        {
            my $flag        = 0;
            for (@lines) 
            {
                my @fields = split (/\s+/, $lines[$count]);
                my $firstchar = substr ($fields[0], 0, 1) ;
                if ($fields[0] !~ m/^$objectName\b/)
                {
                    $tempArray[$temp++] = $lines[$count];
                    $count++;
                }
                elsif ( ($fields[0] =~ m/^$objectName\b/) and ($firstchar eq "A"))
                {
                    $count++;
                     @fields = split (/\s+/, $lines[$count]);
                     $firstchar = substr ($fields[0],0,1);
                    while(($firstchar eq "B"))
                    { 
                        print ("first - B");
                        if ($trace == 1)
                    {
                        $msg = " $lines[$count]";
                        push (@logfile, $msg);
                    }
                        $count++;
                     @fields = split (/\s+/, $lines[$count]);
                     $firstchar = substr ($fields[0],0,1);
                    }
                }
                else
                {
                    if ($trace == 1)
                    {
                        $msg = " $lines[$count]";
                        push (@logfile, $msg);
                    }
                }

            }
            @lines = @tempArray; 
        }
    }
    my $refReturn = \@lines;
    return $refReturn;


}

#*****************************************************************************
# Name :- obsolete_scalar 
# Description :- This routine is used to delete a scalar from config file 
# Input :- objectNameList, Lines
# OutPut:- None
#orgtanisation :- Future Soft
#*****************************************************************************
sub obsolete_scalar 
{
    my ($RefobjectNameList, $Reflines) = @_;
    my @objectNameList = @{$RefobjectNameList};
    my @lines          = @{$Reflines};
    my $countArray     = 0;


    my $start_next = 0;
    my $count = 0;
    for (@objectNameList)
    {
        my @fieldArray = split (/\s+/, $objectNameList[$countArray++]);
        my $objectName = $fieldArray[0];

        if ( ($objectName) && ($objectName =~ /\w/) ) 
        {
            my $length = @lines;
            # Traverse through the config file and search for match
            for ($count = $start_next; $count < $length; $count++) 
            {
                my @fields = split (/\s+/, $lines[$count]);
                if ($fields[0] =~ m/^$objectName\b/) 
                {
                    # Match found for scalar, delete the line and exit the loop
                    splice(@lines,$count, 1);
                    $start_next = $count; # Start search of next object from
                    #this line in config file
                    last;
                }
                elsif ($count eq ($length -1))
                {
                    # Couldnt find the object,exit loop and start search for
                    # next object from begining of the config file
                    #$start_next = 0;
                    last;
                }
            }
        }
    }
    my $refReturn = \@lines;
    return $refReturn;
}

#*****************************************************************************
# Name :- obsolete_table 
# Description :- This routine is used to delete a table from config file 
# Input :- objectNameList, Lines
# OutPut:- None
#orgtanisation :- Future Soft
#*****************************************************************************
sub obsolete_table 
{
    my ($RefobjectNameList, $Reflines) = @_;
    my @objectNameList = @{$RefobjectNameList};
    my @lines          = @{$Reflines};
    my $countArray     = 0;

    my $start_next = 0;
    my $count = 0;
    for (@objectNameList)
    {
        my @fieldArray = split (/\s+/, $objectNameList[$countArray++]);
        my $objectName = $fieldArray[0];

        my $flag        = 0;
        if ( ($objectName) && ($objectName =~ /\w/) )  
        {
            my $length = @lines;
            for ($count = $start_next; $count < $length; $count++) 
            {
                my @fields = split (/\s+/, $lines[$count]);
                if ($fields[0] =~ m/^$objectName\b/) 
                {
                    $flag = 1; # Match found
                    splice(@lines,$count, 1);
                    $count --;
                }
                elsif ($flag == 1)
                {
                    # First mismatch after the table ie entire table deleted
                    # exit loop and search for next object
                    $start_next = $count;
                    last;
                }
                elsif ($count eq ($length - 1))
                {
                    # Couldnt find the object,exit loop and start search for
                    # next object from begining of the config file
                    #$start_next = 0;
                    last;
                }
            }
        }
    }
    my $refReturn = \@lines;
    return $refReturn;
}

#*****************************************************************************
# Name :- obsolete_with_index 
# Description :- This routine is used to delete a entry supplied with the 
#                specified index from configuration file 
# Input :- objectNameList, Lines
# OutPut:- None
#orgtanisation :- Future Soft
#*****************************************************************************
sub obsolete_with_index 
{
    my ($RefobjectNameList, $Reflines) = @_;
    my @objectNameList = @{$RefobjectNameList};
    my @lines          = @{$Reflines};
    my $count      = 0;
    my $countArray = 0;
    my $temp       = 0;

    if ($trace == 1)
    {

        $msg = "\n OBSOLETED WITH INDEX \n";
        push (@logfile, $msg);
    }

    for (@objectNameList)
    {
        my @fieldArray = split (/\s+/, $objectNameList[$countArray++]);
        my $objectName = $fieldArray[0];
        my @tempArray  = " ";
        $count         = 0;
        $temp          = 0;

        if (($objectName) && ($objectName =~ /\w/) ) #warning 
        {
            for (@lines) 
            {
#                next if $lines[$count] =~ /^\s*$/ ; #warning 
                my @fields = split (/\s+/, $lines[$count]);
                if ($fields[0] ne $objectName) 
                {
                    $tempArray[$temp++] = $lines[$count];
                }
                else
                {
                    if ($trace == 1)
                    {
                        $msg = "     $lines[$count]";
                        push (@logfile, $msg);
                    }
                }
                $count++;
            }
            @lines = @tempArray; 
        }
    }

    my $refReturn = \@lines;
    return $refReturn;
}


#******************************************************************************
#Name : DataTypeChange
#Description:This Routine is used to change data type of an object.
#        In the specified file.
#Input: objectList, Lines.
#Organisation:FutureSoftware 
#******************************************************************************
use DataChangePackage;
sub datatype_change
{
    my ($RefobjectList, $RefDatalines) = @_;
    my @objectList = @{$RefobjectList};
    my @linesData  = @{$RefDatalines};   
    my $count      = 0;
    my $temp       = 0;
    my $PackageName                    = "DataChangePackage";

    for (@objectList)
    {
        my @fieldArray   = split (/\s+/, $objectList[$temp++]);
        my $objectName   = $fieldArray[0];
        my $newdataType  = $fieldArray[2];
        my $functionName = $fieldArray[3];
        $count           = 0;

        if (($objectName) && ($objectName =~ /\w/)) #warning 
        {
            for (@linesData)
            {
#                next if $linesData[$count] =~ /^\s*$/ ; #warning 

                my @fields = split (/\s+/, $linesData[$count++]);

                if (($fields[0] =~ m/^$objectName\b/) and 
                    ($functionName ne "NULL"))
                {
                    my $str       = $PackageName. "::" .$functionName;
                    my $funref    = \&$str;
                    my $returnVal = $funref->($fields[2]); 

#Jayalk : modifying subroutine call by reference .
# my  $returnVal = &{$PackageName . "::" . $functionName}($fields[2]);

                    s/$fields[1]/$newdataType/;
                    s/$fields[2]/$returnVal/;
                }
                if($fields[0] =~ m/^$objectName\b/)
                { 
                    s/$fields[1]/$newdataType/;
                }
            } 
        }
    }
    my $refReturn = \@linesData;
    return $refReturn;

}#end of the subroutine 


#******************************************************************************
#Name :- enum_change
#Description :- This function used to chenge enum value of an Object.
#Input :- objectList, Lines
#OutPut :- None
#organisation :- Future Software.
#******************************************************************************
sub enum_change
{

    my($refobjectList,$reflines)=@_;
    my @objectList = @{$refobjectList};
    my @lines = @{$reflines};
    my $count = 0;
    my $temp = 0;

    $msg = "\n Enum Change Starts .........  \n";
    push (@logfile, $msg);
    for(@objectList)
    {
        my @fieldArray = split (/\s+/, $objectList[$temp++]);
        my $objectName = $fieldArray[0];
        my $oldEnumValue = $fieldArray[1];
        my $newEnumValue = $fieldArray[2];

        $count = 0; 
        if( ($objectName ) && ( $objectName =~ /\w/) ) #warning 
        {
            for(@lines)
            {
#                next if $lines[$count] =~ /^\s*$/ ; #warning 
                my @fields = split (/\s+/, $lines[$count]);
                if(($fields[0] =~ m/^$objectName\b/) and 
                    ($oldEnumValue eq $fields[2]))
                {

                    $msg = "\n Old Oid : $lines[$count]  \n";
                    push (@logfile, $msg);
                    my $Newline = $fields[0]."\t ".$fields[1]."\t ".$newEnumValue."\n";
# Jaya                   s/$fields[2]/$newEnumValue/;    
                    $lines[$count] = $Newline ;
                    $msg = "\n New Oid : $lines[$count]  \n";
                    push (@logfile, $msg);

                }
                $count++;
            }
        }
    }
    my $refReturn = \@lines;
    return $refReturn;
}

#******************************************************************************
#Name :- enum_change_optim
#Description :- Optimised function used to chenge enum value of an Object.
#Input :- objectList, Lines
#OutPut :- None
#organisation :- Future Software.
#******************************************************************************
sub enum_change_optim
{

    my($refobjectList,$reflines)=@_;
    my @objectList = @{$refobjectList};
    my @lines = @{$reflines};
    my $count = 0;
    my $start_next = 0;
    my $length = @lines;
    for(@objectList)
    {
        my @fieldArray = split (/\s+/, $_);
        my $objectName = $fieldArray[0];
        my $oldEnumValue = $fieldArray[1];
        my $newEnumValue = $fieldArray[2];

        $count = 0; 
        if( ($objectName ) && ( $objectName =~ /\w/) ) #warning 
        {
            for ($count = $start_next; $count < $length; $count++)
            {
                my @fields = split (/\s+/, $lines[$count]);
                if(($fields[0] =~ m/^$objectName\b/) and 
                    ($oldEnumValue eq $fields[2]))
                {
                    my $Newline = $fields[0]."\t ".$fields[1]."\t ".$newEnumValue."\n";
                    $lines[$count] = $Newline ;
                    $start_next = $count + 1;
                    last; # Exit config file loop, resume from here for next obj
                }
                elsif ($count eq ($length - 1))
                {
                    # Array exhausted without any match, exit loop
                    #$start_next = 0;
                    last;
                }
            }
        }
    }
    my $refReturn = \@lines;
    return $refReturn;
}

#******************************************************************************
#Name:- access_change
#descrition :- This routine used to change access of an object.
#Input :- objectList, Lines.
#OutPut :- none.
#Organisation :- Future Software
#******************************************************************************
sub access_change
{
    my($refobjectList,$reflines)=@_;
    my  @objectList = @{$refobjectList};
    my @lines = @{$reflines};
    my $count = 0;
    my $temp = 0;

    for(@objectList)
    {
        my @fieldArray = split (/\s+/, $objectList[$temp++]);
        my $objectName = $fieldArray[0];
        my $oldAccessValue = $fieldArray[1];
        my $newAccessValue = $fieldArray[2];
        my @tempArray      = "";

        if ( ($objectName)&& ($objectName =~ /\w/) ) #warning 
        {
            for(@lines)
            {
#                next if $lines[$count] =~ /^\s*$/ ; #warning 
                my @fields = split (/\s+/,$lines[$count]);
                if($fields[0] !~ m/^$objectName\b/) 
                {
                    $tempArray[$temp++] = $lines[$count];
                }

                $count++;
            }
            @lines = @tempArray;
        }  
        @tempArray = " ";
        $count = 0;
    }    

    my $refReturn = \@lines;
    return $refReturn;
} 


#******************************************************************************
# Name         : table_index_change
# Description  : This function used to map table index change.
# Input        : $refTableName, $refConfigFile.
# Output       : None.
# Oragnisation : Future Software.
#******************************************************************************
sub table_index_change
{
    my($refTableName, $refConfigFile, $tag_sent ) = @_;

    my @TableName     = @{$refTableName};
    my @ConfigFile    = @{$refConfigFile};
    my $refRet        = \@ConfigFile;
    my $count         = 0;
    my $loopCount     = 0;
    my $NumOfObjects  = 0;
    my $NumOldIndices = 0;
    my $NumNewIndices = 0;
    my $refTableObjects ;
    my @tableObjects  = "";
    my @oldIndices    = "";
    my $refOldIndices = '' ;
    my @newIndices    = "" ;
    my $refNewIndices = '' ;
    my $CopyVar       = '';
    my @TagName       = "";
    my @charCheck     = "";

    for(@TableName)
    {

#        next if $TableName[$count] =~ /^\s*$/ ; #warning 
        my @CheckBlank = split (/\s+/, $TableName[$count++]);

        if ($CheckBlank[0] =~ /\w+|\d+/)
        {

            if (($CheckBlank[0] =~ /</) and ($CheckBlank[0] =~ />/))
            {
                $CopyVar      = $count;
                @TagName   = split (/<|>/, $TableName[($count-1)]);
                @charCheck = split (//, $TagName[1]);

                if($charCheck[0] !~ "/")
                {

                    if ($TagName[1] =~ "TABLE_OBJECTS")
                    {

                        $NumOfObjects = $TableName[$CopyVar++];
                        for (my $var = 0; $var < $NumOfObjects; $var++)
                        {
                            $tableObjects[$var] = $TableName[$CopyVar++];    
                        }
                        $refTableObjects = \@tableObjects;
                    }

                    if ($TagName[1] =~ "OLD_INDICES")
                    {

                        $NumOldIndices = $TableName[$CopyVar++];
                        for (my $var = 0; $var < $NumOldIndices; $var++)
                        {
                            $oldIndices[$var] = $TableName[$CopyVar++];    
                        }
                        $refOldIndices = \@oldIndices;
                    }

                    if ($TagName[1] =~ "NEW_INDICES")
                    {

                        $NumNewIndices =  $TableName[$CopyVar++];
                        for(my $var = 0; $var < $NumNewIndices; $var++)
                        {
                            $newIndices[$var] = $TableName[$CopyVar++];       
                        }
                        $refNewIndices = \@newIndices;
                    }
                }

                else 
                {

                    my $NEW_INDICES = "NEW_INDICES"; 
                    if ($TagName[1] =~ "NEW_INDICES")
                    {

                        if ($tag_sent =~"MULTICAST_TABLE_INDEX_CHANGE")
                        {

                            $refRet = MulticastIndexChange ($refTableObjects, 
                                $NumOldIndices,
                                $refOldIndices, 
                                $NumNewIndices, 
                                $refNewIndices, 
                                $refRet          );
                        }
                        else
                        {

                            $refRet = IndexDataTypeChange ($refTableObjects, 
                                $NumOldIndices, 
                                $refOldIndices, 
                                $NumNewIndices, 
                                $refNewIndices, 
                                $refRet           );    
                        }

                        $refTableObjects = 0;
                        $refOldIndices   = 0;
                        $refNewIndices   = 0;
                    }
                } #end of else 
            }
            $CopyVar = 0; 
        } 
    }  

    return $refRet; 
}


#******************************************************************************
# Name        : IndexDataTypeChange
# Description : This routine is used to change data type of index
# Input       : $refTableObjects, $refOldIndices,$numNewIndices, 
#              $refNewIndices, $refConfigFile
# OutPut      : None 
# Orgnisation : Future Software.
#******************************************************************************
use DataChangePackage;
sub IndexDataTypeChange
{

    my ($TableObjectsRef, $numOldIndices, $OldIndicesRef, 
        $numNewIndices, $NewIndicesRef, $ConfigFileRef)   = @_;

    my @TableObjects    = @{$TableObjectsRef};
    my @OldIndices      = @{$OldIndicesRef};
    my @NewIndices      = @{$NewIndicesRef};
    my @ConfigFileArray = @{$ConfigFileRef};
    my $PackName        = "DataChangePackage";
    my $count           = 0;
    my $temp            = 0;

    if ($trace == 1)
    {
        $msg="\n Table Objects:\n @TableObjects \n";
        push (@logfile,$msg);
        push (@logfile, "\n OldIndices:\n @OldIndices \n");
        push (@logfile, "\n NewIndices:\n @NewIndices \n");
    }


    for (@TableObjects)
    {

        $count            = 0;
        my @TabObjectName = "";
#        next if $TableObjects[$temp] =~ /^\s*$/ ; #warning 
        @TabObjectName    = split (/\s+/, $TableObjects[$temp++]);

        if ($TabObjectName[0] =~ /\w/)
        {

            for (@ConfigFileArray)
            {

#                next if $TableObjects[$count] =~ /^\s*$/ ; #warning 
                my @ConfigFields = split (/\s+/, $ConfigFileArray[$count++]);

                if ($ConfigFields[0] =~ m/^$TabObjectName[0]\b/)
                {

                    my $ObjectToWrite = " ";

                    my @TempObjectIndex  = split (/$TabObjectName[0]/, 
                        $ConfigFields[0]);
                    my @TokenArray  = split (/\./, $TempObjectIndex[1]);
                    $TokenArray[0]    = $TabObjectName[0];
                    $ObjectToWrite    = $TokenArray[0];
                    my $tokenVar      = 0;
                    my $localVar      = 0; 
                    my $ValueToWrite  = 0 ;
                    my $loopVariable  = 0;
                    my $oldVar        = 0;
                    my $Index_Fond    = 0;

                    for (my $k=0; $k < int($numNewIndices); $k++)
                    {

                        my @newIndicesFields = "";
                        @newIndicesFields = split(/\s+/, $NewIndices[$localVar]);
                        $localVar++;                 
                        $oldVar = 0;
                        $Index_Fond = 0;
                        $ObjectToWrite = $TokenArray[0];
                        for(my $l=0 ; $l < int($numOldIndices); $l++)
                        {   
                            my @OldIndicesFields = "";      
                            @OldIndicesFields = split(/\s+/, $OldIndices[$oldVar]);
                            $oldVar++;
                            if(($newIndicesFields[0] =~ /\w/) and 
                                ($OldIndicesFields[0] =~ /\w/))
                            {

                                if($newIndicesFields[0] eq $OldIndicesFields[0])
                                {

                                    my $IndexToSkip= $newIndicesFields[3];
                                    if($newIndicesFields[1] ne "NULL") 
                                    {
                                        my $funValueRet='';
                                        {

                                            my $str = $PackName. "::" .$newIndicesFields[1];
                                            my $funref = \&$str;

#Jayalk : here also subroutine call by ref is modifeid.

                                            $funValueRet = 
                                            $funref->(($TokenArray[($loopVariable+1)]));
                                        }
                                        if ( $IndexToSkip =~ "NULL")
                                        {
                                            $ValueToWrite = $funValueRet; 
                                        } 
                                        else
                                        {
                                            $ValueToWrite = $ValueToWrite.".".$funValueRet;
                                        }
                                        $loopVariable ++;

                                    }
                                    else 
                                    {
                                        if ($newIndicesFields[2] eq "NULL") 
                                        {
                                            my $toSkip = 0;
                                            if($newIndicesFields[3] ne "NULL")
                                            {

                                                $toSkip=int($newIndicesFields[3]);

                                                my $i=$loopVariable+1; 
                                                my $checkVar;
                                                if( $loopVariable == 0)
                                                {
                                                    $ValueToWrite = $TokenArray[$i];
                                                    $checkVar=$toSkip-1;
                                                }
                                                else 
                                                {
                                                    $ValueToWrite = $ValueToWrite;
                                                    $checkVar = $toSkip;
                                                    $i--; 
                                                }

                                                for(my $j =0; $j < $checkVar;  $j++)
                                                {
                                                    $ValueToWrite = $ValueToWrite.".". 
                                                    $TokenArray[$i+1];
                                                    $i++;
                                                }  

                                                $loopVariable = $loopVariable + $toSkip;
                                            }
                                            else {
                                                $ValueToWrite = 
                                                $TokenArray[$loopVariable++];
                                            }

                                        }
                                        else {
                                            $ValueToWrite = $newIndicesFields[2];
                                            $loopVariable ++;
                                        }
                                    }

                                    $Index_Fond = 1;
                                    $tokenVar++;
                                    last;
                                }

                            }

                        }    
                        if($Index_Fond == 1)
                        {
                            $ObjectToWrite = $ObjectToWrite . "." . $ValueToWrite;    
                        }
                        else 
                        {
                            $ObjectToWrite = $ObjectToWrite . "." . $newIndicesFields[2];
                        }
                    }
                    s/$ConfigFields[0]/$ObjectToWrite/gc;   
                }
            }
        } 
    }
    my $arrayRef = \@ConfigFileArray; 
    return $arrayRef;
}


#******************************************************************************
#Name :- object_dependencies
#Description :- This routine is used to find place to insert objects before 
#               Dependant object.
#InPut :- objectList, lines
#output :- None
#Orgnisation :- Future Software
#*******************************************************************************
sub object_dependencies
{
    my($RefobjectList,$Reflines)=@_;
    my @objectList = @{$RefobjectList};
    my @lines = @{$Reflines};       
    my $count = 0;
    my $tempVar = 0;
    my $nextPos = 0;
    my @lineToBeInsert ;
    my $refToLines = \@lines;
    my $refValue = \@lines;

    for (@objectList)
    {
        if (($objectList[$count] !~ /\$/)  and ($objectList[$count] !~ /\$/))
        {

            $lineToBeInsert[$tempVar++] = $objectList[$count];
        } 
        else 
        {

            my @value = split(/\$/, $objectList[$count]);
            my $refLineToBeInsert = \@lineToBeInsert;
            #$refValue = InsertLinesRoutine($value[1], $refLineToBeInsert,
            $refValue = InsertLinesRoutineOptim($value[1], $refLineToBeInsert,
                $refToLines, $tempVar, \$nextPos);
            $tempVar = 0;
            @lineToBeInsert = "";
            $refLineToBeInsert = 0;
            @lines = @{$refValue};

        } 
        $count++;
    }
    return $refValue; 
}


#******************************************************************************
#Name :- InsertLinesRoutine 
#Description :- This routine used insert objects at appropriate Place.
#Input :- $objectName, @lineToInsert, @lines, $temp.
#output :- None.
#Orgnisation :- Future Software.
#******************************************************************************

sub InsertLinesRoutine
{
# Objectnamebefore lines to be inserted , reflinestobe inserted,
# config file ref and count of number of lines.

    my($objectName,$reflineToInsert,$reflines,$temp) = @_;

    my    @lineToInsert = @{$reflineToInsert};
    my    @lines = @{$reflines};
    my    $count = 0;
    my    $tempVar = 0;

    my @tempList = @lines;
    my @tempNewlist;


#vk updation start
#if the objectname is END then add the entries at the end of the config list
#    print"checking for name as END \n";

    if($objectName =~/TOP/)
    {
        if ($trace == 1)
        {
            $msg = "\n Trying to add at the TOP \n";
            push (@logfile, $msg);
        }

        #add at the top without decoding with issnvram.txt.
        $i = 0;
        my $j = 1;
        for(@lineToInsert)
        {
            $tempNewlist[$i++] = $lineToInsert[$j++];
        }

        for(@lines)
        {
            $tempNewlist[$i++] = $_;
        }
        @tempList = @tempNewlist;

        if ($trace == 1)
        {
            push (@logfile, "\n INSERTED AT TOP @lineToInsert \n");
        }

        return \@tempList;
        # }#end of else R2.2        
    }#end of if /TOP/

    if($objectName =~ /END/)
    {
#find the location of CKSUM before that we have to add the lines
        my $endofconfig = wheretag(\@tempList,"CKSUM");     
        chomp($tempList[$endofconfig]);
        for(@lineToInsert)
        {
            $tempList[$endofconfig]=$_;
            $endofconfig++;
        }
        $tempList[$endofconfig++]="CKSUM:";
    }
    else
    { 
#vk updation end
        if( $objectName ne "NULL")
        {
            for(@lines)
            {
                my @fields = split (/\s+/, $lines[$count]);
                if($fields[0] !~ m/^$objectName\b/)
                {
                    $tempList[$tempVar] = $lines[$count];
                }
                if($fields[0] =~ m/$objectName\b/)
                { 
                    my $var='';
                    for ($var = 0; $var < $temp ; $var++)
                    {
                        $tempList[$tempVar++] = $lineToInsert[$var];

                    }

                    $tempList[$tempVar] = $lines[$count];
                    #Vki,, Added 
                    #Here the entries are added before the searched object so to stop
                    #further additions fill the $objectName with some nonsearchable string.
                    $objectName ="XXXXXXXXX"; # Mathew Confirm??
                }

                $tempVar++;  
                $count++;
            }#end of for
        }#end of if($objectName
        else
        {
            my $afterObject = "1.3.6.1.2.1.17.7.1.4.3.1.5";
            my $tempCount = 0;
            my $variable = 0;
            my $count = 0;

            for(@lineToInsert)
            {
                my @objectDetail = split(/\s+/, $lineToInsert[$variable]);
                my @TempNameDetails = split(/$afterObject\b/, $lineToInsert[$variable]);
                my @nameDetails = split(/\./, $TempNameDetails[1]);


                $tempCount = 0;
                $count=0;
                @tempList="";

                for(@lines)
                {
                    my @objectCheck = split(/\s+/, $lines[$count]);
                    my @TempCheckVar = split(/$afterObject\b/, $lines[$count]);
                    my @checkVar = split(/\./, $TempCheckVar[1]);

                    $tempList[$tempCount] = $lines[$count];
                    $tempCount++;
                    $count++;
                    if(($objectCheck[0] =~ $afterObject) and ($nameDetails[1] eq
                            $checkVar[1]) and ($objectCheck[2] !~ "1"))
                    {
                        $tempList[$tempCount] = $lineToInsert[$variable];         
                        $tempCount++;
                    }
                }  
                @lines = @tempList; 
                $variable++;
            }   
        }

    }#end of else
    @lines = @tempList;

    if ($trace == 1)
    {
        push (@logfile, "\n INSERTED @lineToInsert \n");
    }

    my $returnValue = \@lines;
    return $returnValue;

}

#******************************************************************************
#Name :- InsertLinesRoutineOptim 
#Description :- This routine used insert objects at appropriate Place.
#Input :- $objectName, @lineToInsert, @lines, $temp.
#output :- None.
#Orgnisation :- Future Software.
#******************************************************************************
sub InsertLinesRoutineOptim
{
# Objectnamebefore lines to be inserted , reflinestobe inserted,
# config file ref and count of number of lines.

    my($objectName,$reflineToInsert,$reflines,$temp,$nextPosref) = @_;

    my    @lineToInsert = @{$reflineToInsert};
    my    @lines = @{$reflines};
    my    $count = 0;
    my    $tempVar = 0;
    my    $linesLen = 0;

    my @tempList = @lines;
    my @tempNewlist;


#vk updation start
#if the objectname is END then add the entries at the end of the config list
#    print"checking for name as END \n";

    if($objectName =~/TOP/)
    {
        @lines = (@lineToInsert,@lines);
        return \@lines;
    }#end of if /TOP/

    if($objectName =~ /END/)
    {
        pop(@lines);
        @lines = (@lines,@lineToInsert);
        push(@lines,"CKSUM:");
        return \@lines;
    }
    else
    { 
        if( $objectName ne "NULL")
        {
            #for(@lines)
            $linesLen = @lines;
            for($count = $$nextPosref; $count < $linesLen ; $count++)
            {
                my @fields = split (/\s+/, $lines[$count]);
                if($fields[0] =~ m/$objectName\b/)
                { 
                    splice(@lines,$count,0,@lineToInsert);
                    $count += @lineToInsert; 
                    $linesLen +=  @lineToInsert;
                    $$nextPosref = $count;
                    last;
                }
                #$count++;
            }#end of for
            return \@lines;
        }#end of if($objectName
        else
        {
            my $afterObject = "1.3.6.1.2.1.17.7.1.4.3.1.5";
            my $tempCount = 0;
            my $variable = 0;
            my $count = 0;

            for(@lineToInsert)
            {
                my @objectDetail = split(/\s+/, $lineToInsert[$variable]);
                my @TempNameDetails = split(/$afterObject\b/, $lineToInsert[$variable]);
                my @nameDetails = split(/\./, $TempNameDetails[1]);


                $tempCount = 0;
                $count=0;
                @tempList="";

                for(@lines)
                {
                    my @objectCheck = split(/\s+/, $lines[$count]);
                    my @TempCheckVar = split(/$afterObject\b/, $lines[$count]);
                    my @checkVar = split(/\./, $TempCheckVar[1]);

                    $tempList[$tempCount] = $lines[$count];
                    $tempCount++;
                    $count++;
                    if(($objectCheck[0] =~ $afterObject) and ($nameDetails[1] eq
                            $checkVar[1]) and ($objectCheck[2] !~ "1"))
                    {
                        $tempList[$tempCount] = $lineToInsert[$variable];         
                        $tempCount++;
                    }
                }  
                @lines = @tempList; 
                $variable++;
            }   
        }

    }#end of else
    @lines = @tempList;

    print "INSERTED @lineToInsert";

    my $returnValue = \@lines;
    return $returnValue;

}

#******************************************************************************
#subroutine: gettag
#         FutureSoftware Limited.
#Description:gettag which will get the tags list sequentially from the bigining
#        of the file
#******************************************************************************
sub gettags
{
    my ($reflist,$refsubtags)= @_;
    my @words = "";
    my @tags = "";
    my @list = @{$reflist};
    my $i = 0;
    my $status=-1;

    for(@list)
    {
        my  $line = $_;
        if($line=~/</ && $line =~/>/)
        {
            @words = split (/(([a-zA-Z_])+)/,$line);

            if(@words)
            {
#the found tag shouldn't be member of the subtags 
#if so omit and don't add in the tags list.
                $status = wheretag($refsubtags,$words[1]);
                if ($status == -1)
                {
                    $tags[$i]=$words[1];
                    $i++;
                }
            }

        }
    }

    my $j = $i/2;
    $i =0;
    my @maintags;
    while ($j >0)
    {
        $maintags[$i/2]=$tags[$i];
        $i+=2;
        $j--;
    }
#return the list of the tags from the begining of the file
    return @maintags;

} 
#end of the subroutine.


#******************************************************************************
#subroutine:    wheretag
#          FutureSoftware Limited
#this routine "wheretag" is used to find out the location of the searching tag
#in the given array list.
#inputs:Reference of the list in which tag tobe searched
#       and tagtofind
#output:position of the searching tag in the given list
#******************************************************************************

sub wheretag
{
    my ($tagref,$tagtofind) = @_;
    my @tags = @{$tagref};
    my $tagsize = @tags;
    my $i =0;

    for( ; $i < $tagsize; $i++)
    {
        if( ($tags[$i] =~ $tagtofind) and ($tagtofind =~ /\w/) )
        {
            return $i;
        }
    }
    return (-1);
}
#end of wheretag subroutine


#******************************************************************************
#subroutine:    rmblanklines
#          FutureSoftware Limited
#Description:the main functionality of this routine is to remove the blank lines
#            which occured in the given list i
#Input:    List which has the blank lines.
#******************************************************************************
sub rmblanklines
{
    my ($listref) = @_; 
    my @list=@{$listref};
    my @newlist;
    my $listsize = @list;

    for(my $i=0,my $j=0; $i<$listsize;$i++)        
    {
        my $line = shift(@list);
        if($line =~ /\w+/)
        {
            $newlist[$j++]=$line;
        }
    }
#return the reference of the new list
    return (\@newlist);
}


#*****************************************************************************
# subroutine:    argcheck
#          FutureSoftware Limited
#Description:the main functionality of this routine is to check the CommandLine
#            Interface syntax  if the syntax fails it exits from the tool  
#*****************************************************************************

sub argcheck
{

    my $retVal   = 0;
    my $numargs  = $#ARGV + 1 ;
    my $option='' ;

#indu - added for cli command change [07/11/06][START]
        $option = GetOptions('ver'=>\$version,
                'i'=>\$interactivemode,
                'f=s'=>\$filemode);
#indu - added for cli command change [07/11/06][END]

    if(!$option)
    {
        exit(1);
    }
    if(($version)&&($numargs==1))
    {
        print "\n Migrationtool.pl VERSION 1.84\n\n\n";  # BUG 14244 #Bug 14380 #Bug14439 #Bug14620 #bug14763 #bug14875 #bug14927
        exit(0);
    }
#indu - added for cli command change [07/11/06][START]
        if(($interactivemode) && ($numargs eq 1))
        {
                $dispmsg = "\n\t Entered interactive mode\n";
        push(@logfile,$dispmsg);
                interactivemode();
        }
        else
        {
                if($filemode)
                {
                        $dispmsg =  "\n\t Entered file mode (filemode)\n";
            push(@logfile,$dispmsg);
                        $inputFileName = $filemode;
                        $dispmsg = "\n\tThe input file is : $filemode\n";
            push(@logfile,$dispmsg);
                        infilemode();
                }
                else
                {
                        print "\n\t The syntax is perl Migrationtool.pl <-i / -f>\n";
                        print "\n\t Enter i for interactive mode and f for filemode\n";
                        $modeoption=<STDIN>;
                        chop $modeoption;

                        if($modeoption eq "i")
                        {
                print "\n\t The option is : interactive mode\n";
                                interactivemode();
                        }
                        if($modeoption eq "f")
                        {
                print "\n\t The option is : file mode\n";
                print "\n\t Enter the filename\n";
                                $inputFileName = <STDIN>;
                                chop $inputFileName;
                                infilemode();

                        }
                }
        }
if(($interactivemode) || ($modeoption eq "i"))
        {
                        print "\n\t Enter the mode from which migration has to take place\n";
            print "\n\tThe value can be ECNTA or ECNTC\n";
                        $climodefrom = <STDIN>;
                        chop $climodefrom;
            if(!(($climodefrom =~ m/^ECNTA\b/) || ($climodefrom =~ m/^ECNTC\b/) || ($climodefrom =~ m/^ecnta\b/) || ($climodefrom =~ m/^ecntc\b/)))
                            {
                                    $msg = "\n Wrong mode value !!!\n\r";
                                    print $msg;
                                    push(@logfile,$msg);
                                    exitMigTool (1);
                            }
            if($climodefrom =~ m/^ecnta\b/)
            {
                $climodefrom = "ECNTA";
            }
            if($climodefrom =~ m/^ecntc\b/)
            {
                $climodefrom = "ECNTC";
            }
            $dispmsg = "\n\t The FROM mode is :$climodefrom\n";
            push(@dispresult,$dispmsg);
            push(@logfile,$dispmsg);
        }

        if(($interactivemode) || ($modeoption eq "i"))
        {

                if(($cliversionto ne "R1.0") || ($cliversionto ne "R2.0") || ($cliversionto ne "R2.1") || ($cliversionto ne "R2.2") || ($cliversionto ne "R2.3"))
                {
                        print "\n\t Enter the mode to which migration has to take place\n";
            print "\n\tThe value can be ECNTA or ECNTC\n";
                        $climodeto = <STDIN>;
                        chop $climodeto;
            if(!(($climodeto =~ m/^ECNTA\b/) || ($climodeto =~ m/^ECNTC\b/) || ($climodeto =~ m/^ecnta\b/) || ($climodeto =~ m/^ecntc\b/)))
                            {
                                    $msg = "\n Wrong mode value !!!\n\r";
                                    print $msg;
                                    push(@logfile,$msg);
                                    exitMigTool (1);
                            }
            if($climodeto =~ m/^ecnta\b/)
            {
                $climodeto = "ECNTA";
            }
            if($climodeto =~ m/^ecntc\b/)
            {
                $climodeto = "ECNTC";
            }
            $dispmsg = "\n\t The TO mode is :$climodeto\n";
            push(@dispresult,$dispmsg);
            push(@logfile,$dispmsg);
            
            
                }
                else
                {       print "\n\t The TO mode is defaulted to ECNTA if the To version is < R2.4\n";
                        $climodeto ="ECNTA";
            $dispmsg = "\n\t The TO mode is :$climodeto\n";
            push(@dispresult,$dispmsg);
            push(@logfile,$dispmsg);
                }
        }

                #indu - added for cli command change [07/11/06][END]
        if($version)
        {

                if(($climodefrom ne "ECNTA") && ($climodefrom ne "ECNTC"))
                {
                        print "\nThe value of \"FromMode\" should be either \"ECNTA\" or \"ECNTC\"";
                        return -1;
                }

                if(($climodeto ne "ECNTA") && ($climodeto ne "ECNTC"))
                {
                        print "\nThe value of \"ToMode\" should be either \"ECNTA\" or \"ECNTC\"";
                        return -1;
                }


        }
 #indu - added for cli command change [07/11/06][END]

#indu - added for cli command change [07/11/06][START]
        if(($interactivemode) || ($modeoption eq "i"))
        {
        
                if(($cliversionfrom eq "R2.5") || ($cliversionfrom eq "R3.0") || ($cliversionfrom eq "R3.1") || ($cliversionfrom eq "R3.2") || ($cliversionfrom eq "R3.3"))
        {
                print "\n\t Enter the bandwidth parameter to be enabled or not. (yes / no)\n";
                $bandwidth = <STDIN>;
                chop $bandwidth;
        $dispmsg = "\n\t The bandwidth is :$bandwidth\n";
        push(@dispresult,$dispmsg);
        push(@logfile,$dispmsg);

                if (!($bandwidth eq "yes" || $bandwidth eq "no" || $bandwidth eq "YES" || $bandwidth eq "NO"))
                {
                        print "\n Bandwidth option should be yes or no \n";
                        return -1;
                }
                if(($bandwidth eq "yes") || ($bandwidth eq "YES"))
                {
                        $traceBW=1;
                }
                if(($bandwidth eq "no") || ($bandwidth eq "NO"))
                {
                        $traceBW=0;
                }
        }
        }

        #indu - added for cli command change [07/11/06][END]

#No Dual Ip Support R37[16/04/08]
my $ANSI = 0;
my $check = substr ($cliversionto ,0,1);
if($check eq "F")
{
    $ANSI =1;
}


       if((($cliversionfrom eq "R2.0") ||($cliversionfrom eq "R2.1") ||
 ($cliversionfrom eq "R2.2")) && (($cliversionto ge "R4.0") || ($ANSI eq 1)))
      {
             die "Couldn't migrate from R2.2 or earlier releases(does not have support for Single Ip) to R4.0 ";         
      }
        if ((($cliversionfrom eq "R2.3") ||($cliversionfrom eq "R2.4")
             ||($cliversionfrom eq "R2.5") ||($cliversionfrom =~ "R3"))
             && (($cliversionto ge "R4.0") || ($ANSI eq 1)) )
        {
           
               $DualIpSet = 1;
        }   
	#added for migration from R4.4 to R4.5 
	if($cliversionfrom eq "R4.4")
	{
		$cliversionfrom = "R4.3";
	}
	if($cliversionto eq "R4.4")
	{
		$cliversionto = "R4.3";
	}	
        return 0;
}

#******************************************************************************
#Routine:   scalr2table
#         FutureSoftware Limited
#Description:This Routine is used for scalar to table change conversion
#        it takes the input from the parser which is a list.
#        it will update the iss.config file ..
#Input parameters:String reference from the parser, and the main configuration 
#         list reference.
#*****************************************************************************

sub ScalarToTable
{
    my ($ScalarObjRef, $ConfigRef)=@_;
    my @ConfigFile = @{$ConfigRef};
    my @ScalarObject = @{$ScalarObjRef};
    my $ObjectCount = 0;
    my $ConfigCount = 0;
    my @TempConfigFile = "";
    my $TempCount = 0;
    my $DataType = 0;
    my $Value = 0;
    my $Flag = 0;


    for(@ScalarObject)
    {
        my @ObjectName = split(/\s+/, $ScalarObject[$ObjectCount++]);

        if($ObjectName[0] =~ /\w/)
        {
            $TempCount = 0;
            $ConfigCount = 0;
            @TempConfigFile = "";

            for(@ConfigFile)
            {
                my @FieldArray = split(/\s+/, $ConfigFile[$ConfigCount]);
                if($FieldArray[0] =~ m/^$ObjectName[0]\b/)
                {
                    $DataType = $FieldArray[1];
                    $Value = $FieldArray[2];
                }
                if($FieldArray[0] !~ m/^$ObjectName[0]\b/)
                {
                    $TempConfigFile[$TempCount++] = $ConfigFile[$ConfigCount];                   
                }
                $ConfigCount++;
            }
            @ConfigFile = @TempConfigFile;
            $ConfigCount = 0;
            $TempCount = 0;
            my $TempConfigFile = "";

            for(@ConfigFile)
            {
#        next if $ConfigFile[$ConfigCount] =~ /^\s*$/ ; #warning 
                my @CheckList = split(/\s+/, $ConfigFile[$ConfigCount]);

                if($CheckList[0] =~ m/^$ObjectName[2]\b/)
                {
                    my @IndexArray = split(/$ObjectName[2]/, $CheckList[0]);
                    my $Index = join("", @IndexArray); 
                    $Flag = 1;
                    $TempConfigFile[$TempCount++] = $ConfigFile[$ConfigCount++];
                    $TempConfigFile[$TempCount++] = $ObjectName[1].
                    $Index."\t".$DataType."\t".$Value."\n";    
                }

                if($Flag == 0)
                {                 
                    $TempConfigFile[$TempCount++] = $ConfigFile[$ConfigCount++];
                }
                $Flag = 0; 
            } 
            @ConfigFile = @TempConfigFile;
#Remove the End Of line Character,., 
            my $confsize = @ConfigFile-1;
            chomp($ConfigFile[$confsize]);
        }
    } 

    return(\@ConfigFile);
}


#***********************************************************************************************************
#Routine:MulticastIndexChange
#Description:This subroutine is used to change the multicast indices.
#Input :$refTableObjects, $refOldIndices,$numNewIndices, $refNewIndices, $refConfigFile
#OutPut :- None 
#Orgnisation :- Future Software.
#***********************************************************************************************************
use DataChangePackage;
sub MulticastIndexChange
{
#Getting all the passed parameters
    my($TableObjectsRef, $numOldIndices, $OldIndicesRef, $numNewIndices,
        $NewIndicesRef, $ConfigFileRef) = @_;

#Dereferencing the parameters

    my @TableObjects = @{$TableObjectsRef};
    my @OldIndices = @{$OldIndicesRef};
    my @NewIndices = @{$NewIndicesRef};
    my @ConfigFileArray = @{$ConfigFileRef};
    my $PackName = "DataChangePackage";
    my $count = 0;
    my $temp = 0;
    my $LineCount = 0;
    my $DiscardCount = 0;
    my @DiscardLines    = "";
    my @objectInsert    = "";
    my @tempConfigFileArray ="";
    my $matchFound                 = 0;
    my $tempConfCount              = 0;
    my $insertAfter                = "1.3.6.1.4.1.2076.74.1.10.0";
    my $dot1qStaticMulticastEntry  = "1.3.6.1.2.1.17.7.1.3.2.1.3";
    my $fsIgsVlanMcastFwdRawStatus = "1.3.6.1.4.1.2076.74.3.1.1.5";
    #Jayalk 
    my $RetVal          = '' ;

# Splitting the Old Indices parameters into its name,
# length and length present.
# Length Present 0 denotes length is not present in OID.
# 1 means Length is also present in OID.

#Jayalk: Initialization of hash vars was modified.

    my %OldIndicesLength = ();
    my %OldIndicesLenPresent = ();
    my @OldIndicesNames = "";

    for(@OldIndices)
    {
        my @SplitOldIndices = split(/\s+/, $_);
        push(@OldIndicesNames, $SplitOldIndices[0]);        
        %OldIndicesLength = (%OldIndicesLength, $SplitOldIndices[0],
            $SplitOldIndices[1]);   
        %OldIndicesLenPresent = (%OldIndicesLenPresent, $SplitOldIndices[0],
            $SplitOldIndices[2]);
    }
    shift(@OldIndicesNames);

#  Splitting the New Indices parameters into its name, length and  
#  length present.
#  Here Length present 1 indicates that length should be present in
#  the new OID.

    my @NewIndicesNames = "";
    my %NewIndicesLength = ();
    my %NewIndicesLenPresent = ();
    my %NewIndicesConvFun = ();
    my $funName = "";
    my $funref='' ;

    for(@NewIndices)
    {
        my @SplitNewIndices = split(/\s+/, $_);
        push(@NewIndicesNames, $SplitNewIndices[0]);
        %NewIndicesLength = (%NewIndicesLength, $SplitNewIndices[0], 
            $SplitNewIndices[1]);   
        %NewIndicesLenPresent = (%NewIndicesLenPresent,
            $SplitNewIndices[0],$SplitNewIndices[2]);
        %NewIndicesConvFun = (%NewIndicesConvFun, $SplitNewIndices[0],
            $SplitNewIndices[3]);
    }
    shift(@NewIndicesNames);    

#Loop for the table objects that have to be changed.
#For one table object at a time the entire conf file is searched.

    for(@TableObjects)
    {
        $LineCount=0;
        my @TabObjectName = "";
        @TabObjectName = split(/\s+/, $TableObjects[$temp++]);

        if($TabObjectName[0] =~/\w/)
        {
#    The conf file is looped from start to end searching for the
#    table object name

            for(@ConfigFileArray)
            {

#The line is split into objectname,Indices, its data type and value
# SplitConfigFields represent the split values i.e. objectname, its
# indices' values the datatype and value

                $LineCount++;
                my $ConfigFieldCount = 1;
                my @ConfigFields = split(/\s+/, $_);
                my %OldIndicesValue=();

# If the table object name mentioned in migration.dat is equal to the 
# object name read 
#from conf file only then the following changes take place

                if($ConfigFields[0] =~ m/^$TabObjectName[0]\b/)
                {
                    my $ObjectToWrite = " ";
                    my @TempObjectIndex   = split(/$TabObjectName[0]/,
                        $ConfigFields[0]);
                    my @SplitConfigFields  = split(/\./, $TempObjectIndex[1]);
                    $ObjectToWrite      = $TabObjectName[0];
                    $DiscardLines[$DiscardCount++] = $LineCount-1;

#In the below for loop the values of Old indices are read from conf file
# and are 
#stored in an associative array. 
#This value may be required for conversion from
#one index to another. If the Convert function field is NULL in
# migration.dat file
#and if the object is not removed in new indices then the same value
# is restored.

                    for(my $k=0; $k < int($numOldIndices); $k++)
                    {
                        my $temp="";        
                        if(int($OldIndicesLenPresent{$OldIndicesNames[$k]})
                            ne 0) 
                        {
                            $ConfigFieldCount = $ConfigFieldCount+1;    
                            for (my $i=0; $i <
                                int($OldIndicesLength{$OldIndicesNames[$k]});
                                $i++)
                            {
                                if($i == 0)
                                {
                                    $temp = $SplitConfigFields[$ConfigFieldCount++];
                                }   
                                else
                                {
                                    $temp= ($temp . "." . 
                                        $SplitConfigFields[$ConfigFieldCount++]);       
                                }

                            } # end for i
                        }
                        else
                        {

                            for (my $i=0; $i <
                                int($OldIndicesLength{$OldIndicesNames[$k]});
                                $i++)
                            {
                                if($i == 0)
                                {
                                    $temp = $SplitConfigFields[$ConfigFieldCount++];
                                }   
                                else
                                {
                                    $temp= ($temp . "." . 
                                        $SplitConfigFields[$ConfigFieldCount++]);       
                                }
                            }
                        } #end for k


                        %OldIndicesValue =
                        (%OldIndicesValue, $OldIndicesNames[$k], $temp);

                    }


#In this for loop the actual changes take place. It reads the migration.dat
#file and does the necessary changes.
#It also counts the lines for which there is no valid IP address found.
                    for(my $i = 0; $i < int($numNewIndices); $i++)
                    {
                        for(my $j = 0; $j < int($numOldIndices); $j++)
                        {
                            if($NewIndicesNames[$i] eq $OldIndicesNames[$j])
                            {
                                if($NewIndicesConvFun{$NewIndicesNames[$i]}  ne "NULL")
                                {
                                    if($NewIndicesLenPresent{$NewIndicesNames[$i]} eq 1 )
                                    {   
                                        $ObjectToWrite = ($ObjectToWrite . "." .
                                            $NewIndicesLength{$NewIndicesNames[$i]});
                                    }
                                    $funName  = $PackName. "::" .
                                    $NewIndicesConvFun{$NewIndicesNames[$i]};
                                    $funref = \&$funName;
                                    $RetVal = $funref->(($OldIndicesValue{$NewIndicesNames[$i]},$OldIndicesValue{$NewIndicesNames[int($i)-1]}, $IGMPXmlFilePath));

#   $RetVal = &{$PackName. "::" . $NewIndicesConvFun{$NewIndicesNames[$i]#}}(($OldIndicesValue{$NewIndicesNames[$i]},$OldIndicesValue{$NewIndicesNames#[int($i)-1]}, $IGMPXmlFilePath));

                                    if ($RetVal eq "NULL")
                                    {
                                        if ($trace == 1)
                                        {
                                            $msg = " There is no Multicast IP entry for the combination of MAC $OldIndicesValue{$NewIndicesNames[$i]} and Vlan $OldIndicesValue{$NewIndicesNames[int($i)-1]} \n";
                                            push (@logfile, $msg);  
                                        }
                                        $matchFound = -1;
                                    }
                                    $ObjectToWrite = ($ObjectToWrite . "." . $RetVal);
                                }   
                                else
                                {
                                    if($NewIndicesLenPresent{$NewIndicesNames[$i]} eq 1 )
                                    {   
                                        $ObjectToWrite = ($ObjectToWrite . "." . 
                                            $NewIndicesLength{$NewIndicesNames[$i]});
                                    }   
                                    $ObjectToWrite = ($ObjectToWrite . "." . 
                                        $OldIndicesValue{$NewIndicesNames[$i]});
                                }
                                #   break; #Jayalk :use of break is not valid due to 
                                # enabling th option 'use strict'                         
                                $j = int($numOldIndices);
                            }                                       
                        }  # end for j                                      
                    }  # end for i  
#Here substitution takes place. The object in the conf file is replaced 
# with the
#modified modified objectID. In case there is no change then the same value
#is substituted.

                    s/$ConfigFields[0]/$ObjectToWrite/gc;

#Jayalk : below if cond was modified .

                    if ($matchFound eq 0
                        &&
                        ($ConfigFields[0] =~ m/^$dot1qStaticMulticastEntry\b/ ||
                            $ConfigFields[0] =~ m/^$fsIgsVlanMcastFwdRawStatus\b/))
                    {

                        push (@objectInsert, $ConfigFileArray[$LineCount-1]);
                    }
                    $matchFound = 0;
                }
            }
        } #end if TableObjectName
    } # end for TableObject

#Here the lines whose IP multicast addresses are not found are
#discarded from the file.

    for(my $k = 0; $k < int ($DiscardCount); $k++)
    {
        splice(@ConfigFileArray, $DiscardLines[$k], 1);
        for(my $i = $k+1; $i < int($DiscardCount); $i++)
        {
            if($DiscardLines[$i] > $DiscardLines[$k])
            {
                $DiscardLines[$i] = int($DiscardLines[$i])-1;
            } 
        }
    }

    $count = 0;
    for (@ConfigFileArray)
    {

        if ($ConfigFileArray[$count] =~ m/^$insertAfter\b/)
        {
            $tempConfigFileArray[$tempConfCount++] =
            $ConfigFileArray[$count];
            for (@objectInsert)
            {
                $tempConfigFileArray[$tempConfCount++] = $_;
            }
        }
        else
        {
            $tempConfigFileArray[$tempConfCount++] =
            $ConfigFileArray[$count];
        }
        $count++;
    }

    @ConfigFileArray = @tempConfigFileArray;

#The config file array is returned to the calling function.

    my $arrayRef = \@ConfigFileArray;
    return $arrayRef;
}


#***********************************************************************************************************
#Routine:object_insert
#Description:This subroutine is used to insert an object before or after a specified object.
#Input :$list_ref, $config_ref
#OutPut :- $configref 
#Organisation :- Future Software.
#***********************************************************************************************************

sub object_insert
{
    my($RefObjectList, $RefLines)=@_;   

    my @ObjectList = @{$RefObjectList}; 
    my @Lines = @{$RefLines};
# ALU01660534    my $count = 1;
# Since parse function has been rewritten to omit Blank spaces as part of 
# MIGREL_1.65, $NumOfInserts will be at index 0.
    my $count = 0; 
    my $NumOfInserts =""; 
    my @InsertObjects = "";

    $NumOfInserts = $ObjectList[$count++];
    for (my $j = 0; $j < int ($NumOfInserts); $j++)
    {
        push(@InsertObjects, $ObjectList[$count++]);    

    }
    shift(@InsertObjects);

    if ($trace == 1)
    {

        $msg = "\n INSERTED \n";
        push (@logfile, $msg);
    }

#Here all the required information such as the the new object to be
#inserted, the object before/after which it has
#to be inserted and the value, datatype and the length present is
#gathered from the migration.dat file

    for(@InsertObjects)
    {
        my @InsertObjectFields = split(/\s+/, $_);
        my $InsObjectName = $InsertObjectFields[0];
        my $InsertBOrA = $InsertObjectFields[1];
        my $ObjectName = $InsertObjectFields[2];
        my $InsObjType = $InsertObjectFields[3];
        my $InsObjLenPresent = $InsertObjectFields[4];
        my $InsObjValue = $InsertObjectFields[5];   
        my $LineCount = 0;
        my $flag               = '' ;

        for(@Lines)
        {
            $LineCount++;
            my @LineFields = split(/\s+/, $_);

            if ($LineFields[0] =~ m/$ObjectName\b/ && $flag != 1)  
            {
                $flag = 1;

                $LineFields[0] =~ s/$ObjectName/$InsObjectName/g ;          
                $LineFields[1] = " ".$InsObjType;
                if($InsObjLenPresent ne "NULL")
                {
                    $LineFields[2] = $InsObjLenPresent; 
                    $LineFields[3] = $InsObjValue;
                }
                if($InsObjLenPresent eq "NULL")     
                {
                    $LineFields[2] = $InsObjValue;
                    splice(@LineFields, 3, 1);  
                }
                my $InsertedLine = join("\t", @LineFields); 

#Here the Object is inserted before the specified object
                if($InsertBOrA eq "BEFORE")
                {
                    splice(@Lines, $LineCount-1, 0,
                        "$InsertedLine\n");
                }

#Here the Object is inserted after the specified object

                if($InsertBOrA eq "AFTER")
                {
                    splice(@Lines, $LineCount, 0,
                        "$InsertedLine\n");
                }

                if ($trace == 1)
                {

                    $msg = " $InsertedLine\n";
                    push (@logfile, $msg);
                }

            }
            else
            {
                $flag = 0;
            }
        }
    }
    my $arrayRef = \@Lines;
    return $arrayRef;
}

# Sujithg [16/11/05] Starts

#***********************************************************************************************************
#Routine:object_insert_per_vlan
#Description:This subroutine is used to insert an object/vlan before or after a specified object.
#Input :$list_ref, $config_ref
#OutPut :- $configref 
#Organisation :- Future Software.
#***********************************************************************************************************

sub object_insert_per_vlan
{
    my ($RefObjectList, $RefLines)=@_;  
    my @ObjectList = @{$RefObjectList}; 
    my @Lines = @{$RefLines};
#    my ($NumOfInserts, $count, $Object) = (0, 1, 0);  ALU01660534 
# Since parse function has been rewritten to omit Blank spaces as part of 
# MIGREL_1.65, $Object will be at index 0.
    my ($NumOfInserts, $count, $Object) = (0, 0, 0);
    my $matchfound = 0;
    my %VlanInfo = ();
    my @InsertObjects = ();
    my $PrevObjName = "";
    my $LineCount = 0;

    #get vlans from configuration file
    $Object = $ObjectList[$count++];
    $Object =~ s/[\r\n]//g;
    for (@Lines)
    {
        if (/^$Object\.(\d+)/)
        {
            $VlanInfo{$1} = 0; #vlan $1 is found
            $matchfound = 1;
        }
        else
        {
            last if $matchfound == 1;
        }
    }
    #Add vlan 4094 info to config file if not already present
    $VlanInfo{4094} = 0; 

    #get objects to be inserted
    $NumOfInserts = $ObjectList[$count++];
    for (my $j = 0; $j < int ($NumOfInserts); $j++)
    {
        push(@InsertObjects, $ObjectList[$count++]);    
    }

    if ($trace == 1)
    {

        $msg = "\n INSERTED \n";
        push (@logfile, $msg);
    }

#Here all the required information such as the the new object to be inserted, the object before/after which it has
#to be inserted and the value, datatype and the length present is gathered from the migration.dat file
    for (@InsertObjects)
    {
        my @InsertObjectFields = split(/\s+/, $_);
        my $InsObjectName = $InsertObjectFields[0];
        my $InsertBOrA = $InsertObjectFields[1];
        my $ObjectName = $InsertObjectFields[2];
        my $InsObjType = $InsertObjectFields[3];
        my $InsObjLenPresent = $InsertObjectFields[4];
        my $InsObjValue = $InsertObjectFields[5];

        my $ResLine = [];
        my @ResArr = ();
        my $vlancnt = 0;

        unless ($PrevObjName eq $ObjectName)
        {
            $PrevObjName = $ObjectName;
            $LineCount = 0;
            for (@Lines)
            {
                $LineCount++;
                last if (/$ObjectName\b/);
            }
        }
        for my $vlan (sort keys %VlanInfo)
        {
            $ResArr[0] = "$InsObjectName.$vlan";
            $ResArr[1] = " $InsObjType";
            if($InsObjLenPresent ne "NULL")
            {
                $ResArr[2] = "$InsObjLenPresent"; 
                $ResArr[3] = "$InsObjValue\n";
            }
            else
            {
                $ResArr[2] = "$InsObjValue\n";
            }
            $$ResLine[$vlancnt] = join ("\t", @ResArr);

            $vlancnt++;
        }
        if ($InsertBOrA eq "BEFORE")
        {
            splice (@Lines, $LineCount - 1, 0, @$ResLine);
            $LineCount += $vlancnt;
        }
        if ($InsertBOrA eq "AFTER")
        {
            splice (@Lines, $LineCount, 0, @$ResLine);
        }
        if ($trace == 1)
        {
            push (@logfile, @$ResLine);
        }
    }
    return \@Lines;
}

# Sujithg Ends

#******************************************************************************
#getval :
#Description: this routine gets the value of the given tag from the given list
#if the flag = 0, search for the =(EQUALTO for the value) if flag = 1 get the 
#value which is the thirdone in the format of  "OBJECT Type Value"
#******************************************************************************
sub getval
{
    my($listref,$searchfor,$flag) = @_;
    my @contents = @{$listref};
    my $word                      = '';
    my @value                     = "";
    foreach $word (@contents)
    {

        if($word =~ $searchfor)
        {
            #value found for the given searchfor string 
            #FORMAT :    searchforsrring   = value
            if($flag == 0)
            {
                @value = split(/=/,$word);

                if ($trace == 1)
                {
                    push (@logfile, "\n ======================== \n ");
                    push (@logfile, @value);
                    #push (@logfile, "\n ".$value[1]." \n");
                    push (@logfile, " ======================== \n");
                }
                return($value[1]);
            }
            else
            {
                if($flag == 1)
                {
                    @value = split(/\s+/,$word);

                    if ($trace == 10)   # Mathewk
                    {
                        $msg = "\n333333333333333\n";
                        push (@logfile, $msg);
                        push (@logfile, $value[2]);
                        $msg = "\n333333333333333\n";
                        push (@logfile, $msg);
                    }
                    return $value[2];
                }
            }
        }

    }
}


#******************************************************************************
#ipifTableHandling:
#Description: This routine is specially added for R2.2 special requirements
#             Obsolete issDefaultIpAddrCfgMode entry but kep the value,and add
#             this value to ifIpAddrAllocMethod with index 280 datatype 2 and 
#             value of issDefaultIpAddrCfgMode at the begining of all 
#             ifIpAddrAllocMethod 's
#Inputs:  obsoleteObject,newObject,configRef. 
#******************************************************************************

sub ipifTableHandling
{
    if ($trace == 1)
    {
        $msg = "\n At ipifTableHandling *** \n";
        push (@logfile, $msg);
    }

    my ($obsoleteObject,$newObject,$configRef) = @_;

    my @config = @{$configRef};

#Get the value of issDefaultIpAddrCfgMode by calling getval() routine.
    my $defaultIpAddrCfgMode = getval($configRef,$obsoleteObject,1);
    chomp($defaultIpAddrCfgMode);

    if ($trace == 1)
    {
        $msg = "\n defaultIpAddrCfgMode value = $defaultIpAddrCfgMode \n";
        push (@logfile, $msg);
    }


#Got the issDefaultIpAddrCfgMode now Obsolete this from the config file.
    my @obsoleteList="";
    $obsoleteList[0]=$obsoleteObject;

    if ($trace == 1)
    {
        $msg = "\n Obsoleting defaultIpAddrCfgMode \n";
        push (@logfile, $msg);
    }

    $configRef =  obsolete(\@obsoleteList,$configRef);
#the issDefaultIpAddrCfgMode is Obsoleted next step is to insert the newline
#ifIpAddrAllocMethod.280   2   $defaultIpAddrCfgMode 
#before the ifIpAddrAllocMethod.
    my @lineToInsert="";
    my $insertBefore = $newObject;
    $lineToInsert[0] = "$newObject.280"."\t"."2"."\t".$defaultIpAddrCfgMode."\n";

    if ($trace == 1)
    {   
        $msg = "\n New line to be inserted is $lineToInsert[0] \n";
        push (@logfile, $msg); 
    }


    $configRef=InsertLinesRoutine($insertBefore,\@lineToInsert,$configRef,1);

    return($configRef);
}


#******************************************************************************
#nvramContent:
#Description:  issnvram.txt content is returned in a string Reference.
#******************************************************************************

sub nvramContent
{

    my $nvramFile = $issnvramFilePath; 
    my $nvramHandle = IO::File->new($nvramFile ,"r") or 
    die "Couldn't Open issnvram.txt for reading";
    my @issnvram = <$nvramHandle>;
    $nvramHandle->close();
    return \@issnvram;
}


# Mathewk [26/08/05] Starts
#******************************************************************************
# Routine     : getIntermediateDat
# Description : This subroutine is used to get lines between <VERSION_RX_RY> 
#               and </VERSION_RX_RY> tags from migration.dat file.
#               The return value serves as the dat lines for migrating
#               from version X to version Y.
# Input       : $refDatLines, $tempVersionFrom
# OutPut      : None 
# Orgnisation : Future Software.
#******************************************************************************
sub getIntermediateDat
{
    my ($refDatLines, $tempVersionFrom ) = @_;
    my @lines         = @{$refDatLines};
    my @tempArray     = "";
    my $searchBegin   = "<VERSION";
    my $searchEnd     = "</VERSION";
    my $refReturn     = -1;
    my $linesCopied   = 0;
    my $linesSearched = 0;


    for(@lines)
    {


        if ($lines[$linesSearched++] =~ m/$searchBegin/)
        {

            #  Get the <VERSION_RX_RY> tag from .dat file
            my @words   = split (/<|>/, $_);
            #  Split this line to get RX and RY
            my @version = split (/_/, $words[1]);

            chomp ($version[1]);  #  Vertion From
            chomp ($version[2]);  #  Vertion To
            chomp ($tempVersionFrom);

            # Check whether RX equals tempVersionFrom
            if ($version[1] =~ m/$tempVersionFrom/)
            {

                # Copy the lines between start and end of this version tag
                while ($lines[$linesSearched] !~ m/$searchEnd/)
                {

                    $tempArray[$linesCopied++] = $lines[$linesSearched];
                    $linesSearched++;
                }

                # Copy RY to $vertionto because $vertionto becomes 
                # $tempVersionFrom for the next intermediate migration.
                $versionto = $version[2];
                #print "\nMigrating to version $versionto from $tempVersionFrom\n";
                $msg =  "\nMigrating to version $versionto from $tempVersionFrom\n";
                push(@logfile, $msg);
                $refReturn = \@tempArray;

                return $refReturn;
            }
        }
    }

    return $refReturn;  # Returns -1 (ERROR)
}


#******************************************************************************
# Routine     : validateMigration
# Description : This subroutine is used to check whether the .conf file
#               can be migrated to specified final version.
#
#               It checks whether 'CalcIpCheckSum', 'issnvram.txt',
#               'IGMPXmlfile.txt' files are present. 
#
#               It searches for <VERSION_RX_RY> tag in .dat file. 
#               If RX equals the given '$intermediateVersions', it copies RY
#               to '$intermediateVersions'. This continus untill 
#               '$intermediateVersions' equals '$cliversionto' or 
#               it has reached end of .dat file.    
# Input       : $refDatLines, $intermediateVersions, $cliversionto
# OutPut      : None 
# Orgnisation : Future Software.
#******************************************************************************


my $retResult     = -1;
sub validateMigration
{
    my ($refDatLines, $intermediateVersions, $cliversionto) = @_;
    my @lines         = @{$refDatLines};
    my $searchBegin   = "<VERSION";
    my $linesCopied   = 0;
    my $linesSearched = 0;
    my $filepath      = '';

    $retResult     = -1;

    if (($intermediateVersions =~ m/$cliversionto/)&&($climodefrom eq $climodeto))
    {
        $msg = "\n Error: Trying to migrate to the same version \n";
        print $msg;
        push (@logfile, $msg);
        return $retResult;
    }

    # Check whether the required files are present.
    # find() function searches the directory for the given file.
    # If the file is found, it calls the subroutine 'fileFound'
    # which makes $retResult as 1 (SUCCESS).

    # Check the presence of 'CalcIpCheckSum'

    my $is_big_endian     = 0;
    my $is_little_endian  = 0;
    $is_big_endian   = unpack("h*", pack("s", 1)) =~ /01/;
    $is_little_endian = unpack("h*", pack("s", 1)) =~ /^1/;
 
    my $solaris = "solaris";
    if($is_big_endian eq 1)
    {
            print "\n Big Endian =";
            print $is_big_endian;
	    if ($^O eq $solaris)
	    {
	    $filepath = $path . "/./CalcIpCheckSum_BE" ;
    }
	    else
	    {
		    $filepath = $path . "/CalcIpCheckSum_BE_linux" ;
	    }
    }
 
    if($is_little_endian eq 1)
    {
            print "\n Little Endian =";
            print $is_little_endian ;
	    if ($^O eq $solaris)
	    {
	    $filepath = $path . "/CalcIpCheckSum_LE" ;
    }
	    else
	    {
		    $filepath = $path . "/CalcIpCheckSum_LE_linux" ;
	    }
    }

    #$filepath = $path . "/CalcIpCheckSum" ;
    find (\&fileFound, $filepath);

    if ($retResult eq -1)
    {

        $msg = "\n Error: 'CalcIpCheckSum' File Not Found \n";
        print $msg;
        push (@logfile, $msg);
        return $retResult;
    }

    # Check whether 'issnvram.txt" is present in the same path
    # as the input iss.conf file, if we are migrating from
    # R2.1 or less to a version 2.2 and above.

    if (($cliversionfrom eq "R1.0" ||
            $cliversionfrom eq "R2.0" ||
            $cliversionfrom eq "R2.1")
        &&
        (not ($cliversionto eq "R2.0" ||
                $cliversionto eq "R2.1")))
    {

        $retResult     = -1;
        find (\&fileFound, $issnvramFilePath);
        if ($retResult eq -1)
        {

            $msg = "\n Error: '$issnvramFilePath' File Not Found \n";
            print $msg;
            push (@logfile, $msg);
            return $retResult;
        }
    }

    # Here we check whether the final version can be reached.
    $retResult = -1;
    for (@lines)
    {

        if ($lines[$linesSearched++] =~ m/$searchBegin/)
        {

            my @words   = split (/<|>/, $_);
            my @version = split(/_/,$words[1]);

            chomp ($version[1]);  # Vertion From
            chomp ($version[2]);  # Vertion To
            chomp ($intermediateVersions);

            # Check whether 'IGMPXmlfile.txt" is present in the 
            # same path as the input iss.conf file. This file is
            # needed to convert L2 Multicast entries in the R2.2
            # 'iss.conf' file to L3 Multicast entires in R2.3 

            if ($intermediateVersions eq "R2.2" && $cliversionto ne "R2.2") 
            {

                find (\&fileFound, $IGMPXmlFilePath);
                if ($retResult eq -1)
                {

                    $msg = "\n Error: '$IGMPXmlFilePath' File Not Found \n";
                    print $msg;
                    push (@logfile, $msg);
                    return $retResult;
                }
            }

            if ($version[1] eq $intermediateVersions)
            {
                if($version[1] eq "R4.0")
                {
                    my $Char = substr ($cliversionto, 0,1);
                    if($Char eq "F")
                    {
                        $intermediateVersions = "F4.0";
                    }
                    else
                    {
                        $intermediateVersions = $version[2];
                    }
                }
                else
                {
                $intermediateVersions = $version[2];
            }
            }

            if ($intermediateVersions eq $cliversionto)
            {

                $retResult = 1;
                last;
            }
        }
    }

    if (($climodefrom eq "ECNTC")&&($climodeto eq "ECNTA"))
    {
        print "\n Migration is not possible from board type ECNT-C to board type ECNT-A\n";
        return -1;
    }


    if(($climodeto eq "ECNTC") && (($cliversionto eq "R1.0")||($cliversionto eq "R2.0")||($cliversionto eq "R2.1")||
    ($cliversionto eq "R2.2")||($cliversionto eq "R2.3")||
        ($cliversionto eq "R2.4.15")||($cliversionto eq "R2.4.16")))
    {
        print "\nMigration cannot be performed from $cliversionfrom $climodefrom to $cliversionto $climodeto\n";
        return -1;
    }





    if ($retResult eq -1)
    {

        $msg = "\n Error: Sufficent information is not given
        in .dat file \n";
        print $msg;
        push (@logfile, $msg);
    }

    return $retResult;
}


#******************************************************************************
# Routine     : fileFound 
# Description : This subroutine is the callback function of
#               find(). Just makes the value of $retResult as 1 (SUCCESS) 
# Input       : None 
# OutPut      : None 
# Orgnisation : Future Software.
#******************************************************************************
sub fileFound
{

    $retResult = 1;
}


#******************************************************************************
# Routine     : exitMigTool
# Description : This subroutine performs shutdown functionalities 
#               for migration tool. It creates the log file for the whole
#               migration process and exit.
# Input       : $exitStatus
# OutPut      : None 
# Orgnisation : Future Software.
#******************************************************************************
sub exitMigTool
{
    my ($exitStatus) = @_;

    if ($exitStatus == 0)
    {
        $msg = "\n Migration Successful \n\n\tMIGRATED FILE IS :$newconfigfile \n";
        print $msg;
        push (@logfile, $msg);
        print @dispresult;
    }
    else
    {

        $msg = "\n Migration Failed \n";
        print $msg;
        push (@logfile, $msg);
    }

    # @logfile contains the status of migration. Before exiting,
    # contents of @logfile is written into the file specified by user

    #  Jayalk 

        ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
    my $tempp = "";
    $tempp=sprintf ("%02d%02d%4d_%02d%02d%02d\n",$mon+1,$mday,$year+1900,$hour,$min,$sec);
    $dispmsg = "\n\t The logfile is :$miglogfile\n";


    my $fileHandlelog = IO::File -> new ($miglogfile,"w") or 
    die " Could not create log file \n";
    print $fileHandlelog @logfile;
    $fileHandlelog -> close();
        

    # Jayalk
    $dispmsg = "\n\t The logfile is :$miglogfile \n";
        push(@dispresult,$dispmsg);
    push(@logfile,$dispmsg);

    exit ($exitStatus);
}



#******************************************************************************
# Routine     : FunctionForR2_4 
# Description : This subroutine performs functions specific for R2.4 
#               In R2.4, a new table issOpticalPortCtrltable is added
#               inside issConfigControlgroup. This functon finds the
#               number of ports and sets the default value for the new
#               mib objects for each port.
# Input       : Reference to Configuration file 
# OutPut      : None 
# Orgnisation : Future Software.
#******************************************************************************


sub FunctionForR2_4
{

# Jayalk : As IssPortCtrlTable is not in Use 
# The following code is disabled 

    my ($refConfigFile) = @_;
    my @lines         = @{$refConfigFile};
    # Jayalk
    my $searchSingleIP = "1.3.6.1.4.1.2076.81.1.53.0" ;
    my $i            = 0;



#    my @insertString  = "";
#    my $searchString  = "1.3.6.1.4.1.2076.81.2.2.1.2";
#    my $Opt1GAutoNeg  = "1.3.6.1.4.1.2076.81.2.3.1.2";
#    my $insertAfter   = "1.3.6.1.4.1.2076.81.2.2.1.10";
#    my $command       = "";
#    my $maxPorts     = 1;
#    my $j            = 0;

    # Mathewk [02-02-06] Starts 
    # We search for the index of $insertAfter which extends to 
    # the number of ports present. 
#    for (@lines)
#    {
#        if ($lines[$i] =~ m/^$insertAfter\b/)
#        {
#            my @fields = split (/\s+/, $lines[$i]);
#            my @temp   = split (/\./, $fields[0]);
    #
#            if ($temp[12] > $maxPorts)
#            {
    #
#                $maxPorts = $temp[12];
#            }
#        }
#        $i++;
#    }
#    $i           = 0;
#    $insertAfter = $insertAfter. "." . $maxPorts;
    # Number of entries between <INSERT> tag is equal to the number
    # of ports present   
#    push (@insertString, $maxPorts);
    # Mathewk [02-02-06] Ends 


    for (@lines)
    {

        # Mathewk [02-02-06] Starts  
        # If issPortCtrlMode != Auto then
        #     set issOpticalPortCtrlMode1GAutoNegotiation = off
        # else
        #     set issOpticalPortCtrlMode1GAutoNegotiation = on

#        if ($lines[$i] =~ m/^$searchString\b/)
#        {
        #
#            my @fields = split (/\s+/, $lines[$i]);
#            my @temp   = split (/\./, $fields[0]);

        # AutoMode == 1 
#            if ($fields[2] != 1)
#            {
        #
#                 # set issOpticalPortCtrlMode1GAutoNegotiation = off
#                 $command = $Opt1GAutoNeg.".".$temp[12]. " AFTER $insertAfter " . "2 NULL 2\n";
#            }
#            else
#            {
        #
#                 # set issOpticalPortCtrlMode1GAutoNegotiation = on
#                 $command = $Opt1GAutoNeg.".".$temp[12]. " AFTER $insertAfter " . "2 NULL 1\n";
#            }
#            push (@insertString, $command);
#        }
        # Mathewk [02-02-06] Ends 




        #Jayalk
        if ($lines[$i] =~ m/^$searchSingleIP\b/)
        {

            my @fields = split (/\s+/, $lines[$i]);
            my $Position  ; 
            my $temp     =  0;
            my @DeleteObject =  $fields[0] ;
            my @newstring = "" ;
            push(@newstring , $lines[$i]);
            my $Insertline = \@newstring ;

            #  Jayalk:currently  Top obj is "1.3.6.1.4.1.2076.81.1.34.0"
            #  Now trying to Insert Single IP status  at the Top of conf .

            $Position = "TOP"; 
            $configref = obsolete (\@DeleteObject,$configref );
            $configref = InsertLinesRoutine ($Position, $Insertline,
                $configref, $temp);

            $msg = "\n Single IP status was inserted at the top  \n";
            push (@logfile, $msg);
        }

        $i++;
    }   

    # Normally in .dat file, we give the strings to be inserted
    # between <INSERT> tag. But since the number of ports vary, 
    # we can not give them between <INSERT> tag in .dat file. So we add 
    # new mibs to be inserted to an array @insertString, in the same format
    # as we give in .dat file. This array is passed to the function 
    # object_insert () which performs insertion.

#    my $listref = \@insertString;
#    $configref  = object_insert ($listref, $configref);


    return $configref; 

}



#******************************************************************************
# Routine     : FunctionForR2_5 
# Description : This subroutine performs functions specific for R2.4 
#               In R2.4, a new table issOpticalPortCtrltable is added
#               inside issConfigControlgroup. This functon finds the
#               number of ports and sets the default value for the new
#               mib objects for each port.
# Input       : Reference to Configuration file 
# OutPut      : None 
# Orgnisation : Future Software.
#******************************************************************************
sub FunctionForR2_5
{

    my ($refConfigFile) = @_;

    my @lines         = @{$refConfigFile};
    my $vlanEgressPorts = "1.3.6.1.2.1.17.7.1.4.3.1.2";
    my $i = 0;
    my $port17Count = 0;


    for (@lines)
    {
        if ($lines[$i] =~ m/^$vlanEgressPorts\b/)
        {

            my ($vlan, $portlist) =  ($lines[$i] =~ /$vlanEgressPorts\.(\d+)\s+\d+\s+\d+\t+(.*)/);
            my @b = split (//, $portlist);
#           print "vlan:$vlan  \tportlist:$portlist \tb1:".ord("$b[0]")." \tb2:".ord("$b[1]")." \tb3:".ord("$b[2]")."\n";
            if (ord($b[2]) >= 128) {
                $port17Count ++;

                if ($port17Count > 3) {
                    printf "R2_5 WARNING: port17 will be removed from egressPortList of vlan:$vlan\n";
                }
            }
        }
        $i++;
    }

    return 1;
}


# Mathewk [26/08/95] Ends 

# <vikash>
#******************************************************************************
# Routine     : FunctionForR3_0 
# Description : This subroutine displays the warning in various cases 
# Input       : Reference to Configuration file 
# OutPut      : None 
# Orgnisation : Future Software.
#******************************************************************************
sub FunctionForR3_0
{
    my ($refConfigFile) = @_;

    my @lines         = @{$refConfigFile};
    my $vlanEgressPorts = "1.3.6.1.2.1.17.7.1.4.3.1.2";
    my $i = 0;
    my $port17Count = 0;


    for (@lines)
    {
        if ($lines[$i] =~ m/^$vlanEgressPorts\b/)
        {

            my ($vlan, $portlist) =  ($lines[$i] =~ /$vlanEgressPorts\.(\d+)\s+\d+\s+\d+\t+(.*)/);
            my @b = split (//, $portlist);
#           print "vlan:$vlan  \tportlist:$portlist \tb1:".ord("$b[0]")." \tb2:".ord("$b[1]")." \tb3:".ord("$b[2]")."\n";
            if (ord($b[2]) >= 128) {
                $port17Count ++;

                if ($port17Count > 3) {
                    printf "R2_5 WARNING: port17 will be removed from egressPortList of vlan:$vlan\n";
                }
            }
        }
        $i++;
    }


}



#</vikash> 

# Sujithg [16/11/05] Starts
#******************************************************************************
# Routine     : FunctionForR3_3 
# Description : This subroutine performs functions specific for R3.0
#               For each vlan having router port configured (static) in R2.4
#               enable querier for that vlan and make query interval same as
#               that of router port interval (in R2.4) for that vlan in R3.0
# Input       : Reference to Configuration file 
# OutPut      : None 
# Orgnisation : Future Software.
#******************************************************************************
sub FunctionForR3_3
{
    my ($refConfigFile) = @_;
    my @Lines = @$refConfigFile;
    my ($matchfound, $cnt, $linecnt) = (0, 0, 0);
    my $vlansearch = "1.3.6.1.4.1.2076.74.3.2.1.4";
    my %Vlans = ();
    my @ResArr1 = ();
    my @ResArr2 = ();
    my $ResCnt = 0;

    #search for vlans having statically configured router port and record the router port interval
    for (; $linecnt <= $#Lines; $linecnt++)
    {
        my @Fields = split /\s+/, $Lines[$linecnt];
        if ($Fields[0] =~ /^$vlansearch\.(\d+)\.\d+/)
        {
            #fsIgsVlanRouterPortQueryTimer for vlan $1 has been defined, so record the interval
            $Vlans{$1} = [1, $Fields[2]];
            $cnt++;
            $matchfound = 1;
        }
        else
        {
            last if $matchfound == 1;
        }
    }
    #remove the statically configured router port interval entries
    splice (@Lines, $linecnt - $cnt, $cnt);
    #adjust the offset after removing the entries
    $linecnt -= $cnt;

    #for each (fsIgsVlanRouterPortQueryTimer.<VlanId>)
    #{
    #   fsIgsVlanQuerier.<VlanId> = 1
    #   fsIgsVlanQueryInterval.<VlanId> = fsIgsVlanRouterPortQueryTimer.<VlanId>
    #}
    for my $vlanid (sort keys %Vlans)
    {
        $ResArr1[$ResCnt] = "1.3.6.1.4.1.2076.74.3.3.1.7.$vlanid\t 2\t$Vlans{$vlanid}->[0]\n";
        $ResArr2[$ResCnt++] = "1.3.6.1.4.1.2076.74.3.3.1.8.$vlanid\t 2\t$Vlans{$vlanid}->[1]\n";
    }
    push (@ResArr1, @ResArr2);

    for (; $linecnt <= $#Lines; $linecnt++)
    {
        my $search = "1.3.6.1.4.1.2076.74.3.3.1.9";
        if ($Lines[$linecnt] =~ /^$search/)
        {
            last;
        }
    }
    splice (@Lines, $linecnt, 0, @ResArr1);

    return \@Lines;
}


# Sujithg Ends
# Kishore [16/10/06] Starts
#******************************************************************************
# Routine     : FunctionForR3_3ph2 
# Description : This subroutine performs functions specific for R3.3ph2
#       vlan mode RB and epon user to user communication is enale and delete the IP interfaces keep one.    
# Input       : Reference to Configuration file 
# OutPut      : None 
# Orgnisation : Future Software.
#******************************************************************************
sub FunctionForR3_3ph2
{
    my ($refConfigFile) = @_;
    my @Lines = @$refConfigFile;
    my ($matchfound, $cnt, $linecnt) = (0, 0, 0);
    my $Vlans ;
#   my $ResCnt = 0;
        my @Data;
        my @TempFileArray ;
        my $i              = 0;     
    #search for vlans having mode epon
    for (; $linecnt <= $#Lines; $linecnt++)
    {
           my $vlansearch = "1.3.6.1.4.1.2076.65.1.18.1.1"; 
               my @Fields = split /\s+/, $Lines[$linecnt];
            if ($Fields[0] =~ m/^$vlansearch\b/)
        {
                  if ($Fields[2]==9 or $Fields[2]==8)
                    {  
                        @TempFileArray = split (/$vlansearch/, $Fields[0]);
                $Data[$i++]  = $TempFileArray[1];
                    }   
            }
        }
 #searching mode    
       my $size  = @Data;
       my $insertBefore = "1.3.6.1.4.1.2076.65.1.18.1.3";
       my @lineToInsert="";
    
    for (my $count =0; $count <$size; $count++)
       {
        $lineToInsert[$count] = "$insertBefore"."$Data[$count]\t 2\t 1\n";
       }

       my $i=0; 
    for (; $i <= $#Lines; $i++)
    {
      my $search = "1.3.6.1.4.1.2076.65.1.20";
      #print "out side loop\n";
      if ($Lines[$i] =~ /^$search/)
        {
            last;
                        #print "if loop$i\n";
        }
    }
        

       splice (@Lines,$i, 0, @lineToInsert); 
#for removing the IP Interfaces more than one.
    my ($matchfound, $cnt, $linecnt) = (0, 0, 0);
    my %Vlanindexinfo = ();
    my @ResArr1 = ();
    my @ResArr2 = ();
#   my $ResCnt = 0;
        my @info = ();
        my %Vlanindexinfo1 = ();
        my %Vlanindexinfo2 = ();
        #find the vlanid for the index and map the index and the Vlanid 
        my $linecnt1=0;
        for($linecnt1=0; $linecnt1 <= $#Lines; $linecnt1++)
    {       my $v="vlan";
                my $Tofindvlan="1.3.6.1.2.1.31.1.1.1.18";
            my @Fields = split /\s+/, $Lines[$linecnt1];    
               if ($Fields[0]=~/^$Tofindvlan\.(\d+)/) 
        {
                my @vlanid = split (/$v/, $Fields[3]);  
                        $Vlanindexinfo{$1} = $vlanid[1]; #vlan $1 is found
            $matchfound = 1;
        }
        else
        {
            last if $matchfound == 1;
        }
          }
          
       #for the vlan find the mode type and enable the flag
         my $linecnt2=0;
         $matchfound=0;
        for ($linecnt2=0; $linecnt2 <= $#Lines; $linecnt2++)
       {
           my $vlanmode="1.3.6.1.4.1.2076.65.1.18.1.1";
           my @Fields = split /\s+/, $Lines[$linecnt2];
           if ($Fields[0] =~ m/^$vlanmode\.(\d+)\b/)
           {
             my $x=$1;
             my $y = 0;
             my $z=0;
             for my $index (sort keys %Vlanindexinfo)
             {
             my $x=$1;
             if($Vlanindexinfo{$index}==$x)
               {
                if($Fields[2]==8){
                $Vlanindexinfo1{$index}=1;
                $matchfound =1;
                #print "In mode--$index $1\n"; 
                }
               else {  
                 $Vlanindexinfo1{$index}=0;
             last if $matchfound == 1;
                 } 
             }
            }
          } 
    }
    
    #search for vlansindex  having l3 enabled and the enable the flag.
        my $x=$1;
         for my $index (sort keys %Vlanindexinfo)
         {
          $matchfound =0;
        for ($linecnt=0; $linecnt <= $#Lines; $linecnt++)
        {
#                  my @Fields = split /\s+/, $Lines[$linecnt];
                  my @Fields = split /\s+/, $Lines[$linecnt];
                  my $vlansearchindex = "1.3.6.1.4.1.2076.27.1.4.1.2";
                 if ($Fields[0]=~/^$vlansearchindex\.(\d+)\b/) 
                     {
                        $x=$1;
                        if  ($Fields[2]==136 and $x==$index) {
                             $Vlanindexinfo2{$index} = 1; #vlan $1 is found
                             $matchfound = 1;
                             #print "interface index $x and $index\n"; 
                             }
                   } 
                   else {
                               #$Vlanindexinfo2{$index}=0;
                            last if $matchfound == 1;
                           }
             
            }
         }
      
        my @Finalindex = ();  
        my $i=0;
        $matchfound =0;
    foreach my $index (sort keys %Vlanindexinfo)
        {
    foreach my $index (sort keys %Vlanindexinfo1)
        {
    foreach my $index (sort keys %Vlanindexinfo2) {
           $matchfound=0; 
            if (($Vlanindexinfo1{$index}==1) and ($Vlanindexinfo2{$index}==1))
            {
              $Finalindex[$i]=$index;  
               $i++;
              $matchfound == 1;
            }
        else{                       
             last if $matchfound == 1;
            }
            } 

        }
       }
     my %seen = ();
     @Finalindex= grep {!$seen{$_}++} @Finalindex;
   $i=1;
   for(@Finalindex){
      if($Finalindex[$i]){
    $ResArr2[$i] = "1.3.6.1.4.1.2076.27.1.4.1.4.$Finalindex[$i]\n";
      $i++;
        }
   }
   
   $i=1;
   for(@Finalindex){
   if($Finalindex[$i]){     
    $ResArr1[$i] = "1.3.6.1.4.1.2076.27.1.4.1.7.$Finalindex[$i]\t 2\t 6\n";
     print"\n The following interface index is deleted\n";
     print"\n $Finalindex[$i]\n"; 
     $i++;
        }
      }         
    print "--$Finalindex[0]-$Finalindex[1]-$Finalindex[2]-$Finalindex[3]-$Finalindex[4]\n";
    splice (@Lines, $#Lines, 0, @ResArr1);
    #change of enum for the interface down. 
    my $count = 0;
    my $temp1 = 1;
    for(@ResArr2)
    {
        my @fieldArray = split (/\s+/, $ResArr2[$temp1++]);
        my $objectName = $fieldArray[0];
        my $newEnumValue = 2;

        $count = 0; 
        if( ($objectName ) && ( $objectName =~ /\w/) ) #warning 
        {
            for(@Lines)
            {
                my @fields = split (/\s+/, $Lines[$count]);
                if(($fields[0] =~ m/^$objectName\b/))
                {
                   my $Newline = $fields[0]."\t ".$fields[1]."\t".$newEnumValue."\n";
                  $Lines[$count] = $Newline ;

                }
               $count++;
            }
        }
    }



       return \@Lines;
}
# kishore end


#indu - for issSwitchMacMode[22/11/06][START]
sub FunctionForR3_3macmode
{
    if(($interactivemode)||($modeoption eq "i"))
    {   
        
        print "\n\t Enter the climacmode option\n";
        print "\n\t Enter 1 for native mode and 2 for legacy mode\n";
        $climacmode = <STDIN>; 
        chop $climacmode;
        if(!(($climacmode =~ m/^1\b/)|| ($climacmode =~ m/^2\b/)))
        {
            $msg = "\n Wrong mode option!!!";
            print $msg;
            push(@logfile,$msg);
            exitMigTool (1);    
        }   
        $dispmsg = "\n\t The macmode option is :$climacmode\n";
        push (@dispresult,$dispmsg);
        push (@logfile,$dispmsg);
    }
    
    if(($filemode) || ($modeoption eq "f"))
    {
        if(($macmode =~ m/^Legacy\b/) || ($macmode =~ m/^LEGACY\b/))
        {
            $climacmode = 2;
        }
        if(($macmode =~ m/^Native\b/) || ($macmode =~ m/^NATIVE\b/))
        {
            $climacmode = 1;
        }
                if (!($macmode =~ m/^Legacy\b/ || $macmode =~ m/^Native\b/ || $macmode =~ m/^LEGACY\b/ || $macmode =~ m/^NATIVE\b/))
                {
                        print "\n Macmode should be legacy or native \n";
            exitMigTool(1);
                }
    }
        my ($refConfigFile) = @_;
        my @lines         = @{$refConfigFile};
        my @searchswitchmacmode = "";
    $searchswitchmacmode[1] = "1.3.6.1.4.1.2076.81.1.56.0"."\t "."2"."\t".$climacmode."\n";
    my $i=0;

    my $currentpos;
    my $temp     =  0;
        my @newstring = "" ;
        my $Insertline = \@newstring ;


        $currentpos = "TOP";
        $configref = InsertLinesRoutine ($currentpos, \@searchswitchmacmode,
                                $configref, $temp);

        $msg = "\n Mac mode inserted at the top  \n";
        push (@logfile, $msg);
        $i++;
    return $configref;

}
#indu - for issSwitchMacMode[22/11/06][END]




#Karthikeyan.T - for Vlan 4093 Interface ID rename[14/06/07][START]
sub FunctionFor3_5
{
    if(($interfaceBase eq "") && (($interactivemode)||($modeoption eq "i")))
    {   
        
        print "\n\t Enter the Interface Base option\n";
        print "\n\t Enter a value betwee 1 - 448 (Default = 256)\n";
        $interfaceBase = <STDIN>; 
        chop $interfaceBase;
        if(($interfaceBase < 1) || ($interfaceBase > 448))
        {
                        print "\n InterfaceBase should range between 1-448 \n";
            exitMigTool(1); 
        }   
        $dispmsg = "\n\t The InterfaceBase is :$interfaceBase\n";
        push (@dispresult,$dispmsg);
        push (@logfile,$dispmsg);
    }

    if(($filemode) || ($modeoption eq "f"))
    {
        if(($interfaceBase < 1) || ($interfaceBase > 448))
        {
                        print "\n InterfaceBase should range between 1-448 \n";
            exitMigTool(1);
        }
    }
    my (@lines) = "";
    my $InterfaceIndex279=$interfaceBase+23;
    my $InterfaceIndex280=$interfaceBase+24;
#ifMainTable
    $lines[0] = "1.3.6.1.4.1.2076.27.1.4.1.4.279 1.3.6.1.4.1.2076.27.1.4.1.4.".($InterfaceIndex279);
    $lines[1] = "1.3.6.1.4.1.2076.27.1.4.1.7.279 1.3.6.1.4.1.2076.27.1.4.1.7.".($InterfaceIndex279);

    $lines[2] = "1.3.6.1.4.1.2076.27.1.4.1.2.280 1.3.6.1.4.1.2076.27.1.4.1.2.".($InterfaceIndex280);
    $lines[3] = "1.3.6.1.4.1.2076.27.1.4.1.3.280 1.3.6.1.4.1.2076.27.1.4.1.3.".($InterfaceIndex280);
    $lines[4] = "1.3.6.1.4.1.2076.27.1.4.1.4.280 1.3.6.1.4.1.2076.27.1.4.1.4.".($InterfaceIndex280);
    $lines[5] = "1.3.6.1.4.1.2076.27.1.4.1.6.280 1.3.6.1.4.1.2076.27.1.4.1.6.".($InterfaceIndex280);
    $lines[6] = "1.3.6.1.4.1.2076.27.1.4.1.7.280 1.3.6.1.4.1.2076.27.1.4.1.7.".($InterfaceIndex280);
#ifIpTable  
    $lines[7] = "1.3.6.1.4.1.2076.27.1.5.1.1.280 1.3.6.1.4.1.2076.27.1.5.1.1.".($InterfaceIndex280);
    $lines[8] = "1.3.6.1.4.1.2076.27.1.5.1.5.280 1.3.6.1.4.1.2076.27.1.5.1.5.".($InterfaceIndex280);
    $lines[9] = "1.3.6.1.4.1.2076.27.1.5.1.6.280 1.3.6.1.4.1.2076.27.1.5.1.6.".($InterfaceIndex280);
#ifIvrTable
    $lines[10] = "1.3.6.1.4.1.2076.27.1.8.1.1.279 1.3.6.1.4.1.2076.27.1.8.1.1.".($InterfaceIndex279);

    $lines[11] = "1.3.6.1.4.1.2076.27.1.8.1.1.280 1.3.6.1.4.1.2076.27.1.8.1.1.".($InterfaceIndex280);
#ifMultiNetIpTable
    $lines[12] = "1.3.6.1.4.1.2076.27.1.11.1.2.280 1.3.6.1.4.1.2076.27.1.11.1.2.".($InterfaceIndex280);
    $lines[13] = "1.3.6.1.4.1.2076.27.1.11.1.4.280 1.3.6.1.4.1.2076.27.1.11.1.4.".($InterfaceIndex280);
#ifXTable
    $lines[14] = "1.3.6.1.2.1.31.1.1.1.14.279 1.3.6.1.2.1.31.1.1.1.14.".($InterfaceIndex279);
    $lines[15] = "1.3.6.1.2.1.31.1.1.1.16.279 1.3.6.1.2.1.31.1.1.1.16.".($InterfaceIndex279);
    $lines[16] = "1.3.6.1.2.1.31.1.1.1.18.279 1.3.6.1.2.1.31.1.1.1.18.".($InterfaceIndex279);
    
    $lines[17] = "1.3.6.1.2.1.31.1.1.1.14.280 1.3.6.1.2.1.31.1.1.1.14.".($InterfaceIndex280);
    $lines[18] = "1.3.6.1.2.1.31.1.1.1.16.280 1.3.6.1.2.1.31.1.1.1.16.".($InterfaceIndex280);
    $lines[19] = "1.3.6.1.2.1.31.1.1.1.18.280 1.3.6.1.2.1.31.1.1.1.18.".($InterfaceIndex280);
#fsIpifTable
    $lines[20] = "1.3.6.1.4.1.2076.2.1.23.1.2.279 1.3.6.1.4.1.2076.2.1.23.1.2.".($InterfaceIndex279);
    $lines[21] = "1.3.6.1.4.1.2076.2.1.23.1.3.279 1.3.6.1.4.1.2076.2.1.23.1.3.".($InterfaceIndex279);
    $lines[22] = "1.3.6.1.4.1.2076.2.1.23.1.4.279 1.3.6.1.4.1.2076.2.1.23.1.4.".($InterfaceIndex279);
    $lines[23] = "1.3.6.1.4.1.2076.2.1.23.1.5.279 1.3.6.1.4.1.2076.2.1.23.1.5.".($InterfaceIndex279);
    $lines[24] = "1.3.6.1.4.1.2076.2.1.23.1.6.279 1.3.6.1.4.1.2076.2.1.23.1.6.".($InterfaceIndex279);
    $lines[25] = "1.3.6.1.4.1.2076.2.1.23.1.7.279 1.3.6.1.4.1.2076.2.1.23.1.7.".($InterfaceIndex279);
    
    $lines[26] = "1.3.6.1.4.1.2076.2.1.23.1.2.280 1.3.6.1.4.1.2076.2.1.23.1.2.".($InterfaceIndex280);
    $lines[27] = "1.3.6.1.4.1.2076.2.1.23.1.3.280 1.3.6.1.4.1.2076.2.1.23.1.3.".($InterfaceIndex280);
    $lines[28] = "1.3.6.1.4.1.2076.2.1.23.1.4.280 1.3.6.1.4.1.2076.2.1.23.1.4.".($InterfaceIndex280);
    $lines[29] = "1.3.6.1.4.1.2076.2.1.23.1.5.280 1.3.6.1.4.1.2076.2.1.23.1.5.".($InterfaceIndex280);
    $lines[30] = "1.3.6.1.4.1.2076.2.1.23.1.6.280 1.3.6.1.4.1.2076.2.1.23.1.6.".($InterfaceIndex280);
    $lines[31] = "1.3.6.1.4.1.2076.2.1.23.1.7.280 1.3.6.1.4.1.2076.2.1.23.1.7.".($InterfaceIndex280);
#fsIpArpProxyTable  
    $lines[32] = "1.3.6.1.4.1.2076.2.2.16.1.3.279 1.3.6.1.4.1.2076.2.2.16.1.3.".($InterfaceIndex279);

    $lines[33] = "1.3.6.1.4.1.2076.2.2.16.1.3.280 1.3.6.1.4.1.2076.2.2.16.1.3.".($InterfaceIndex280);

    my $listref = \@lines;
    $configref = renamed($listref,$configref);
    
    
{
    my $ObjectTobeMoved   = "1.3.6.1.4.1.2076.151.1.1.1."; 
    my $InsertBefore      = "1.3.6.1.4.1.2076.152.3.1." ; 
    $configref  = Alter_Object_Position($ObjectTobeMoved, $InsertBefore ,$configref);
}


    return $configref;
}
#Karthikeyan.T - for Vlan 4093 Interface ID rename[14/06/07][END]

#Jayalk [ANTms40388] [10Apr2007]
#******************************************************************************
# Routine     : IssEgresRateChangeforR2_1 
# Description : This subroutine performs the Egress Rate changes made  
#               from R2.0 to R2.1  while migrating to R2.1 .
# Input       : Reference to Configuration file 
# OutPut      : Reference to Configuration file 
# Orgnisation : Aricent 
#******************************************************************************

sub IssEgressRateChangeforR2_1
{
    my ($refConfigFile)=@_;
    my @lines         = @{$refConfigFile};
    my $IssEgressRate  = "1.3.6.1.4.1.2076.90.2.1.8";
        my $index = 0;
   
        $msg = "\n R2.1 IssEgressRate Changes Start ....... ";
    push (@logfile, $msg);
    
    for (; $index <= $#lines; $index++)
    {
        # Search for Egress Rate  
        my @fields = split (/\s+/, $lines[$index]);

        if ( $fields[0] =~ m/^$IssEgressRate/ )
    {
           $msg = "\nEgressRate for $fields[0] :  Old-Value = $fields[2] ,";
       push (@logfile, $msg);
        
           if( ($fields[2] == 999)  ||  ($fields[2] == 1000 ) )
           {
                 $fields[2] = 16383;   # i.e 1Gbps with a Granularity of 64  
           }
           else
           {
           # In R2.0 The value being set in HW  =  Set-Value * 1024 
           # In R2.1 The Value being set in HW  =  Set-Value * 64 
           # So To make the HW values compatible from R2.0 to R2.1 the
           # Conversion Rule could be  (r2.0-Value) * 1024 /64 .            

                $fields[2] = $fields[2] * 16  ;

           }
            my $Newline = $fields[0]."\t ".$fields[1]."\t ".$fields[2]."\n" ;
            $lines[$index] = $Newline;    
            $msg = "  New-Value = $fields[2] ";
            push (@logfile, $msg);

        }
    

    } 
    $msg = "\n R2.1 IssEgressRate Changes End  ";
    push (@logfile, $msg);

    my $refReturn = \@lines;
    return $refReturn;


}

# fix for ANTmt45471 starts here
sub EgressRateChangeforR3_7
{
    my ($refConfigFile)=@_;
    my @lines         = @{$refConfigFile};
    my $IssEgressRate  = "1.3.6.1.4.1.2076.90.2.1.8";
        my $index = 0;
   
        $msg = "\n R2.1 IssEgressRate Changes Start ....... ";
    push (@logfile, $msg);
    
    for (; $index <= $#lines; $index++)
    {
        # Search for Egress Rate  
        my @fields = split (/\s+/, $lines[$index]);

        if ( $fields[0] =~ m/^$IssEgressRate/ )
    {
           $msg = "\nEgressRate for $fields[0] :  Old-Value = $fields[2] ,";
       push (@logfile, $msg);
        
           if ($fields[2] == 16383) 
           {
                 $fields[2] = 16384;   # i.e 1Gbps with a Granularity of 64  
           }
          
            my $Newline = $fields[0]."\t ".$fields[1]."\t ".$fields[2]."\n" ;
            $lines[$index] = $Newline;    
            $msg = "  New-Value = $fields[2] ";
            push (@logfile, $msg);

        }
    

        } 
    $msg = "\n R3.7 IssEgressRate Changes End  ";
    push (@logfile, $msg);

    my $refReturn = \@lines;
    return $refReturn;


}

# fix for ANTmt45471 ends here
# Jayalk [27-Jul-2006]  Start:  */

#******************************************************************************
# Routine     : IssPortCtrlTableForR2_1 
# Description : This subroutine performs the IssPortCtrlTable Change made 
#               from R2.0 to R2.1  while migraing to R2.1 .
# Input       : Reference to Configuration file 
# OutPut      : Reference to Configuration file 
# Orgnisation : Future Software.
#******************************************************************************


sub IssPortCtrlTableForR2_1
{

    my ($refConfigFile)=@_;
    my @lines         = @{$refConfigFile};
    my $PortCtrlMode  = "1.3.6.1.4.1.2076.81.2.2.1.2.";
    my $PortCtrlSpeed = "1.3.6.1.4.1.2076.81.2.2.1.4.";
    my $ObjPortCtrlMode = "";

    my $index1 = 0;
    my $index2 = 0;

    $msg = "\n Port Control  Table Change Starts .........  \n";
    push (@logfile, $msg);

    for (@lines)
    {
        # Search for PortCtrlMode
        my @fields1 = split (/\s+/, $lines[$index1]);
        if ($fields1[0] =~ m/^$PortCtrlMode/ )
        {
            if( $fields1[2] == 1)   # Mode is auto   
            {
                # do-nothing 
            }
            else # mode is no-negotiation
            {
                # get the Port Number;
                my @dotFields = split(/$PortCtrlMode/,$fields1[0]);
                my $PortNum = $dotFields[1];
                my $PortCtrlSpeedIndex = $PortCtrlSpeed.$PortNum;

                # search for the PortCtrlSpeed corr to the current Index 
                $index2 = 0;

                for (; $index2 <= $#lines; $index2++)
                {
                    my @fields2 = split(/\s+/, $lines[$index2] );
                    my $Issmode = 1;  #default mode 
                    # look for the PortCtrlSpeed for the above  Port
                    if ($fields2[0] =~ /^$PortCtrlSpeedIndex$/ )
                    {

                        #  switch( $fields2[2] ) Switch Stmt is not supporting 

                        if( $fields2[2] == 1 ) 
                        {
                            $Issmode = 4; # set to tenMBPS  
                        }
                        elsif($fields2[2] == 2 )
                        {
                            $Issmode = 3;  # set to hundredMBPS
                        }
                        elsif($fields2[2] == 3 )
                        {
                            $Issmode = 2;  #set to oneGB
                        }
                        elsif($fields2[2] == 4 )
                        {
                            $Issmode = 1; # set to Auto 
                        }

                        # Modify the Issmode as calculated above.
                        $msg = "\nOld PortCtrlMode : $lines[$index1]  ";
                        push (@logfile, $msg);

                        $msg = "\nNewPortCtrlMode  to be Inserted = $Issmode ";
                        push (@logfile, $msg);
                        my $Newline = $fields1[0]."\t ".$fields1[1]."\t ".$Issmode."\n" ;

                        $lines[$index1] = $Newline;    

                        $msg = "\nNew PortCtrlMode : $lines[$index1] \n";
                        push (@logfile, $msg);
                        # break the loop;
                        $index2 = $#lines + 1;

                    } # End of if $fileds2. 

                } # End of for  $index2

            } # End of else

        } #End of if fields1 .

        $index1++;

    } # End of Outer for loop .

    $msg = "\n Port Control Table Change Ends   \n";
    push (@logfile, $msg);

    my $refReturn = \@lines;
    return $refReturn;

}

# Jayalk [27-Jul-2006]  End:  */


#Jayalk : 17/Aug/2006 

#******************************************************************************
# Routine     :  Alter_Object_Position
# Description : This subroutine moves the position of an object(s) 
#               before the specified Object.
#               Ex: The Position of
#               IssPortCtrlEntries( 1.3.6.1.4.1.2076.81.2.2.1.x.x ) can be moved
#               to just before 
#               ffFastForwardingEnable( 1.3.6.1.4.1.2076.27.2.1.0 )
# 
# Input       : Object to be moved,InsertBeforeObject,
#               Reference to Configuration file 
# OutPut      : Reference to Configuration file 
# Orgnisation : Future Software.
#******************************************************************************



sub Alter_Object_Position
{

    my ($ObjectTobeMoved , $InsertBefore ,$refConfigFile)=@_;
    my @lines         = @{$refConfigFile};
    my @LinestoInsert  = " ";

    my $SearchString  = $ObjectTobeMoved ;
    my $SearchTag     = $InsertBefore ;
    my $Flag = "0";
    my @DeleteObject  =  $ObjectTobeMoved;

    my $i =0;
    my $count = 0;
    for (@lines)
    {
        my @fields = split (/\s+/, $lines[$i]);
        if ($fields[0] =~ m/^$SearchString/ )
        {
            # collect all the lines to be moved into the array.
            $LinestoInsert[$count++] = $lines[$i];
        }
        if ($fields[0] =~ m/^$SearchTag/ )
        {
            $Flag = "1";
        }
        $i++;

    }
    if( $count != 0 and $Flag eq "1")
    {
        # First Delete the OIDs from the current location.

        $refConfigFile = obsolete (\@DeleteObject,$refConfigFile );

        # Insert the collected lines at proper location.   

        $refConfigFile = InsertLinesRoutine ($InsertBefore,
            \@LinestoInsert,$refConfigFile,$count);


    }
    return $refConfigFile ;
}




#<vikash>
sub parserCacTable
{

    my ($IGMPXmlFile)  = @_;
    my $enddbName      = "</dbName>";
    my $endvalue       = "</value>";
    my $linecount      = 0;
    my $i              = 0;
    my @tempFileArray;
    my @data;
    my $flag;

    my $filehandle = IO::File->new($IGMPXmlFilePath,"r") or
    die "Error: Could not open $IGMPXmlFilePath \n";

    my @lines          = <$filehandle>;

    for (@lines)
    {

        if ($lines[$linecount++] =~ m/<dbName>/)
        {

            @data = split (/<dbName>|$enddbName/, $lines[$linecount-1]);
            if ($data[1] eq "mcSrcAd" || $data[1] eq "vlanID"||$data[1] eq "susBitr")
            {

                $tempFileArray[$i] = $data[1];
                $flag              = 1;
            }
        }

        if ($lines[$linecount] =~ m/<value>/)
        {

            @data = split (/<value>|$endvalue/, $lines[$linecount]);
            if ($flag ==1)
            {

                $tempFileArray[$i] = sprintf ("%s\t%s\n", $tempFileArray[$i], $data[1]);
                $flag              = 0;
                $i++;
            }
        }
    }

    return \@tempFileArray;
}


#******************************************************************************
# Name FunctionFor3_1 
# Description :- This function is used for conversion of 3.1 realase
# Input :- iss.conf file
# OutPut :- None
#******************************************************************************
sub FunctionFor3_1 
{
    my ($refconfigfile) = @_;
    my @configfile      = @{$refconfigfile};
    my $countRow=0;
    my $vlanId; 

    my $DottedDecIpAddress;
    my $HexIpAddress;
    my @DecSpIpAddress;
    my $susBitr=0;
    my $validMcAddr=1;

    my @cacTableRow="";

    my $refFileArray = parserCacTable($IGMPXmlFilePath);

    my @FileArray = @{$refFileArray};
    $i = 0;
    my $flag_cac=0;


    for (@FileArray)
    {

        my @SplitFileArray = split(/\s+/, $_);
        if($SplitFileArray[0] eq "mcSrcAd")
        {
            $validMcAddr=1; 
#here convert it to decimal notation


            $HexIpAddress = sprintf("%x", $SplitFileArray[1]);
            my $k = 0;
            for(my $j = 0; $j < 4 ; $j++)
            {

                $DecSpIpAddress[$j] = hex(substr($HexIpAddress, $k,2));     
                $k = int($k)+2;
            }
            $DottedDecIpAddress = join(".", @DecSpIpAddress);


            if(($DecSpIpAddress[0] < 224) || ($DecSpIpAddress[0] > 239))
            {
                $validMcAddr=0;
                $msg="\nInvalid Multicast Address :$SplitFileArray[1]\n";

                push (@logfile, $msg);

            } 





        } 

        if($SplitFileArray[0] eq "susBitr")
        {
            $susBitr = $SplitFileArray[1];

        }
        if($SplitFileArray[0] eq "vlanID")
        {
            $vlanId = $SplitFileArray[1];   

            $flag_cac=1;    


        }


        if(($flag_cac)&&($validMcAddr))
        {

#make the row-status create and wait

            $cacTableRow[$countRow]="1.3.6.1.4.1.2076.74.6.2.1.1.7."."$vlanId"."."."$DottedDecIpAddress"."."."0.0.0.0"."\t"."2"."\t"." "."5"."\n";
            $countRow++;
#sustained bit rate

            if($traceBW)
            { 
                $susBitr*= 0.85;

                $susBitr=int $susBitr ;
#   print "Sustained Bit Rate:$susBitr","\n";



                $cacTableRow[$countRow]="1.3.6.1.4.1.2076.74.6.2.1.1.4."."$vlanId"."."."$DottedDecIpAddress"."."."0.0.0.0"."\t"."2"."\t"." "."$susBitr" ."\n";


                $countRow++;
            }
            else
            {

                $cacTableRow[$countRow]="1.3.6.1.4.1.2076.74.6.2.1.1.4."."$vlanId"."."."$DottedDecIpAddress"."."."0.0.0.0"."\t"."2"."\t"." "."0"."\n";


                $countRow++;
            }
#guaranteed flag

            $cacTableRow[$countRow]="1.3.6.1.4.1.2076.74.6.2.1.1.5."."$vlanId"."."."$DottedDecIpAddress"."."."0.0.0.0"."\t"."2"."\t"." "."2"."\n";

            $countRow++;

#Budle ID=0
            $cacTableRow[$countRow]="1.3.6.1.4.1.2076.74.6.2.1.1.6."."$vlanId"."."."$DottedDecIpAddress"."."."0.0.0.0"."\t"."2"."\t"." "."0"."\n";

            $countRow++;
#making the row status active

            $cacTableRow[$countRow]="1.3.6.1.4.1.2076.74.6.2.1.1.7."."$vlanId"."."."$DottedDecIpAddress"."."."0.0.0.0"."\t"."2"."\t"." "."1"."\n";

            $countRow++;


            $flag_cac=0;#resetting the flag
        }
#$configref= InsertLinesRoutine ("END", \@cacTableRow,$configref, $countRow);

    }


    $configref= InsertLinesRoutine ("END", \@cacTableRow,$configref, $countRow);
}

#</vikash> 

#Jayalk [15/3/2006]: End 

# indu-preethu [29/09/06][START]
#***********************************************************************************************************
#Routine:object_port_change
#Description: This subroutine changes the portbitmap value of the specified object in the migration.dat
#         file. Ports can be added or deleted from the port bit map. If the object represents the 
#         portlist of a vlan, then only for a particular vlan the port can be added or deleted.
#Input :$list_ref, $config_ref
#OutPut :- $configref 
#Organisation :- Flextronics Software Solutions.
#***********************************************************************************************************
sub object_port_change
{
    #print "\nEntering Port object change function\n";
    my($refobjectlist, $reflines)=@_;   
    my @objectlist = @{$refobjectlist}; 
    my @Lines = @{$reflines};
    my $count = 0;
    my $numofadd_deletions =""; 
    my @ObjectPortChange = "";

    $numofadd_deletions = $objectlist[$count++];
        $msg ="\nThe number of lines in the tag is $numofadd_deletions\n"; 
    push (@logfile, $msg);
    for (my $j = 0; $j < int ($numofadd_deletions); $j++)
    {
        push(@ObjectPortChange, $objectlist[$count++]); 

    }

    shift(@ObjectPortChange);
       $msg ="\nThe line read from the dat file is \n@ObjectPortChange\n";
    push (@logfile, $msg);
    for(@ObjectPortChange)
    {
    $msg ="\nThe line stored is $_\n";
    push (@logfile, $msg);
        my @ObjectPortChangeFields = split(/\s+/, $_);
       $msg ="\nThe value of the name is $ObjectPortChangeFields[0]\n";
    push (@logfile, $msg);
        my $ObjectPortChangeName = $ObjectPortChangeFields[0];
        my $ObjectPortChangeAddOrDelete = $ObjectPortChangeFields[1];
        my $ObjectPortChangePortDetails = $ObjectPortChangeFields[2];
        my $ObjectPortChangeVlan = $ObjectPortChangeFields[3];
        my $LineCount = 0;

        my @PortDetailsArray = "";
        $msg ="\nThe Object name is $ObjectPortChangeName\n";
        push (@logfile, $msg);
        $msg ="\nThe Operation is $ObjectPortChangeAddOrDelete\n";
        push (@logfile, $msg);
        $msg ="\nThe Object port details are $ObjectPortChangePortDetails\n";
        push (@logfile, $msg);
        $msg ="\nThe Object port vlan is $ObjectPortChangeVlan\n";
        push (@logfile, $msg);


        #Extract the port details from the migration dat file.
        while($ObjectPortChangePortDetails != 0)
        {
            my $bit = $ObjectPortChangePortDetails % 2;
            unshift(@PortDetailsArray, $bit);
            $ObjectPortChangePortDetails = int($ObjectPortChangePortDetails / 2);
        }
        pop(@PortDetailsArray);
        my $LengthOfPortDetailsArray = @PortDetailsArray;

        for(@Lines)
        {
            $LineCount++;
            my @LineFields;
            my $True = 0;
            if($_ =~ m/$ObjectPortChangeName\b/)
            {
                my @val = $_;
                my $val;
                
                @LineFields = split(//, $_);
                my $LenLineFields = @LineFields;
                
                my $LineFieldPortBitMap;
                $LineFieldPortBitMap = substr($_, -4,3);
                
                my $UnpackedHex = unpack("H*", $LineFieldPortBitMap);
                my @UnpackedPortBitMap;
                @UnpackedPortBitMap =  unpack("b*", $LineFieldPortBitMap);

                @LineFields = split(/\s+/, $_);
                my $Oid = $LineFields[0];
                $msg ="\nThe OID is $Oid\n";
                push (@logfile, $msg);
                $msg ="\nThe Hex of the port details is $UnpackedHex\n";
                "\nThe line stored is $_\n";
                push (@logfile, $msg);
                $msg ="\nUnpacked array is @UnpackedPortBitMap\n";
                push (@logfile, $msg);

                my @UnpackedPortArray = split(//, $UnpackedPortBitMap[0]);
                my $LengthUnpackedArray = @UnpackedPortArray;
                $msg ="\nLength unpacked array $LengthUnpackedArray\n";
                push (@logfile, $msg);

                my $k = 0;
                for($k = 0; $k < $LengthOfPortDetailsArray ; $k++ )
                {
                    if($PortDetailsArray[$k] == 1)
                    {
                    $msg ="\nThe port to be modified is ($LengthOfPortDetailsArray-$k)\n";
                    push (@logfile, $msg);
                        my $PortBitMapArrayIndex;
                        my $Remainder = ($LengthOfPortDetailsArray-$k)%8;
                        my $NoOfBytes = int(($LengthOfPortDetailsArray-$k)/8);
                        if($Remainder > 0)
                        {
                            $PortBitMapArrayIndex = ($NoOfBytes+1)*8 - $Remainder;

                        }   
                        if($Remainder == 0)
                        {
                            $PortBitMapArrayIndex = ($NoOfBytes-1)*8; 
                        }

                        {
                        if($UnpackedPortArray[$PortBitMapArrayIndex] == 0)
                        {
                         
                         if($ObjectPortChangeAddOrDelete eq "ADD")
                        {
                            if($ObjectPortChangeVlan eq "NULL")
                            { 
                                $True = 1;
                                $UnpackedPortArray[$PortBitMapArrayIndex] = 1;
                            }
                            else
                            {
                                if($LineFields[0]=~ m/\.$ObjectPortChangeVlan$/)
                                {
                                    $msg ="\nThe vlan $ObjectPortChangeVlan has been mocdified\n";
                                    push (@logfile, $msg);
                                    $True = 1;
                                    $UnpackedPortArray[$PortBitMapArrayIndex] = 1;
                                }
                            }
                            
                        }
                        }
                        if($UnpackedPortArray[$PortBitMapArrayIndex] == 1)
                        {
                        if($ObjectPortChangeAddOrDelete eq "DELETE")
                        {
                            if($ObjectPortChangeVlan eq "NULL")
                            { 
                                $True = 1;
                                $UnpackedPortArray[$PortBitMapArrayIndex] = 0;
                            }
                            else
                            {
                                if($LineFields[0]=~ m/\.$ObjectPortChangeVlan$/)
                                {
                                    $True = 1;
                                    $msg ="\nThe vlan $ObjectPortChangeVlan has been mocdified\n";
                                    push (@logfile, $msg);
                                    $UnpackedPortArray[$PortBitMapArrayIndex] = 0;
                                }
                            }
                        }
                        }
                        }
                    }
                }

                   $msg ="\nThe value of port array after change is @UnpackedPortArray\n";
                   push (@logfile, $msg);
                {
                    my $PackedPortArray;
                    $PackedPortArray = join('', @UnpackedPortArray);
                    $PackedPortArray = pack("b*", $PackedPortArray);

                    my @Split = split(//, $PackedPortArray);

                    @LineFields = split(//, $_);
                    $LineFields[$LenLineFields-4] = $Split[0];
                    $LineFields[$LenLineFields-3] = $Split[1];
                    $LineFields[$LenLineFields-2] = $Split[2];
                    $val = join("", @LineFields);
                    $_ = $val;
                        
                    $True = 0;
                }
            }

        }



    #print "\nExiting Port object change function\n";
    }
    my $arrayRef = \@Lines;
    return $arrayRef;
}

#***********************************************************************************************************
#Routine:obsolete_with_value
#Description: This subroutine obsoletes the object Id from the configuration file(iss.conf) based on 
#         the value of the object ID. As of now support is provided to remove the OID based on 
#         the portbitmap,integer and static_mac_portbitmap. The static_mac_portbitmap removes the
#         static mac addresses on a particular port. In future if a OID has to be obsoleted based on
#         a certain different kind of value then that feature has to be coded with a new tag.
#Input :$list_ref, $config_ref
#OutPut :- $configref 
#Organisation :- Flextronics Software Solutions.
#***********************************************************************************************************
sub obsolete_with_value 
{
    #print "\nEntering subroutine obsolete_with_value\n";

    my($refobjectlist, $reflines)=@_;   
    my @objectlist = @{$refobjectlist}; 
    my @Lines = @{$reflines};
    my $count = 0;

    my $NumOfDeletions = ""; 
    my @ObjectDelete = "";

    #print "\nThe object list contains @objectlist\n";
    $NumOfDeletions = $objectlist[$count++];
    #print "\nThe number of deletions is $NumOfDeletions\n";
    for (my $j = 0; $j < int ($NumOfDeletions); $j++)
    {
        push(@ObjectDelete, $objectlist[$count++]); 

    }

    for(@ObjectDelete)
    {
        #print "\nThe line stored is $_\n";
        my @ObjectDeleteFields = split(/\s+/, $_);
                #print "\nThe value of the name is $ObjectDeleteFields[0]\n";
        my $ObjectDeleteName = $ObjectDeleteFields[0];
        my $ObjectDeleteType = $ObjectDeleteFields[1];
        my $ObjectDeleteByteLength = $ObjectDeleteFields[2];
        my $ObjectDeleteDetails= $ObjectDeleteFields[3];
        my $LineCount = 0;
        my @PortDetailsArray = "";
        my $NotEqual = 0;
        my @tempArray = " ";
        my $temp = 0;

        chomp($ObjectDeleteName);
        chomp($ObjectDeleteType); 
        chomp($ObjectDeleteDetails);    
#Here since there is no support for "switch" statement we are using if statements.

        if($ObjectDeleteType eq "PORTBITMAP")
        {

            while($ObjectDeleteDetails != 0)
            {
                my $bit = $ObjectDeleteDetails % 2;
                unshift(@PortDetailsArray, $bit);
                $ObjectDeleteDetails = int($ObjectDeleteDetails/ 2);
            }

            pop(@PortDetailsArray);
            my $LengthOfPortDetailsArray = @PortDetailsArray;
            
        #   print "\nThe bytes in the dat file is $ObjectDeleteByteLength\n";
            if($LengthOfPortDetailsArray < ($ObjectDeleteByteLength*8))
            {
              for(my $m = ($LengthOfPortDetailsArray);$m < ($ObjectDeleteByteLength*8); $m++)
              {
                unshift(@PortDetailsArray, 0); 
              }

            }

        #   print "\nThe value of port details in DAT file is @PortDetailsArray\n";
            $LengthOfPortDetailsArray = ($ObjectDeleteByteLength*8);
        #   print "\nThe length of Portdetails array is $LengthOfPortDetailsArray\n";

            for(@Lines)
            {
                my @LineFields = split(/\s+/, $_);

                $NotEqual = 0;
                if($LineFields[0] =~ m/$ObjectDeleteName\b/)
                {
                    my $LineFieldPortBitMap = $LineFields[3];
                    my @UnpackedPortBitMap =  unpack("b*", $LineFieldPortBitMap);

        #           print "\nUnpacked array is @UnpackedPortBitMap\n";
                    my @UnpackedPortArray = split(//, $UnpackedPortBitMap[0]);
                    my $LengthUnpackedArray = @UnpackedPortArray;

        #               print"\nThe length of unpacked array is $LengthUnpackedArray\n";
                    my $k = 0;
                    for($k = 0; $k < $LengthOfPortDetailsArray ; $k++ )
                    {
                        my $LogicalPort = ($LengthOfPortDetailsArray-$k);
                        my $Remainder = ($LengthOfPortDetailsArray-$k)%8;
                        my $NoOfBytes = int(($LengthOfPortDetailsArray-$k)/8);
                        my $PortBitMapArrayIndex;
                        if($Remainder > 0)
                        {
                            $PortBitMapArrayIndex = ($NoOfBytes+1)*8 - $Remainder;

                        }   
                        if($Remainder == 0)
                        {
                            $PortBitMapArrayIndex = ($NoOfBytes-1)*8; 
                        }                   
        #           print "\nThe Logical port and PortBitMapArrayIndex is $LogicalPort and $PortBitMapArrayIndex\n";
                    if(($UnpackedPortArray[$PortBitMapArrayIndex] != $PortDetailsArray[$k]))
                    {
        #               print "\nThe bitmap specified are not equal\n";
                        $NotEqual = 1;
                    }

                    }
                    if($NotEqual == 0)
                    {
        #               print "\nDeleted a line\n";
                    }
                    else
                    {
                        $tempArray[$temp++] = $Lines[$LineCount];
                    }
                }
                else
                {
                    $tempArray[$temp++] = $Lines[$LineCount];
                }
            
                $LineCount++;
            }
            @Lines = @tempArray; 

        }

        if($ObjectDeleteType eq "INTEGER")
        {

            for(@Lines)
            {
                my @LineFields = split(/\s+/, $_);
                if($LineFields[0] =~ m/$ObjectDeleteName\b/)
                {
                    my $LineFieldValue = $LineFields[2];
                    if($LineFieldValue != $ObjectDeleteDetails)
                    {

                        $tempArray[$temp++] = $Lines[$LineCount];

                    }
                }
                else
                {
                    $tempArray[$temp++] = $Lines[$LineCount];
                }

                $LineCount++;
            }
            @Lines = @tempArray;

        }
        
        if($ObjectDeleteType eq "STATIC_MAC_BITMAP")
        {
            my @TempConfLines = @Lines;
        #   print "\nSTATIC_MAC_BITMAP is recognised\n";
            while($ObjectDeleteDetails != 0)
            {
                my $bit = $ObjectDeleteDetails % 2;
                unshift(@PortDetailsArray, $bit);
                $ObjectDeleteDetails = int($ObjectDeleteDetails/ 2);
            }

            pop(@PortDetailsArray);
            my $LengthOfPortDetailsArray = @PortDetailsArray;
            
        #   print "\nThe bytes in the dat file is $ObjectDeleteByteLength\n";
            if($LengthOfPortDetailsArray < ($ObjectDeleteByteLength*8))
            {
              for(my $m = ($LengthOfPortDetailsArray);$m < ($ObjectDeleteByteLength*8); $m++)
              {
                unshift(@PortDetailsArray, 0); 
              }

            }

        #   print "\nThe value of port details in DAT file is @PortDetailsArray\n";
            $LengthOfPortDetailsArray = ($ObjectDeleteByteLength*8);
        #   print "\nThe length of Portdetails array is $LengthOfPortDetailsArray\n";

            for(@Lines)
            {
                my @LineFields = split(/\s+/, $_);

                $NotEqual = 0;
                if($LineFields[0] =~ m/$ObjectDeleteName\b/)
                {
                    my $LineFieldPortBitMap = $LineFields[3];
                    my @UnpackedPortBitMap =  unpack("b*", $LineFieldPortBitMap);

        #           print "Coming in\n";
        #           print "\nUnpacked array is @UnpackedPortBitMap\n";
                    my @UnpackedPortArray = split(//, $UnpackedPortBitMap[0]);
                    my $LengthUnpackedArray = @UnpackedPortArray;

        #               print"\nThe length of unpacked array is $LengthUnpackedArray\n";
                    my $k = 0;
                    for($k = 0; $k < $LengthOfPortDetailsArray ; $k++ )
                    {
                        my $LogicalPort = ($LengthOfPortDetailsArray-$k);
                        my $Remainder = ($LengthOfPortDetailsArray-$k)%8;
                        my $NoOfBytes = int(($LengthOfPortDetailsArray-$k)/8);
                        my $PortBitMapArrayIndex;
                        if($Remainder > 0)
                        {
                            $PortBitMapArrayIndex = ($NoOfBytes+1)*8 - $Remainder;

                        }   
                        if($Remainder == 0)
                        {
                            $PortBitMapArrayIndex = ($NoOfBytes-1)*8; 
                        }                   
        #           print "\nThe Logical port and PortBitMapArrayIndex is $LogicalPort and $PortBitMapArrayIndex\n";
                    if(($UnpackedPortArray[$PortBitMapArrayIndex] != $PortDetailsArray[$k]))
                    {
        #               print "\nThe bitmap specified are not equal\n";
                        $NotEqual = 1;
                    }

                    }
                    if($NotEqual == 0)
                    {
                        my @MacAddress = split($ObjectDeleteName, $LineFields[0]);
                        my $LengthMac = @MacAddress;
                        my $JoinedMacAddress = $MacAddress[1];
        #               print "\nThe value of MAC address is $JoinedMacAddress\n";
        #               print "\nThe length of MAC address is $LengthMac\n";
                        my $TempLineCount = 0;
                        my @MatchedLineArray;
                        my $Found = 0;
                        for(@TempConfLines)
                        {
                            my @TempLineFields = split(/\s+/, $_);

                            if($TempLineFields[0] =~ m/$JoinedMacAddress\b/)
                            {
                                $MatchedLineArray[$Found++] = $TempLineCount;
        #                       print "\nFound $Found-1 match\n";

                            }   
                            $TempLineCount++;
                        
                        }
                        for(my $a = ($Found-1); $a >= 0; $a--)
                        {
                            splice(@TempConfLines, $MatchedLineArray[$a],1);
                        }
                        

                    }
                    
                }
                
                $LineCount++;
            }
        @Lines = @TempConfLines;
        }

    }

    my $arrayRef = \@Lines;
    return $arrayRef;
}

sub cpe_vlans_insert
{
       $msg ="\nEntered CPE VLANS INSERT function\n";
       push (@logfile, $msg);

    my ($RefConfigFile) = @_; # iss.conf
    my @ConfigFile = @{$RefConfigFile};
    
    my $External_Vlan = "";
    my @ConfigFileSplitFields= "";
    #OIDs defined here
        
    my $CPE_OID = "1.3.6.1.4.1.2076.65.1.21.1.2";
    my $VLANMODE_OID = "1.3.6.1.4.1.2076.65.1.18.1.1";
    my $EXT_MGMT_OID = "1.3.6.1.4.1.2076.81.1.34.0";
    # ANTmt11160 naveenra start
    my $VLAN_EGRESS_PORTS_OID = "1.3.6.1.2.1.17.7.1.4.3.1.2"; 
    my @VlansWithControlPort= "";
    my $i= 0;
    my $j= 0;
    my $VlanWithCntlPortCount = 0;
    my $VlanID = 0;
    # ANTmt11160 naveenra end
    my $INTERNAL_MGMT_OID = 4094;
    my $NO_CPE_VLANS = 3;
    my $VlanArrayCount = 0;
    my $CpeVlanCount = 0;
    my @TempArray = "";
    my $TempArrayCount = 0;

    for(@ConfigFile)
    {
        @ConfigFileSplitFields= split(/\s+/, $_);
        #Check for External management Vlan
        if($ConfigFileSplitFields[0] =~ m/$EXT_MGMT_OID\b/)
        {
            $External_Vlan = $ConfigFileSplitFields[2];
                $msg ="\nThe external management Vlan ID is $External_Vlan\n";
                push (@logfile, $msg);
        }
        # ANTmt11160 naveenra start
        if ($ConfigFileSplitFields[0] =~ m/$VLAN_EGRESS_PORTS_OID\b/)
        {
            my @OIDSplit = split(/\./, $ConfigFileSplitFields[0]);
            my $LenOIDSplit = @OIDSplit;
            $VlanID = $OIDSplit[$LenOIDSplit-1];
            my $portlist = $ConfigFileSplitFields[3];
            my @b = split (//, $portlist);
            if (ord($b[2]) >= 128) 
            {
                $VlansWithControlPort[$VlanWithCntlPortCount] = $VlanID;
                $VlanWithCntlPortCount ++;
            }
        }
    }   
    #Here we configure the CPE management VLANS. The first three Vlans excluding the
    #External and Internal management Vlans are configured as CPE management Vlans.

        
    $msg = "\n===========================================================\n";
    push (@logfile,$msg);
    push (@dispresult,$msg);
    $msg = "The following Vlans will be added as CPE management Vlans...\n\n";
    push (@logfile,$msg);
    push (@dispresult,$msg);
    for(@ConfigFile)
    {
        @ConfigFileSplitFields= split(/\s+/, $_);

        $TempArray[$TempArrayCount++] = $_;
        if ($ConfigFileSplitFields[0] =~ m/$VLANMODE_OID\b/)
        {
            my @OIDSplit = split(/\./, $ConfigFileSplitFields[0]);
            my $LenOIDSplit = @OIDSplit;
            $VlanID = $OIDSplit[$LenOIDSplit-1];
            $j= 0;
            if(($i < $VlanWithCntlPortCount) || ($CpeVlanCount < $NO_CPE_VLANS))
            {
                while($j < $VlanWithCntlPortCount)
                {
                    if($VlanID eq $VlansWithControlPort[$j])
                    {
                        if(($VlanID ne $INTERNAL_MGMT_OID)&&($VlanID ne $External_Vlan)&&($VlanID ne 1))
                        {
                            my $TempOIDLine = "$CPE_OID.$VlanID" . "\t " . "2" . "\t" ."5\n"; 
                            $TempArray[$TempArrayCount++] = $TempOIDLine;
                            #Add this line after the mode setting.  
                
                            my $TempOIDLine1 = "$CPE_OID.$VlanID" . "\t " .  "2" . "\t". "1\n"; 
                            $TempArray[$TempArrayCount++] = $TempOIDLine1;
                            $CpeVlanCount++;
                            $msg ="\nThe vlan added as CPE is $VlanID\n";
                            push (@logfile, $msg);
                            push (@dispresult,$msg);
                            $i++;
                            last;
                        }
                        $i++;
                    } 
                    $j++;
                }
            }           
        }   
    }
        # ANTmt11160 naveenra end

    $msg = "\n===========================================================\n";
    push (@logfile,$msg);
    push (@dispresult,$msg);

    my $arrayRef = \@TempArray;
    return $arrayRef;

}
# indu-preethu [29/09/06][END]
#indu - added for cli command change [07/11/06][START]
sub interactivemode
{
        print "\n\t Enter the input configuration file\n";
    print "\n\t Example : issa.conf \n";
        $destfile = <STDIN>;
        chop $destfile;
    $destfile =~ s/\s+$//;
    $destfile =~ s/^\s+//; 
    $dispmsg = "\n\t The input conf file is :$destfile\n";
    push(@dispresult,$dispmsg);
    push(@logfile,$dispmsg);

        print "\n\t Enter the output configuration file\n";
    print "\n\t Example : issnew.conf \n";
        $newconfigfile = <STDIN>;
        chop $newconfigfile;
    $newconfigfile =~ s/\s+$//;
    $newconfigfile =~ s/^\s+//; 
    $dispmsg = "\n\t The output conf file is :$newconfigfile\n";
    push(@dispresult,$dispmsg);
    push(@logfile,$dispmsg);

        print "\n\t Enter the version FROM which migration has to be done\n";
    print "\n\t Example : R2.4 / R2.5\n";
        $cliversionfrom = <STDIN>;
        chop $cliversionfrom;
    $cliversionfrom =~ s/\s+$//;
    $cliversionfrom =~ s/^\s+//; 
    $dispmsg = "\n\t The FROM version is: $cliversionfrom\n";
    push(@dispresult,$dispmsg);
    push(@logfile,$dispmsg);

        print "\n\t Enter the version TO which migration has to be done\n";
    print "\n\t Example : R2.5 / R3.3 \n";
        $cliversionto = <STDIN>;
        chop $cliversionto;
    $cliversionto =~ s/\s+$//;
    $cliversionto =~ s/^\s+//; 
    $dispmsg = "\n\t The TO version is :$cliversionto\n";
    push(@dispresult,$dispmsg);
    push(@logfile,$dispmsg);

    # ALU01660534 / 13724 / 14380/14439/14620/14763/14875/14927 - Changed to R6.0
    # Check for version for supporting releases higher than F4.5/R6.0 
    $cliversionfromOrig  = $cliversionfrom; 
    $cliversionfrom = CheckVersionForHigherRelease($cliversionfrom);


    # ALU01613898 / 13724 / 14380 / 14439/ 14620 /14763/14875/14927 - Changed to R6.0
    # Check for version for supporting releases higher than F4.5/R6.0 
    $cliversiontoOrig  = $cliversionto; 
    $cliversionto = CheckVersionForHigherRelease($cliversionto);

    if(($cliversionfrom eq "R3.6") && ($cliversionto ge "R4.3"))
    {
	    print "\n\t Enter the User-to-User-Hairpin Status\n";
	    print "\n\t Example : TRUE / FALSE\n";
        	$HairpinStatus = <STDIN>;
	        chop $HairpinStatus;
	    $HairpinStatus =~ s/\s+$//;
	    $HairpinStatus =~ s/^\s+//; 
	    $dispmsg = "\n\t The User-to-User-Hairpin Status is: $HairpinStatus\n";
	    push(@dispresult,$dispmsg);
	    push(@logfile,$dispmsg);
    }

        $inputfile = "migration.dat";
    $dispmsg = "\n\t The dat file is :$inputfile\n";
    push(@dispresult,$dispmsg);
    push(@logfile,$dispmsg);

    $miglogfile = "logfile";
        ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
    my $tempp = "";
    $tempp=sprintf ("%02d%02d%4d_%02d%02d%02d",$mon+1,$mday,$year+1900,$hour,$min,$sec);
        $miglogfile=$miglogfile.$tempp.".txt";
    $dispmsg = "\n\t The logfile is :$miglogfile\n";
    push(@dispresult,$dispmsg);
    push(@logfile,$dispmsg);


}

sub infilemode
{
        my $inputFileHandle = IO::File -> new ($inputFileName, "r") or
        die "Couldn't Open $inputFileName for reading";
        my @linesinput  =  <$inputFileHandle>;
        
    my $linecount = 0;

        for(@linesinput)
        {
                @resultarray = split(/=/,$_);
        chop $resultarray[1];
                if($resultarray[0] eq "PARAMS END\n")
                 {
                        last;
                 }      
            if($resultarray[0] =~ m/^PARAMS START\b/)
            {
                print "\n\t Reading parameters in progress\n";
            }
            if($resultarray[0] =~ m/^Inputconffile\b/)
            {
                            $destfile = $resultarray[1];
                $dispmsg = "\n\t The input conf file is :$destfile\n";
                push(@dispresult,$dispmsg);
                push(@logfile,$dispmsg);
            }
            #if($resultarray[0] =~ m/^Datfile\b/)
            #{
                        #   $inputfile = $resultarray[1];
            #   $dispmsg = "\n\t The dat file is :$inputfile\n";
            #   push(@dispresult,$dispmsg);
            #   push(@logfile,$dispmsg);
            #}
            if($resultarray[0] =~ m/^Outputconffile\b/)
            {
                            $newconfigfile = $resultarray[1];
                $dispmsg = "\n\t The output conf file is :$newconfigfile\n";
                push(@dispresult,$dispmsg);
                push(@logfile,$dispmsg);
            }
            if($resultarray[0] =~ m/^Versionfrom\b/)
            {
                            $cliversionfrom = $resultarray[1];
                $dispmsg = "\n\t The FROM version is: $cliversionfrom\n";
                push(@dispresult,$dispmsg);
                push(@logfile,$dispmsg);
                # ALU01660534 / 13724 / 14380 / 14439 /14620 /14763/14875/14927- Changed to R6.0
		# Check for version for supporting releases higher than F4.5/R6.0 
    		$cliversionfromOrig  = $cliversionfrom; 
		$cliversionfrom = CheckVersionForHigherRelease($cliversionfrom);
            }
            if($resultarray[0] =~ m/^Hairpinstatus\b/)
            {
                            $HairpinStatus = $resultarray[1];
                $dispmsg = "\n\t The User-to-User-Hairpin status is: $HairpinStatus\n";
                push(@dispresult,$dispmsg);
                push(@logfile,$dispmsg);
		if (!(($HairpinStatus =~m/^TRUE\b/) || ($HairpinStatus =~m/^FALSE\b/)))
		{
                    $msg = "\n Accepted values for User-to-User-Hairpin status are TRUE and FALSE !!!\n\r";
                    print $msg;
                    push(@logfile,$msg);
                    exitMigTool (1);
                }
		$linesininput ++;
            }
            if($resultarray[0] =~ m/^Versionto\b/)
            {
                            $cliversionto = $resultarray[1];
                $dispmsg = "\n\t The TO version is :$cliversionto\n";
                push(@dispresult,$dispmsg);
                push(@logfile,$dispmsg);
                # ALU01613898 / 13724 / 14380 /14439/14620/14763/14875/14927 - Changed to R6.0
		# Check for version for supporting releases higher than F4.5/R6.0
    		$cliversiontoOrig  = $cliversionto; 
		$cliversionto = CheckVersionForHigherRelease($cliversionto);
            }
            if($resultarray[0] =~ m/^Modefrom\b/)
            {
                            $climodefrom = $resultarray[1];
                $dispmsg = "\n\t The FROM mode is :$climodefrom\n";
                push(@dispresult,$dispmsg);
                push(@logfile,$dispmsg);
                if(!(($climodefrom =~ m/^ECNTA\b/) || ($climodefrom =~ m/^ECNTC\b/) || ($climodefrom =~ m/^ecnta\b/) || ($climodefrom =~ m/^ecntc\b/)))
                {
                    $msg = "\n Accepted values for Modefrom are ECNTA and ECNTA !!!\n\r";
                    print $msg;
                    push(@logfile,$msg);
                    exitMigTool (1);
                }
            }
            if($resultarray[0] =~ m/^Modeto\b/)
            {
                            $climodeto = $resultarray[1];
                $dispmsg = "\n\t The TO mode is :$climodeto\n";
                push(@dispresult,$dispmsg);
                push(@logfile,$dispmsg);
                                if(!(($climodeto=~ m/^ECNTA\b/) || ($climodeto=~ m/^ECNTC\b/) || ($climodeto=~ m/^ecnta\b/) || ($climodeto=~ m/^ecntc\b/)))
                                {
                                        $msg = "\n Accepted values for Modeto are ECNTA and ECNTA !!!\n\r";
                                        print $msg;
                                        push(@logfile,$msg);
                                        exitMigTool (1);
                                }
            }
            if($resultarray[0] =~ m/^Trace\b/)
            {
                            $trace = $resultarray[1];
                $dispmsg = "\n\t The trace is :$trace\n";
                push(@dispresult,$dispmsg);
                push(@logfile,$dispmsg);
            }
            if($resultarray[0] =~ m/^Bandwidth\b/)
            {
                            $bandwidth = $resultarray[1];
                $dispmsg = "\n\t The bandwidth is :$bandwidth\n";
                push(@dispresult,$dispmsg);
                push(@logfile,$dispmsg);
            }
            #if($resultarray[0] =~ m/^Miglogfile\b/)
            #{
                        #   $miglogfile = $resultarray[1];
                #   $dispmsg = "\n\t The logfile is :$miglogfile\n";
            #   push(@dispresult,$dispmsg);
            #   push(@logfile,$dispmsg);
            #}
            if($resultarray[0] =~ m/^Macmode\b/)
            {
                            $macmode = $resultarray[1];
                $dispmsg = "\n\t The macmode is :$macmode\n";
                push(@dispresult,$dispmsg);
                push(@logfile,$dispmsg);
            }
            if($resultarray[0] =~ m/^InterfaceBase\b/)
            {
                            $interfaceBase = $resultarray[1];
                $dispmsg = "\n\t The InterfaceBase is :$interfaceBase\n";
                push(@dispresult,$dispmsg);
                push(@logfile,$dispmsg);
            }
            
                        $linecount ++;
    }
            
                        $miglogfile = "logfile";
            ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
            my $tempp = "";
            $tempp=sprintf ("%02d%02d%4d_%02d%02d%02d",$mon+1,$mday,$year+1900,$hour,$min,$sec);
                        $miglogfile = $miglogfile.$tempp.".txt";
            $dispmsg = "\n\t The logfile is :$miglogfile\n";
            push(@dispresult,$dispmsg);
            push(@logfile,$dispmsg);

            $inputfile = "migration.dat";
            $dispmsg = "\n\t The dat file is :$inputfile \n";
            push(@logfile,$dispmsg);
            push(@dispresult,$dispmsg);

	    if (($cliversionfrom eq "R3.6") && ($cliversionto ge "R4.3") && ($HairpinStatus eq ""))
	    {
                $dispmsg = "\n\t Please provide all the required informations. Insufficient inputs provided in the file\n";
                push(@logfile,$dispmsg);
                print $dispmsg;
                $msg="\n\t User-to-User-Hairpin Status not present in inputfile\n";
                print $msg;
                push(@logfile,$msg);
                push(@dispresult,$msg);
                exitMigTool(1); 
	    }

            if($linecount ne $linesininput)
            {
                $dispmsg = "\n\t Please provide all the required informations. Insufficient number of inputs provided in the file\n";
                push(@logfile,$dispmsg);
                print $dispmsg;
                  if($destfile eq "")
                  {
                    $msg="\n\t The input conf file date not present in inputfile\n";
                    print $msg;
                    push(@logfile,$msg);
                    push(@dispresult,$msg);
                  }
                  if($newconfigfile eq "")
                  {
                    $msg="\n\t The output conf file data not present in inputfile\n";
                    print $msg;
                    push(@logfile,$msg);
                    push(@dispresult,$msg);
                  }
                  if($cliversionfrom eq "")
                  {
                    $msg="\n\t The version from not present in inputfile\n";
                    print $msg;
                    push(@logfile,$msg);
                    push(@dispresult,$msg);
                  }
                  if($cliversionto eq "")
                  {
                    $msg="\n\t The version to not present in inputfile\n";
                    print $msg;
                    push(@logfile,$msg);
                    push(@dispresult,$msg);
                  }
                  if($climodefrom eq "")
                  {
                    $msg="\n\t The from mode not present in inputfile\n";
                    print $msg;
                    push(@logfile,$msg);
                    push(@dispresult,$msg);
                  }
                  if($climodeto eq "")
                  {
                    $msg="\n\t The to mode not present in inputfile\n";
                    print $msg;
                    push(@logfile,$msg);
                    push(@dispresult,$msg);
                  }
                  if($trace eq 0)
                  {
                    $msg="\n\t The trace option not present in inputfile\n";
                    print $msg;
                    push(@logfile,$msg);
                    push(@dispresult,$msg);
                  }
                  if($bandwidth eq "")
                  {
                    $msg="\n\t The bandwidth option not present in inputfile\n";
                    print $msg;
                    push(@logfile,$msg);
                    push(@dispresult,$msg);
                  }
                  #if($miglogfile eq "")
                  #{
                  # print "\n\t The miglogfile data not present in inputfile\n";
                  #}

                  
                    exitMigTool(1); 
            }
}
#indu - added for cli command change [07/11/06][END]
################################################################################
#Routine    :FunctionFor3_4
#Description:This function will handle the database migration from 3.3 to R3.4
#Input      :refLines
#output     :None
#return     :configref
#author     :selvam M
################################################################################
sub FunctionFor3_4
{
    my ($refLines)=@_;
    my @lines=@{$refLines};
    my $ArpProxyenable="1.3.6.1.4.1.2076.2.2.16.1.2.";
    my $NewArpProxyenable="1.3.6.1.4.1.2076.2.2.16.1.3.";
    my $lineTobeInserted=0;
    my $linecount=0;
    my $True=1;
    my $False=0;
    my $foundArpProxy=0;
    my $sptline=0;
    foreach $sptline (@lines)
    {
        if($sptline =~ m/^$ArpProxyenable/)
        {
            if($trace==1)
            {
            print "\n Line Number".$linecount."\n";
            print "proxy arp enable line matched\n";
            }
            my @fields=split(/\s+/,$sptline);
            my @ipaddress=split(/$ArpProxyenable/,$fields[0]);
            if($trace==1)
            {
            print "\n first index=". $ipaddress[1];
            }

            my $arpIpAddress=$ipaddress[1];
            my $IfIndex=get_IfIndex(\@lines,$arpIpAddress);
            if($trace==1)
            {
            print "\n IfIndex=".$IfIndex."\n";
            }

            my $FFVrfId=get_FFVrfid(\@lines);
            if($trace==1)
            {
            print "FFVrfId=".$FFVrfId."\n";
            }

            if($True==check_IfIndex_IsunderFFVrf(\@lines,$IfIndex,$FFVrfId))
            {
            if($trace==1)
            {
                print "ifindex is present in FF\n";
            }

                $lineTobeInserted=$NewArpProxyenable.$IfIndex.".$arpIpAddress"."\t ".$fields[1]."\t".$fields[2]."\n";
                $foundArpProxy=$True;
            }

        }
        if($foundArpProxy==$True)
        {
            $lines[$linecount] = $lineTobeInserted;
            $foundArpProxy=$False;
#           splice(@lines, $linecount, 1);#delete the old line
#           splice(@lines, $linecount, 0, $lineTobeInserted); #insert new line
        }
        $linecount++; #To find the position for insert or delete from file
    }
         return \@lines;
}   

################################################################################
#Routine    :get_IfIndex
#Description:This function is used to find the IfIndex from IP address
#Input      :$refTempLines,$refarpIpAddress
#output     :None
#return     :$ifIndex
#author     :selvam M
################################################################################
sub get_IfIndex
{
    my ($refTempLines,$refarpIpAddress)=@_;
    my @templines=@{$refTempLines};
#   my $ipAddress=$refarpIpAddress;
    my $ipMltiNetTble="1.3.6.1.4.1.2076.27.1.11.1.4.";
    if($trace==1)
    {
    print "\nIpAddrees". $refarpIpAddress."\n";
    }
    my $ifIndex=0;
    my $sptline=0;
        foreach  $sptline (@templines) 
        {
        if($sptline =~ m/$ipMltiNetTble/ && $sptline =~ m/$refarpIpAddress/)
        {
            if($trace==1)
            {
            print "\n line found \n";
            }

            my @dtFields=split(/\s+/,$sptline);
            my @indices=split(/\./,$dtFields[0]);
            if($trace==1)
            {
            print "\n".$indices[12]."\n";
            }

            $ifIndex=$indices[12];
            
        }
    }
    $ifIndex; #return ifindex
    
}

################################################################################
#Routine    :get_FFVrfid
#Description:This function is used to find the Fast Path VRF Id
#Input      :refTempLines
#output     :None
#return     :$ffvrfig
#author     :selvam M
################################################################################
sub get_FFVrfid
{
    my ($refTempLines)= @_;
    my @aTmpLines=@{$refTempLines};
    my $vrfmode="1.3.6.1.4.1.2076.91.1.1.4.";
    my $ffvrfid=0;
    my $sptline=0;
    foreach $sptline (@aTmpLines)
    {
        if($sptline =~ m/$vrfmode/)
        {
        my @dtfields=split(/\s+/,$sptline);
        if($dtfields[2]==2)#FFVrf
        {
            my @fields=split(/$vrfmode/,$dtfields[0]);
            $ffvrfid=$fields[1]; #assign FF vrf id
        }
        }
    }
    $ffvrfid; #return fast path vrf id
}

################################################################################
#Routine    :check_IfIndex_IsunderFFVrf
#Description:This function is used to verify the ifindex is belong to FF or not?
#Input      :$refAllLines,$refIfIndex,$refFFVrfId
#output     :$True or $False
#return     :$ffvrfig
#author     :selvam M
################################################################################
sub check_IfIndex_IsunderFFVrf
{
    my ($refAllLines,$refIfIndex,$refFFVrfId)=@_;
    my @refallLines=@{$refAllLines};
    my $ifIpVrfId="1.3.6.1.4.1.2076.27.1.5.1.6.";
    my $ifIndex=0;
    my $ffVrfId=0;
    my $True=1;
    my $False=0;
    my $found=$False;
    my $sptline=0;
    foreach $sptline (@refallLines)
    {
        if( $sptline =~ m/$ifIpVrfId/)
        {
            my @dtfields=split(/\s+/,$sptline);
            my @fields=split(/$ifIpVrfId/,$dtfields[0]);
            $ifIndex=$fields[1];
            $ffVrfId=$dtfields[2];
            if($refIfIndex==$ifIndex && $refFFVrfId==$ffVrfId)
            {
            if($trace==1)
            {
                print "ifIndex found in FF\n";
                print "ifidx=".$ifIndex."vrfid=".$ffVrfId."\n";
            }

                $found=$True;
                last;
            }
        }
    }
    $found;

}
            
# UNNUMBERED Interface - 9526 
#****************************************************************
# Routine       :   FunctionFor_unnumbered_int 
# Description  :   This subroutine checks Unnumbered Interface Configuration.
# Input           :  Reference to Configuration file 
# OutPut         :  None 
# Orgnisation  :  Aricent Communication Software.
#****************************************************************
sub FunctionFor_unnumbered_int
{
my ($refLines)=@_;
    my @lines=@$refLines;
    my $linecnt = 0;
    my $Value =0;
    my $UnNumberedInter = "1.3.6.1.4.1.2076.27.1.12.1.1";
    my @ObjectName = ""; 
    $ObjectName[0] = $UnNumberedInter;
    my $newindex             = 0;
    my $oldobject            = "";
    my $tempobject           = "";
    my $newobject            = "";
    #fix for ALU00977467 
    my $currobj = "INTERFACE INDEX MAPPING"; 
    
    for (; $linecnt <= $#lines; $linecnt++)
    {
        my @Fields = split /\s+/, $lines[$linecnt];

        if ($Fields[0] =~ $UnNumberedInter)
        {
          #fix for ALU00977467
          if( validateIndex($Fields[2],$currobj) eq "INVALID")
          {
             next;
          } 
          $newindex = ifIndexMap($Fields[2]);
          $newobject = "$UnNumberedInter    $Fields[1]  $newindex";
          $lines[$linecnt] =~ s/$Fields[2]\b/$newindex/; 
        }       
    }
    $configref = \@lines;
    return ($configref);
}
# No Dual Ip Support R37[16-04-08]
#****************************************************************
# Routine       :   FunctionFor3_7_No_Dual_Ip 
# Description  :   This subroutine checks the Single Ip Mode.
#                       If the Single Ip Mode is disabled then Migration will fail.
# Input           :  Reference to Configuration file 
# OutPut         :  None 
# Orgnisation  :  Aricent Communication Software.
#****************************************************************

sub FunctionFor4_0_No_Dual_Ip
{
my ($refLines)=@_;
    my @lines=@$refLines;
    my $linecnt = 0;
    my $SingleIpSearch = "1.3.6.1.4.1.2076.81.1.53.0";
       my @ObjectName = ""; 
        $ObjectName[0] = $SingleIpSearch;

    #Check for the Single Ip mode wheather it is enabled(1) or disabled(2)
        for (; $linecnt <= $#lines; $linecnt++)
    {
        my @Fields = split /\s+/, $lines[$linecnt];

        if ($Fields[0] =~ $SingleIpSearch)
        {
            #Check if the Single Ip Mode is disabled, then fail to migrate. 
            if($Fields[2] == 2)
                {
               die " Migration failed due to Single Ip Mode is Disabled. Not Supported for R4.0";
                }
                else
            {
            $configref = obsolete (\@ObjectName, $configref);
            }
        }       
    }
    return ($configref);
}

# ANTmt40020 -START 
sub RipIfAndMD5KeyCheck
{

    my ($refLines)=@_;
    my @lines=@$refLines;
    my $linecnt = 0;
    my $IfCount = 0;
    my $KeyCount = 0;
    my $RipIf = "1.3.6.1.4.1.2076.152.3.1.8";
    my $RipMD5Key = "1.3.6.1.4.1.2076.151.1.5.1.3.1";
    my $IfMsg = "\n   Number of Active Rip Interfaces exceed the limit \n";
    my $KeyMsg = "\n   Number of RIP MD5 keys exceed the limit \n";
    my $WarnMsg = "\n  WARNING : \n";
    for (; (($linecnt <= $#lines) && (($KeyCount < 6 ) || ($IfCount < 11))); $linecnt++)
    {
        my @Fields = split /\s+/, $lines[$linecnt];
        if (($Fields[0] =~ /^$RipIf/) && ($Fields[2] == 1))
        {
            $IfCount += 1;
        }       
        if (($Fields[0] =~ /^$RipMD5Key/))
        {
            $KeyCount += 1;
        }
    }
    if (($IfCount/2) > 10)
    {
        print $WarnMsg;
        print "========================================================== \n";
        print $IfMsg;
        print "\n========================================================== \n";
        push (@logfile,$WarnMsg);
        push (@logfile,$IfMsg);
        
    }
    if ($KeyCount > 5)
    {
        print $WarnMsg;
        print "========================================================== \n";
        print $KeyMsg;
        print "\n========================================================== \n";
        push (@logfile,$WarnMsg);
        push (@logfile,$KeyMsg);
    }

    return ($configref);

}

# ANTmt40020 - END 

#ANTmt42349 - start
sub AddVersionAtBeg
{
    my ($refLines,$toVersion)=@_;
    my @Arr1 = @{$refLines};
    substr($toVersion,0,1) = 'A';
        $toVersion = $toVersion . "\n" ;
    unshift(@Arr1, $toVersion);
        return \@Arr1;
}
#ANTmt42349 - end


#R4.1 Start
sub RemoveVersionAtBeg
{
    my ($refLines,$toVersion)=@_;
    my @Arr1 = @{$refLines};
    splice(@Arr1, 0 , 1);
    return \@Arr1;
}
#R4.1 End

sub AddVersionForFGNAtBeg
{
    my ($refLines,$toVersion)=@_;
    my @Arr1 = @{$refLines};
    substr($toVersion,0,1) = 'N';
        $toVersion = $toVersion . "\n" ;
    unshift(@Arr1, $toVersion);
        return \@Arr1;
}



sub FunctionForR4_3
{
    my ($refConfigFile)=@_;
    my @lines         = @{$refConfigFile};
    my $HairpinOID  = "1.3.6.1.4.1.2076.65.1.22.0";
    my $index = 0;
   
    if (($cliversionfrom ne "R3.6") || ($HairpinStatus ne "TRUE"))
    {
    	my $refReturn = \@lines;
    	return $refReturn;
    }
        $msg = "\n R4.3 User-to-User-Hairpin Changes Start ....... ";
    push (@logfile, $msg);
    
    for (; $index <= $#lines; $index++)
    {
        my @fields = split (/\s+/, $lines[$index]);

        if ( $fields[0] =~ m/^$HairpinOID/ )
    	{
            $fields[2] = 1;
            my $Newline = $fields[0]."\t ".$fields[1]."\t ".$fields[2]."\n" ;
            $lines[$index] = $Newline;    
        }
    

    } 
    $msg = "\n R4.3 User-to-User-Hairpin Changes End  ";
    push (@logfile, $msg);

    my $refReturn = \@lines;
    return $refReturn;


}

# ALU01613898 start
#******************************************************************************
# Routine     : CheckVersionForHigherRelease
# Description : This subroutine checks whether the input version no is 
#		greater than R6.0/F4.5. If it is so, it will return R6.0/F4.5 
#		respectively.
# Input       : Version no 
# OutPut      : R6.0 or F4.5 or Input itself
# Orgnisation : Future Software.
#******************************************************************************
sub CheckVersionForHigherRelease
{
    my $version = $_[0];
    chomp($version);

# Check whether the version no is proper
    if ($version =~ m/(R|F)(\d+)(\.)(\d+$)/)
    {
        my $stream = $1;
	my $versionreal = $2;
	my $versiondec = $4;
        if ($stream eq "R")
        {
	  # Bug Id: 13503 - R5.1 Support - Default to R5.1 if the version is higher than R5.1 
          # BUG 14244 - R5.4 Support - Default to R5.4 if the version is higher than R5.4
	  # BUG 14439 - R5.4 Support - Default to R5.5 if the version is higher than R5.5
          # BUG 14620 - R5.6 Support - Default to R5.6 if the version is higher than R5.6
          # BUG 14763 - R5.7 Support - Default to R5.7 if the version is higher than R5.7
          # BUG 14875 - R5.8 Support - Default to R5.8 if the version is higher than R5.8
	  # BUG 14927 - R6.0 Support - Default to R6.0 if the version is higher than R6.0
	  if ((($versionreal == 6) && ($versiondec > 0)) || ($versionreal > 6))
            {
	       # BUG 14244 
		$version = "R6.0";
            }
	  #ALU01818309  
            elsif (($versionreal == 4) && ($versiondec > 6))
            {
	       #ALU01818309 
		$version = "R4.6";
            }
          #ALU01822907 
       	  elsif (($versionreal == 4) && ($versiondec == 4)) 
            {
		$version = "R4.3";
            }
        }
        elsif ($stream eq "F")
        {
# Bug Id: 13724 / 14380 - F4.5 Support - Default to F4.5 if the version is higher than F4.5 
	   #ALU01817313 
		if ((($versionreal == 4) && ($versiondec > 5)) || ($versionreal > 4))
            {
	       #ALU01817313  
		$version = "F4.5";  # BUG13724 #Bug 14380 
            }
        }
    }
    else
    {
# The version no is not proper
        if ($trace == 1)
        {
            $msg = "\nCheckVersionForHigherRelease: Version no. is not proper";
            push (@logfile, $msg);
            push (@dispresult,$msg);
        }
    }
    return $version;
}

# ALU01613898 end

#******************************************************************************
# Routine     : validateMigrationHigherRel 
# Description : This subroutine validates the FROM and TO cli version for
#		releases higher than R6.0/F4.5
#		* If both FROM and TO versions are higher than R6.0/F4.5, Check 
#		  whether TO version is greater than FROM version 
#		  and return 0 or -1 accordingly
#		* Returns 1 if any of FROM or TO version is not in correct format 
# Input       : None 
# OutPut      :  1 - No error(But processing of iss.conf is needed)
#		-1 - Error
#		 0 - No change in iss.conf
# Orgnisation : Future Software.
#******************************************************************************
sub validateMigrationHigherRel
{
    my ($toversion, $fromversion, $toversionOrig, $fromversionOrig) = @_;
    my $fromReal = "";
    my $fromDec = "";
    my $fromStream = "";
    my $toReal = "";
    my $toDec = "";
    my $toStream = "";
    my $result = "";

# if TO version or FROM version is not in correct format we will return
# and the function validateMigration will proceed.
    if ($toversionOrig =~ m/(R|F)(\d+)(\.)(\d+$)/)
    {
	$fromStream = $1;
	$fromReal = $2;
	$fromDec = $4;
    }
    else
    {
	$result = 1;
	return $result;
    }
    if ($fromversionOrig =~ m/(R|F)(\d+)(\.)(\d+$)/)
    {
	$toStream = $1;
	$toReal = $2;
	$toDec = $4;
    }
    else
    {
        $result = 1;
        return $result;
    }

    if (($fromStream ne $toStream) || ($toversionOrig eq $fromversionOrig))
    {
        $result = 1;
        return $result;
    }
    if (($fromReal > $toReal) || (($fromReal == $toReal) && ($fromDec > $toDec)))
    {
        $msg = "\n Error : FROM version greater than TO version\n";
        print $msg;
        push(@logfile,$msg);
        $result = -1;
        return $result;
    }
# For cases like F4.4 ECNTC TO F4.5 ECNTC 
    elsif (($toversion eq $fromversion) && ($climodefrom eq $climodeto))
    {
        $result = 0;
        return $result;
    }	

# For cases like F4.3 ECNTA to F4.4 ECNTC
    $result = 1;
    return $result;
}
